@@ -7,8 +7,7 @@
 
 #include <kio/global.h>
 
-#include <QtCore/QObject>
-#include <QtCore/QString>
+#include <QObject>
 
 class DataProtocolTest : public QObject
 {
