@@ -54,7 +54,7 @@ class JobSpy : public QObject
     Q_OBJECT
 public:
     JobSpy(KIO::Job *job)
-        : QObject(0),
+        : QObject(nullptr),
           m_spy(job, SIGNAL(result(KJob*))),
           m_error(0)
     {
@@ -107,6 +107,8 @@ private Q_SLOTS:
 
         m_srcFile = m_srcDir + "/srcfile";
         m_srcLink = m_srcDir + "/link";
+
+        qRegisterMetaType<KIO::CopyJob*>();
     }
 
     void cleanupTestCase()
@@ -155,7 +157,7 @@ private Q_SLOTS:
         QUrl destUrl = QUrl::fromLocalFile(desktopPath);
         QDropEvent dropEvent(QPoint(10, 10), Qt::CopyAction, &m_mimeData, Qt::LeftButton, Qt::NoModifier);
         KIO::DropJob *job = KIO::drop(&dropEvent, destUrl, KIO::HideProgressInfo);
-        job->setUiDelegate(0);
+        job->setUiDelegate(nullptr);
         QSignalSpy spy(job, SIGNAL(itemCreated(QUrl)));
 
         // Then the application is run with the source file as argument
@@ -212,9 +214,9 @@ private Q_SLOTS:
         const QUrl destUrl = QUrl::fromLocalFile(dest);
         m_mimeData.setUrls(QList<QUrl>() << QUrl::fromLocalFile(srcFile));
         QDropEvent dropEvent(QPoint(10, 10), dropAction, &m_mimeData, Qt::LeftButton, modifiers);
-        KIO::DropJob *job = KIO::drop(&dropEvent, destUrl, KIO::HideProgressInfo);
-        job->setUiDelegate(0);
-        job->setUiDelegateExtension(0);
+        KIO::DropJob *job = KIO::drop(&dropEvent, destUrl, KIO::HideProgressInfo | KIO::NoPrivilegeExecution);
+        job->setUiDelegate(nullptr);
+        job->setUiDelegateExtension(nullptr);
         JobSpy jobSpy(job);
         QSignalSpy copyJobSpy(job, SIGNAL(copyJobStarted(KIO::CopyJob*)));
         QSignalSpy itemCreatedSpy(job, SIGNAL(itemCreated(QUrl)));
@@ -265,7 +267,7 @@ private Q_SLOTS:
         m_mimeData.setUrls(QList<QUrl>() << QUrl::fromLocalFile(srcFile));
         QDropEvent dropEvent(QPoint(10, 10), dropAction, &m_mimeData, Qt::LeftButton, modifiers);
         KIO::DropJob *job = KIO::drop(&dropEvent, QUrl(QStringLiteral("trash:/")), KIO::HideProgressInfo);
-        job->setUiDelegate(0);
+        job->setUiDelegate(nullptr);
         QSignalSpy copyJobSpy(job, SIGNAL(copyJobStarted(KIO::CopyJob*)));
         QSignalSpy itemCreatedSpy(job, SIGNAL(itemCreated(QUrl)));
 
@@ -308,7 +310,7 @@ private Q_SLOTS:
         m_mimeData.setUrls(QList<QUrl>() << trashUrl);
         QDropEvent dropEvent(QPoint(10, 10), Qt::CopyAction, &m_mimeData, Qt::LeftButton, Qt::NoModifier);
         KIO::DropJob *job = KIO::drop(&dropEvent, QUrl::fromLocalFile(m_srcDir), KIO::HideProgressInfo);
-        job->setUiDelegate(0);
+        job->setUiDelegate(nullptr);
         QSignalSpy copyJobSpy(job, SIGNAL(copyJobStarted(KIO::CopyJob*)));
         QSignalSpy spy(job, SIGNAL(itemCreated(QUrl)));
 
@@ -340,7 +342,7 @@ private Q_SLOTS:
         m_mimeData.setUrls(QList<QUrl>() << trashUrl);
         QDropEvent dropEvent(QPoint(10, 10), Qt::CopyAction, &m_mimeData, Qt::LeftButton, Qt::NoModifier);
         KIO::DropJob *job = KIO::drop(&dropEvent, destUrl, KIO::HideProgressInfo);
-        job->setUiDelegate(0);
+        job->setUiDelegate(nullptr);
         QSignalSpy copyJobSpy(job, SIGNAL(copyJobStarted(KIO::CopyJob*)));
         QVERIFY2(job->exec(), qPrintable(job->errorString()));
 
@@ -351,6 +353,31 @@ private Q_SLOTS:
         QVERIFY2(items.contains("trash:" + QChar(0x2044) + ".desktop"), qPrintable(items.join(',')));
     }
 
+    void shouldDropFromTrashToTrash() // #378051
+    {
+        // Given a file in the trash
+        QVERIFY(QFileInfo(m_srcFile).isWritable());
+        KIO::CopyJob *copyJob = KIO::move(QUrl::fromLocalFile(m_srcFile), QUrl(QStringLiteral("trash:/")));
+        QSignalSpy copyingDoneSpy(copyJob, &KIO::CopyJob::copyingDone);
+        QVERIFY(copyJob->exec());
+        const QUrl trashUrl = copyingDoneSpy.at(0).at(2).value<QUrl>();
+        QVERIFY(trashUrl.isValid());
+        QVERIFY(!QFile::exists(m_srcFile));
+
+        // When dropping the trashed file in the trash
+        m_mimeData.setUrls(QList<QUrl>() << trashUrl);
+        QDropEvent dropEvent(QPoint(10, 10), Qt::CopyAction, &m_mimeData, Qt::LeftButton, Qt::NoModifier);
+        KIO::DropJob *job = KIO::drop(&dropEvent, QUrl(QStringLiteral("trash:/")), KIO::HideProgressInfo);
+        job->setUiDelegate(nullptr);
+        QSignalSpy copyJobSpy(job, SIGNAL(copyJobStarted(KIO::CopyJob*)));
+        QSignalSpy spy(job, SIGNAL(itemCreated(QUrl)));
+
+        // Then an error should be reported and no files action should occur
+        QVERIFY(!job->exec());
+        QCOMPARE(job->error(), KIO::ERR_DROP_ON_ITSELF);
+    }
+
+
     void shouldDropToDirectoryWithPopup_data()
     {
         QTest::addColumn<QString>("dest"); // empty for a temp dir
@@ -390,8 +417,8 @@ private Q_SLOTS:
         QUrl destUrl = QUrl::fromLocalFile(dest);
         QDropEvent dropEvent(QPoint(10, 10), Qt::CopyAction /*unused*/, &m_mimeData, Qt::LeftButton, Qt::NoModifier);
         KIO::DropJob *job = KIO::drop(&dropEvent, destUrl, KIO::HideProgressInfo);
-        job->setUiDelegate(0);
-        job->setUiDelegateExtension(0); // no rename dialog
+        job->setUiDelegate(nullptr);
+        job->setUiDelegateExtension(nullptr); // no rename dialog
         JobSpy jobSpy(job);
         qRegisterMetaType<KFileItemListProperties>();
         QSignalSpy spyShow(job, SIGNAL(popupMenuAboutToShow(KFileItemListProperties)));
@@ -439,7 +466,7 @@ private Q_SLOTS:
         QAction appAction1(QStringLiteral("action1"), this);
         QAction appAction2(QStringLiteral("action2"), this);
         QList<QAction *> appActions; appActions << &appAction1 << &appAction2;
-        job->setUiDelegate(0);
+        job->setUiDelegate(nullptr);
         job->setApplicationActions(appActions);
         JobSpy jobSpy(job);
 
@@ -466,17 +493,19 @@ private Q_SLOTS:
 private:
     static QMenu *findPopup()
     {
-        Q_FOREACH (QWidget *widget, qApp->topLevelWidgets()) {
+        const QList<QWidget *> widgetsList = qApp->topLevelWidgets();
+        for (QWidget *widget : widgetsList) {
             if (QMenu *menu = qobject_cast<QMenu *>(widget)) {
                 return menu;
             }
         }
-        return Q_NULLPTR;
+        return nullptr;
     }
     static Qt::DropActions popupDropActions(QMenu *menu)
     {
         Qt::DropActions actions;
-        Q_FOREACH (QAction *action, menu->actions()) {
+        const QList<QAction *> actionsList = menu->actions();
+        for (const QAction *action : actionsList) {
             const QVariant userData = action->data();
             if (userData.isValid()) {
                 actions |= userData.value<Qt::DropAction>();
