@@ -22,15 +22,16 @@
 #include "kfilewidget.h"
 
 #include <QLabel>
+#include <QTemporaryDir>
 #include <QTest>
 #include <QStandardPaths>
+#include <QSignalSpy>
 
 #include <kdiroperator.h>
 #include <KFileFilterCombo>
 #include <klocalizedstring.h>
 #include <kurlnavigator.h>
 #include <KUrlComboBox>
-#include <KFileFilterCombo>
 #include "kiotesthelper.h" // createTestFile
 
 #include <QAbstractItemView>
@@ -476,6 +477,9 @@ private Q_SLOTS:
         QMimeData *mimeData = new QMimeData();
         mimeData->setUrls(QList<QUrl>() << fileUrl);
 
+        KDirLister *dirLister = fileWidget.dirOperator()->dirLister();
+        QSignalSpy spy(dirLister, SIGNAL(completed(const QUrl &_url)));
+
         QDragEnterEvent event1(QPoint(), Qt::DropAction::MoveAction, mimeData, Qt::MouseButton::LeftButton, Qt::KeyboardModifier::NoModifier);
 
         QVERIFY(qApp->sendEvent(fileWidget.dirOperator()->view()->viewport(), &event1));
@@ -488,8 +492,8 @@ private Q_SLOTS:
         // QVERIFY(QTest::qWaitForWindowActive(&fileWidget));
 
         // once we drop a file the dirlister scans the dir
-        // wait a bit to the dirlister time to finish
-        QTest::qWait(100);
+        // wait for the completed signal from the dirlister
+        spy.wait();
 
         // Verify the expected populated name.
         QCOMPARE(fileWidget.baseUrl().adjusted(QUrl::StripTrailingSlash), dirUrl);
@@ -504,6 +508,28 @@ private Q_SLOTS:
         QCOMPARE(urls.size(), 1);
         QCOMPARE(urls[0], fileUrl);
     }
+
+    void testCreateNestedNewFolders()
+    {
+        // when creating multiple nested new folders in the "save as" dialog, where folders are
+        //created and entered, kdirlister would hit an assert (in reinsert()), bug 408801
+        QTemporaryDir tempDir;
+        QVERIFY(tempDir.isValid());
+        const QString dir = tempDir.path();
+        const QUrl url = QUrl::fromLocalFile(dir);
+        KFileWidget fw(url);
+        fw.setOperationMode(KFileWidget::Saving);
+        fw.setMode(KFile::File);
+
+        // create the nested folders
+        for (int i = 1; i < 6; ++i) {
+            fw.dirOperator()->mkdir(QStringLiteral("folder%1").arg(i), true);
+            // simulate the time the user will take to type the new folder name
+            QTest::qWait(1000);
+        }
+
+        QVERIFY(QFile::exists(dir + QStringLiteral("/folder1/folder2/folder3/folder4/folder5")));
+    }
 };
 
 QTEST_MAIN(KFileWidgetTest)
