@@ -18,7 +18,7 @@
     Boston, MA 02110-1301, USA.
 */
 
-#include <QtTest/QtTest>
+#include <QtTest>
 
 #include <QDialog>
 #include <QLineEdit>
@@ -60,11 +60,11 @@ private Q_SLOTS:
     void storedPutIODeviceQrcFile()
     {
         // Given a source (in a Qt resource file) and a destination file
-        const QString src = ":/kio5/newfile-templates/.source/HTMLFile.html";
+        const QString src = QStringLiteral(":/kio5/newfile-templates/.source/HTMLFile.html");
         QVERIFY(QFile::exists(src));
         QFile srcFile(src);
         QVERIFY(srcFile.open(QIODevice::ReadOnly));
-        const QString dest = m_tmpDir.path() + "/dest";
+        const QString dest = m_tmpDir.path() + QStringLiteral("/dest");
         QFile::remove(dest);
         const QUrl destUrl = QUrl::fromLocalFile(dest);
 
@@ -100,10 +100,11 @@ private Q_SLOTS:
         QTest::newRow("url desktop file") << "Link to Location " << "" << "tmp_link.desktop" << "tmp_link.desktop";
         QTest::newRow("url desktop file no extension") << "Link to Location " << "" << "tmp_link" << "tmp_link";
         QTest::newRow("url desktop file .pl extension") << "Link to Location " << "" << "tmp_link.pl" << "tmp_link.pl.desktop";
-        QTest::newRow("symlink") << "Basic link" << "" << "thelink" << "thelink";
+        QTest::newRow("symlink") << "Basic Link" << "" << "thelink" << "thelink";
         QTest::newRow("folder") << "Folder..." << "New Folder" << "folder1" << "folder1";
         QTest::newRow("folder_default_name") << "Folder..." << "New Folder" << "New Folder" << "New Folder";
-        QTest::newRow("folder_with_suggested_name") << "Folder..." << "New Folder (1)" << "New Folder" << "New Folder";
+        QTest::newRow("folder_with_suggested_name") << "Folder..." << "New Folder (1)" << "New Folder (1)" << "New Folder (1)";
+        QTest::newRow("folder_with_suggested_name_but_user_overrides") << "Folder..." << "New Folder (2)" << "New Folder" << "";
         QTest::newRow("application") << "Link to Application..." << "Link to Application" << "app1" << "app1.desktop";
     }
 
@@ -125,17 +126,18 @@ private Q_SLOTS:
         menu.checkUpToDate();
         QAction *action = coll.action(QStringLiteral("the_action"));
         QVERIFY(action);
-        QAction *textAct = 0;
-        Q_FOREACH (QAction *act, action->menu()->actions()) {
+        QAction *textAct = nullptr;
+        const QList<QAction *> actionsList = action->menu()->actions();
+        for (QAction *act : actionsList) {
             if (act->text().contains(actionText)) {
                 textAct = act;
             }
         }
         if (!textAct) {
-            Q_FOREACH (QAction *act, action->menu()->actions()) {
+            for (const QAction *act : actionsList) {
                 qDebug() << act << act->text() << act->data();
             }
-            const QString err = "action with text \"" + actionText + "\" not found.";
+            const QString err = QLatin1String("action with text \"") + actionText + QLatin1String("\" not found.");
             QVERIFY2(textAct, qPrintable(err));
         }
         textAct->trigger();
@@ -146,7 +148,7 @@ private Q_SLOTS:
             nauiDialog->setSuggestedName(typedFilename);
             nauiDialog->setSuggestedUrl(QUrl(QStringLiteral("file:///etc")));
         } else if (KPropertiesDialog *propsDialog = qobject_cast<KPropertiesDialog *>(dialog)) {
-            QLineEdit *lineEdit = propsDialog->findChild<QLineEdit *>("KFilePropsPlugin::nameLineEdit");
+            QLineEdit *lineEdit = propsDialog->findChild<QLineEdit *>(QStringLiteral("KFilePropsPlugin::nameLineEdit"));
             QVERIFY(lineEdit);
             QCOMPARE(lineEdit->text(), expectedDefaultFilename);
             lineEdit->setText(typedFilename);
@@ -160,22 +162,29 @@ private Q_SLOTS:
         QSignalSpy spy(&menu, SIGNAL(fileCreated(QUrl)));
         QSignalSpy folderSpy(&menu, SIGNAL(directoryCreated(QUrl)));
         dialog->accept();
-        const QString path = m_tmpDir.path() + '/' + expectedFilename;
+        QString path = m_tmpDir.path() + QLatin1Char('/') + expectedFilename;
         if (actionText == QLatin1String("Folder...")) {
-            QVERIFY(folderSpy.wait(1000));
-            emittedUrl = folderSpy.at(0).at(0).toUrl();
-            QVERIFY(QFileInfo(path).isDir());
+            if (expectedFilename.isEmpty()) {
+                // This is the "Folder already exists" case; expect an error dialog
+                dialog = parentWidget.findChild<QDialog *>();
+                dialog->accept();
+                path.clear();
+            } else {
+                QVERIFY(folderSpy.wait(1000));
+                emittedUrl = folderSpy.at(0).at(0).toUrl();
+                QVERIFY(QFileInfo(path).isDir());
+            }
         } else {
             if (spy.isEmpty()) {
                 QVERIFY(spy.wait(1000));
             }
             emittedUrl = spy.at(0).at(0).toUrl();
             QVERIFY(QFile::exists(path));
-            if (actionText != QLatin1String("Basic link")) {
+            if (actionText != QLatin1String("Basic Link")) {
                 QFile file(path);
                 QVERIFY(file.open(QIODevice::ReadOnly));
                 const QByteArray contents = file.readAll();
-                if (actionText.startsWith("HTML")) {
+                if (actionText.startsWith(QLatin1String("HTML"))) {
                     QCOMPARE(QString::fromLatin1(contents.left(6)), QStringLiteral("<html>"));
                 }
             }
