@@ -25,7 +25,7 @@
 
 #include <QtTestWidgets>
 #include <QDir>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QHostInfo>
 
 #include <iostream>
@@ -84,15 +84,15 @@ static void runFilterTest(const QString &a, const QString &expectedResult = null
         case KUriFilterData::LocalFile:
         case KUriFilterData::LocalDir:
             qDebug() << "*** Result: Local Resource =>  '"
-                     << filterData->uri().toLocalFile() << "'" << endl;
+                     << filterData->uri().toLocalFile() << "'";
             break;
         case KUriFilterData::Help:
             qDebug() << "*** Result: Local Resource =>  '"
-                     << filterData->uri().url() << "'" << endl;
+                     << filterData->uri().url() << "'";
             break;
         case KUriFilterData::NetProtocol:
             qDebug() << "*** Result: Network Resource => '"
-                     << filterData->uri().url() << "'" << endl;
+                     << filterData->uri().url() << "'";
             break;
         case KUriFilterData::Shell:
         case KUriFilterData::Executable:
@@ -111,7 +111,7 @@ static void runFilterTest(const QString &a, const QString &expectedResult = null
 
         if (!expectedResult.isEmpty()) {
             // Hack for other locales than english, normalize google hosts to google.com
-            cmd = cmd.replace(QRegExp(QStringLiteral("www\\.google\\.[^/]*/")), QStringLiteral("www.google.com/"));
+            cmd.replace(QRegularExpression(QStringLiteral("www\\.google\\.[^/]*/")), QStringLiteral("www.google.com/"));
             if (cmd != expectedResult) {
                 qWarning() << a;
                 QCOMPARE(cmd, expectedResult);
@@ -233,7 +233,12 @@ void KUriFilterTest::noFiltering_data()
     setupColumns();
     // URI that should require no filtering
     addRow("http://www.kde.org", QStringLiteral("http://www.kde.org"), KUriFilterData::NetProtocol);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 12, 1)
+    // qtbase commit eaf4438b3511c preserves the double slashes
     addRow("http://www.kde.org/developer//index.html", QStringLiteral("http://www.kde.org/developer//index.html"), KUriFilterData::NetProtocol);
+#else
+    addRow("http://www.kde.org/developer//index.html", QStringLiteral("http://www.kde.org/developer/index.html"), KUriFilterData::NetProtocol);
+#endif
     addRow("file:///", QStringLiteral("/"), KUriFilterData::LocalDir);
     addRow("file:///etc", QStringLiteral("/etc"), KUriFilterData::LocalDir);
     addRow("file:///etc/passwd", QStringLiteral("/etc/passwd"), KUriFilterData::LocalFile);
@@ -253,6 +258,9 @@ void KUriFilterTest::localFiles_data()
     addRow("///", QStringLiteral("/"), KUriFilterData::LocalDir);
     addRow("////", QStringLiteral("/"), KUriFilterData::LocalDir);
     addRow("///tmp", QStringLiteral("/tmp"), KUriFilterData::LocalDir);
+    addRow("///tmp/", QStringLiteral("/tmp/"), KUriFilterData::LocalDir);
+    addRow("///tmp//", QStringLiteral("/tmp/"), KUriFilterData::LocalDir);
+    addRow("///tmp///", QStringLiteral("/tmp/"), KUriFilterData::LocalDir);
 
     if (QFile::exists(QDir::homePath() + QLatin1String("/.bashrc"))) {
         addRow("~/.bashrc", QDir::homePath() + QStringLiteral("/.bashrc"), KUriFilterData::LocalFile, QStringList(QStringLiteral("kshorturifilter")));
@@ -303,8 +311,16 @@ void KUriFilterTest::shortUris_data()
 {
     setupColumns();
     // hostnames are lowercased by QUrl
+#if QT_VERSION >= QT_VERSION_CHECK(5, 12, 1)
+    // qtbase commit eaf4438b3511c preserves the double slashes
     addRow("http://www.myDomain.commyPort/ViewObjectRes//Default:name=hello",
            QStringLiteral("http://www.mydomain.commyport/ViewObjectRes//Default:name=hello"), KUriFilterData::NetProtocol);
+#else
+    addRow("http://www.myDomain.commyPort/ViewObjectRes//Default:name=hello",
+           QStringLiteral("http://www.mydomain.commyport/ViewObjectRes/Default:name=hello"), KUriFilterData::NetProtocol);
+#endif
+    addRow("http://www.myDomain.commyPort/ViewObjectRes/Default:name=hello?a=a///////",
+           QStringLiteral("http://www.mydomain.commyport/ViewObjectRes/Default:name=hello?a=a///////"), KUriFilterData::NetProtocol);
     addRow("ftp://ftp.kde.org", QStringLiteral("ftp://ftp.kde.org"), KUriFilterData::NetProtocol);
     addRow("ftp://username@ftp.kde.org:500", QStringLiteral("ftp://username@ftp.kde.org:500"), KUriFilterData::NetProtocol);
 
@@ -451,14 +467,13 @@ void KUriFilterTest::environmentVariables()
 void KUriFilterTest::internetKeywords_data()
 {
     setupColumns();
-    QString sc;
-    addRow(sc.sprintf("gg%cfoo bar", s_delimiter).toUtf8(), QStringLiteral("https://www.google.com/search?q=foo+bar&ie=UTF-8"), KUriFilterData::NetProtocol);
-    addRow(sc.sprintf("bug%c55798", s_delimiter).toUtf8(), QStringLiteral("https://bugs.kde.org/show_bug.cgi?id=55798"), KUriFilterData::NetProtocol);
+    addRow(QString::asprintf("gg%cfoo bar", s_delimiter).toUtf8(), QStringLiteral("https://www.google.com/search?q=foo+bar&ie=UTF-8"), KUriFilterData::NetProtocol);
+    addRow(QString::asprintf("bug%c55798", s_delimiter).toUtf8(), QStringLiteral("https://bugs.kde.org/show_bug.cgi?id=55798"), KUriFilterData::NetProtocol);
 
-    addRow(sc.sprintf("gg%cC++", s_delimiter).toUtf8(), QStringLiteral("https://www.google.com/search?q=C%2B%2B&ie=UTF-8"), KUriFilterData::NetProtocol);
-    addRow(sc.sprintf("gg%cC#", s_delimiter).toUtf8(), QStringLiteral("https://www.google.com/search?q=C%23&ie=UTF-8"), KUriFilterData::NetProtocol);
-    addRow(sc.sprintf("ya%cfoo bar was here", s_delimiter).toUtf8(), nullptr, -1);   // this triggers default search, i.e. google
-    addRow(sc.sprintf("gg%cwww.kde.org", s_delimiter).toUtf8(), QStringLiteral("https://www.google.com/search?q=www.kde.org&ie=UTF-8"), KUriFilterData::NetProtocol);
+    addRow(QString::asprintf("gg%cC++", s_delimiter).toUtf8(), QStringLiteral("https://www.google.com/search?q=C%2B%2B&ie=UTF-8"), KUriFilterData::NetProtocol);
+    addRow(QString::asprintf("gg%cC#", s_delimiter).toUtf8(), QStringLiteral("https://www.google.com/search?q=C%23&ie=UTF-8"), KUriFilterData::NetProtocol);
+    addRow(QString::asprintf("ya%cfoo bar was here", s_delimiter).toUtf8(), nullptr, -1);   // this triggers default search, i.e. google
+    addRow(QString::asprintf("gg%cwww.kde.org", s_delimiter).toUtf8(), QStringLiteral("https://www.google.com/search?q=www.kde.org&ie=UTF-8"), KUriFilterData::NetProtocol);
     addRow(QStringLiteral("gg%1é").arg(s_delimiter).toUtf8() /*eaccent in utf8*/, QStringLiteral("https://www.google.com/search?q=%C3%A9&ie=UTF-8"), KUriFilterData::NetProtocol);
     addRow(QStringLiteral("gg%1прйвет").arg(s_delimiter).toUtf8() /* greetings in russian utf-8*/, QStringLiteral("https://www.google.com/search?q=%D0%BF%D1%80%D0%B9%D0%B2%D0%B5%D1%82&ie=UTF-8"), KUriFilterData::NetProtocol);
 }
