@@ -54,14 +54,15 @@
 #include <QTimer>
 #include <QFile>
 #include <QFileInfo>
-#include <QLinkedList>
 #include <sys/stat.h> // mode_t
 #include <QPointer>
 
 #include "job_p.h"
 #include <kdiskfreespaceinfo.h>
 #include <kfilesystemtype.h>
+#include <kfileutils.h>
 
+#include <list>
 
 #include <QLoggingCategory>
 Q_DECLARE_LOGGING_CATEGORY(KIO_COPYJOB_DEBUG)
@@ -72,6 +73,14 @@ using namespace KIO;
 //this will update the report dialog with 5 Hz, I think this is fast enough, aleXXX
 #define REPORT_TIMEOUT 200
 
+#if !defined(NAME_MAX)
+    #if defined(_MAX_FNAME)
+        #define NAME_MAX _MAX_FNAME //For Windows
+    #else
+        #define NAME_MAX 0
+    #endif
+#endif
+
 enum DestinationState {
     DEST_NOT_STATED,
     DEST_IS_DIR,
@@ -166,8 +175,8 @@ public:
     bool m_bURLDirty;
     // Used after copying all the files into the dirs, to set mtime (TODO: and permissions?)
     // after the copy is done
-    QLinkedList<CopyInfo> m_directoriesCopied;
-    QLinkedList<CopyInfo>::const_iterator m_directoriesCopiedIterator;
+    std::list<CopyInfo> m_directoriesCopied;
+    std::list<CopyInfo>::const_iterator m_directoriesCopiedIterator;
 
     CopyJob::CopyMode m_mode;
     bool m_asMethod;
@@ -240,7 +249,7 @@ public:
     bool shouldOverwriteFile(const QString &path) const;
     bool shouldSkip(const QString &path) const;
     void skipSrc(bool isDir);
-    void renameDirectory(QList<CopyInfo>::iterator it, const QUrl &newUrl);
+    void renameDirectory(const QList<CopyInfo>::iterator &it, const QUrl &newUrl);
     QUrl finalDestUrl(const QUrl &src, const QUrl &dest) const;
 
     void slotStart();
@@ -296,8 +305,11 @@ public:
 CopyJob::CopyJob(CopyJobPrivate &dd)
     : Job(dd)
 {
+    Q_D(CopyJob);
     setProperty("destUrl", d_func()->m_dest.toString());
-    QTimer::singleShot(0, this, SLOT(slotStart()));
+    QTimer::singleShot(0, this, [d]() {
+        d->slotStart();
+    });
     qRegisterMetaType<KIO::UDSEntry>();
 }
 
@@ -322,7 +334,7 @@ void CopyJobPrivate::slotStart()
         return;
     }
     if (m_mode == CopyJob::CopyMode::Move) {
-        Q_FOREACH (const QUrl &url, m_srcList) {
+        for (const QUrl &url : qAsConst(m_srcList)) {
             if (m_dest.scheme() == url.scheme() && m_dest.host() == url.host()) {
                 QString srcPath = url.path();
                 if (!srcPath.endsWith(QLatin1Char('/')))
@@ -342,7 +354,9 @@ void CopyJobPrivate::slotStart()
     */
     m_reportTimer = new QTimer(q);
 
-    q->connect(m_reportTimer, SIGNAL(timeout()), q, SLOT(slotReport()));
+    q->connect(m_reportTimer, &QTimer::timeout, q, [this]() {
+        slotReport();
+    });
     m_reportTimer->start(REPORT_TIMEOUT);
 
     // Stat the dest
@@ -377,7 +391,7 @@ void CopyJobPrivate::slotResultStating(KJob *job)
             info.uDest = m_dest;
             // Append filename or dirname to destination URL, if allowed
             if (destinationState == DEST_IS_DIR && !m_asMethod) {
-                const QString fileName = srcurl.scheme() == "data" ? "data" : srcurl.fileName(); // #379093
+                const QString fileName = srcurl.scheme() == QLatin1String("data") ? QStringLiteral("data") : srcurl.fileName(); // #379093
                 info.uDest = addPathToUrl(info.uDest, fileName);
             }
 
@@ -403,6 +417,7 @@ void CopyJobPrivate::slotResultStating(KJob *job)
                 // copying. The target file or directory does not exist yet, which
                 // might confuse KDiskFreeSpaceInfo.
                 path = fileInfo.absolutePath();
+                fileInfo.setFile(path);
             }
             KDiskFreeSpaceInfo freeSpaceInfo = KDiskFreeSpaceInfo::freeSpaceInfo(path);
             if (freeSpaceInfo.isValid()) {
@@ -412,6 +427,16 @@ void CopyJobPrivate::slotResultStating(KJob *job)
              }
             //TODO actually preliminary check is even more valuable for slow NFS/SMB mounts,
             //but we need to find a way to report connection errors to user
+
+            // Also check for writability, before spending time stat'ing everything (#141564)
+            // TODO: this is done only for local files, but we could use the UDSEntry to do this portably
+            // ... assuming all kioslaves set permissions correctly
+            if (!m_privilegeExecutionEnabled && fileInfo.exists() && !fileInfo.isWritable()) {
+                q->setError(ERR_WRITE_ACCESS_DENIED);
+                q->setErrorText(path);
+                q->emitResult();
+                return;
+            }
         }
 
         const bool isGlobalDest = m_dest == m_globalDest;
@@ -555,7 +580,9 @@ bool CopyJob::doResume()
     Q_D(CopyJob);
     switch (d->state) {
         case STATE_INITIAL:
-            QTimer::singleShot(0, this, SLOT(slotStart()));
+            QTimer::singleShot(0, this, [d]() {
+                d->slotStart();
+            });
             break;
         default:
             // not implemented
@@ -578,6 +605,7 @@ void CopyJobPrivate::slotReport()
         Q_FALLTHROUGH();
     case STATE_COPYING_FILES:
         q->setProcessedAmount(KJob::Files, m_processedFiles);
+        q->setProcessedAmount(KJob::Bytes, m_processedSize + m_fileProcessedSize);
         if (m_bURLDirty) {
             // Only emit urls when they changed. This saves time, and fixes #66281
             m_bURLDirty = false;
@@ -636,7 +664,7 @@ void CopyJobPrivate::slotEntries(KIO::Job *job, const UDSEntryList &list)
 
 void CopyJobPrivate::slotSubError(ListJob *job, ListJob *subJob)
 {
-    const QUrl url = subJob->url();
+    const QUrl &url = subJob->url();
     qCWarning(KIO_CORE) << url << subJob->errorString();
 
     Q_Q(CopyJob);
@@ -649,8 +677,11 @@ void CopyJobPrivate::addCopyInfoFromUDSEntry(const UDSEntry &entry, const QUrl &
 {
     struct CopyInfo info;
     info.permissions = entry.numberValue(KIO::UDSEntry::UDS_ACCESS, -1);
-    info.mtime = QDateTime::fromMSecsSinceEpoch(1000 * entry.numberValue(KIO::UDSEntry::UDS_MODIFICATION_TIME, -1));
-    info.ctime = QDateTime::fromMSecsSinceEpoch(1000 * entry.numberValue(KIO::UDSEntry::UDS_CREATION_TIME, -1));
+    const auto timeVal = entry.numberValue(KIO::UDSEntry::UDS_MODIFICATION_TIME, -1);
+    if (timeVal != -1) {
+        info.mtime = QDateTime::fromMSecsSinceEpoch(1000 * timeVal, Qt::UTC);
+    }
+    info.ctime = QDateTime::fromMSecsSinceEpoch(1000 * entry.numberValue(KIO::UDSEntry::UDS_CREATION_TIME, -1), Qt::UTC);
     info.size = static_cast<KIO::filesize_t>(entry.numberValue(KIO::UDSEntry::UDS_SIZE, -1));
 
     if (info.size != (KIO::filesize_t) - 1) {
@@ -697,11 +728,11 @@ void CopyJobPrivate::addCopyInfoFromUDSEntry(const UDSEntry &entry, const QUrl &
                     fnu == KProtocolInfo::FromUrl) {
                 //destFileName = url.fileName(); // Doesn't work for recursive listing
                 // Count the number of prefixes used by the recursive listjob
-                int numberOfSlashes = fileName.count('/'); // don't make this a find()!
+                int numberOfSlashes = fileName.count(QLatin1Char('/')); // don't make this a find()!
                 QString path = url.path();
                 int pos = 0;
                 for (int n = 0; n < numberOfSlashes + 1; ++n) {
-                    pos = path.lastIndexOf('/', pos - 1);
+                    pos = path.lastIndexOf(QLatin1Char('/'), pos - 1);
                     if (pos == -1) { // error
                         qCWarning(KIO_CORE) << "kioslave bug: not enough slashes in UDS_URL" << path << "- looking for" << numberOfSlashes << "slashes";
                         break;
@@ -747,7 +778,7 @@ QUrl CopyJobPrivate::finalDestUrl(const QUrl& src, const QUrl &dest) const
     Q_Q(const CopyJob);
     if (dest.scheme() == QLatin1String("trash")) {
         const QMap<QString, QString>& metaData = q->metaData();
-        QMap<QString, QString>::ConstIterator it = metaData.find("trashURL-" + src.path());
+        QMap<QString, QString>::ConstIterator it = metaData.find(QLatin1String("trashURL-") + src.path());
         if (it != metaData.constEnd()) {
             qCDebug(KIO_COPYJOB_DEBUG) << "finalDestUrl=" << it.value();
             return QUrl(it.value());
@@ -806,7 +837,13 @@ void CopyJobPrivate::statCurrentSrc()
                     // Different protocols, we'll create a .desktop file
                     // We have to change the extension anyway, so while we're at it,
                     // name the file like the URL
-                    info.uDest = addPathToUrl(info.uDest, KIO::encodeFileName(m_currentSrcURL.toDisplayString()) + ".desktop");
+                    QByteArray encodedFilename = QFile::encodeName(m_currentSrcURL.toDisplayString());
+                    const int truncatePos = NAME_MAX - (info.uDest.toDisplayString().length() + 8); // length(.desktop) = 8
+                    if (truncatePos > 0) {
+                        encodedFilename.truncate(truncatePos);
+                    }
+                    const QString decodedFilename = QFile::decodeName(encodedFilename);
+                    info.uDest = addPathToUrl(info.uDest, KIO::encodeFileName(decodedFilename) + QLatin1String(".desktop"));
                 }
             }
             files.append(info);   // Files and any symlinks
@@ -848,7 +885,7 @@ void CopyJobPrivate::statCurrentSrc()
             }
         }
 
-        // if the file system doesn't support deleting, we do not even stat
+        // if the source file system doesn't support deleting, we do not even stat
         if (m_mode == CopyJob::Move && !KProtocolManager::supportsDeleting(m_currentSrcURL)) {
             QPointer<CopyJob> that = q;
             emit q->warning(q, buildErrorString(ERR_CANNOT_DELETE, m_currentSrcURL.toDisplayString()));
@@ -954,10 +991,12 @@ void CopyJobPrivate::startListing(const QUrl &src)
     m_bURLDirty = true;
     ListJob *newjob = listRecursive(src, KIO::HideProgressInfo);
     newjob->setUnrestricted(true);
-    q->connect(newjob, SIGNAL(entries(KIO::Job*,KIO::UDSEntryList)),
-               SLOT(slotEntries(KIO::Job*,KIO::UDSEntryList)));
-    q->connect(newjob, SIGNAL(subError(KIO::ListJob*,KIO::ListJob*)),
-               SLOT(slotSubError(KIO::ListJob*,KIO::ListJob*)));
+    q->connect(newjob, &ListJob::entries, q, [this](KIO::Job *job, const KIO::UDSEntryList &list) {
+        slotEntries(job, list);
+    });
+    q->connect(newjob, &ListJob::subError, q, [this](KIO::ListJob *job, KIO::ListJob *subJob) {
+        slotSubError(job, subJob);
+    });
     q->addSubjob(newjob);
 }
 
@@ -993,7 +1032,7 @@ bool CopyJobPrivate::shouldOverwriteFile(const QString &path) const
 
 bool CopyJobPrivate::shouldSkip(const QString &path) const
 {
-    Q_FOREACH (const QString &skipPath, m_skipList) {
+    for (const QString &skipPath : qAsConst(m_skipList)) {
         if (path.startsWith(skipPath)) {
             return true;
         }
@@ -1001,22 +1040,22 @@ bool CopyJobPrivate::shouldSkip(const QString &path) const
     return false;
 }
 
-void CopyJobPrivate::renameDirectory(QList<CopyInfo>::iterator it, const QUrl &newUrl)
+void CopyJobPrivate::renameDirectory(const QList<CopyInfo>::iterator &it, const QUrl &newUrl)
 {
     Q_Q(CopyJob);
     emit q->renamed(q, (*it).uDest, newUrl); // for e.g. KPropertiesDialog
 
     QString oldPath = (*it).uDest.path();
-    if (!oldPath.endsWith('/')) {
-        oldPath += '/';
+    if (!oldPath.endsWith(QLatin1Char('/'))) {
+        oldPath += QLatin1Char('/');
     }
 
     // Change the current one and strip the trailing '/'
     (*it).uDest = newUrl.adjusted(QUrl::StripTrailingSlash);
 
     QString newPath = newUrl.path(); // With trailing slash
-    if (!newPath.endsWith('/')) {
-        newPath += '/';
+    if (!newPath.endsWith(QLatin1Char('/'))) {
+        newPath += QLatin1Char('/');
     }
     QList<CopyInfo>::Iterator renamedirit = it;
     ++renamedirit;
@@ -1068,8 +1107,8 @@ void CopyJobPrivate::slotResultCreatingDirs(KJob *job)
             if (m_bAutoSkipDirs) {
                 // We don't want to copy files in this directory, so we put it on the skip list
                 QString path = oldURL.path();
-                if (!path.endsWith('/')) {
-                    path += '/';
+                if (!path.endsWith(QLatin1Char('/'))) {
+                    path += QLatin1Char('/');
                 }
                 m_skipList.append(path);
                 skip(oldURL, true);
@@ -1083,7 +1122,7 @@ void CopyJobPrivate::slotResultCreatingDirs(KJob *job)
                 } else {
                     if (m_bAutoRenameDirs) {
                         const QUrl destDirectory = (*it).uDest.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash);
-                        const QString newName = KIO::suggestName(destDirectory, (*it).uDest.fileName());
+                        const QString newName = KFileUtils::suggestName(destDirectory, (*it).uDest.fileName());
                         QUrl newUrl(destDirectory);
                         newUrl.setPath(concatPaths(newUrl.path(), newName));
                         renameDirectory(it, newUrl);
@@ -1116,7 +1155,7 @@ void CopyJobPrivate::slotResultCreatingDirs(KJob *job)
     } else { // no error : remove from list, to move on to next dir
         //this is required for the undo feature
         emit q->copyingDone(q, (*it).uSource, finalDestUrl((*it).uSource, (*it).uDest), (*it).mtime, true, false);
-        m_directoriesCopied.append(*it);
+        m_directoriesCopied.push_back(*it);
         dirs.erase(it);
     }
 
@@ -1137,10 +1176,7 @@ void CopyJobPrivate::slotResultConflictCreatingDirs(KJob *job)
 
     const UDSEntry entry = ((KIO::StatJob *)job)->statResult();
 
-    // Its modification time:
-    const QDateTime destmtime = QDateTime::fromMSecsSinceEpoch(1000 * entry.numberValue(KIO::UDSEntry::UDS_MODIFICATION_TIME, -1));
-    const QDateTime destctime = QDateTime::fromMSecsSinceEpoch(1000 * entry.numberValue(KIO::UDSEntry::UDS_CREATION_TIME, -1));
-
+    QDateTime destmtime, destctime;
     const KIO::filesize_t destsize = entry.numberValue(KIO::UDSEntry::UDS_SIZE);
     const QString linkDest = entry.stringValue(KIO::UDSEntry::UDS_LINK_DEST);
 
@@ -1156,7 +1192,9 @@ void CopyJobPrivate::slotResultConflictCreatingDirs(KJob *job)
                  (*it).uSource.adjusted(QUrl::StripTrailingSlash).path() == linkDest)) {
             options |= RenameDialog_OverwriteItself;
         } else {
-            options |= RenameDialog_Overwrite;
+                options |= RenameDialog_Overwrite;
+                destmtime = QDateTime::fromMSecsSinceEpoch(1000 * entry.numberValue(KIO::UDSEntry::UDS_MODIFICATION_TIME, -1), Qt::UTC);
+                destctime = QDateTime::fromMSecsSinceEpoch(1000 * entry.numberValue(KIO::UDSEntry::UDS_CREATION_TIME, -1), Qt::UTC);
         }
     }
 
@@ -1183,6 +1221,7 @@ void CopyJobPrivate::slotResultConflictCreatingDirs(KJob *job)
     case Result_AutoRename:
         m_bAutoRenameDirs = true;
     // fall through
+        Q_FALLTHROUGH();
     case Result_Rename: {
         QUrl newUrl((*it).uDest);
         newUrl.setPath(newPath, QUrl::DecodedMode);
@@ -1193,6 +1232,7 @@ void CopyJobPrivate::slotResultConflictCreatingDirs(KJob *job)
     case Result_AutoSkip:
         m_bAutoSkipDirs = true;
     // fall through
+        Q_FALLTHROUGH();
     case Result_Skip:
         m_skipList.append(existingDest);
         skip((*it).uSource, true);
@@ -1292,7 +1332,7 @@ void CopyJobPrivate::slotResultCopyingFiles(KJob *job)
                     || (m_conflictError == ERR_IDENTICAL_FILES)) {
                 if (m_bAutoRenameFiles) {
                     QUrl destDirectory = (*it).uDest.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash);
-                    const QString newName = KIO::suggestName(destDirectory, (*it).uDest.fileName());
+                    const QString newName = KFileUtils::suggestName(destDirectory, (*it).uDest.fileName());
                     QUrl newDest(destDirectory);
                     newDest.setPath(concatPaths(newDest.path(), newName));
                     emit q->renamed(q, (*it).uDest, newDest); // for e.g. kpropsdlg
@@ -1381,7 +1421,7 @@ void CopyJobPrivate::slotResultCopyingFiles(KJob *job)
     qCDebug(KIO_COPYJOB_DEBUG) << files.count() << "files remaining";
 
     // Merge metadata from subjob
-    KIO::Job *kiojob = dynamic_cast<KIO::Job *>(job);
+    KIO::Job *kiojob = qobject_cast<KIO::Job *>(job);
     Q_ASSERT(kiojob);
     m_incomingMetaData += kiojob->metaData();
     q->removeSubjob(job);
@@ -1409,8 +1449,7 @@ void CopyJobPrivate::slotResultErrorCopyingFiles(KJob *job)
         // Its modification time:
         const UDSEntry entry = static_cast<KIO::StatJob *>(job)->statResult();
 
-        const QDateTime destmtime = QDateTime::fromMSecsSinceEpoch(1000 * entry.numberValue(KIO::UDSEntry::UDS_MODIFICATION_TIME, -1));
-        const QDateTime destctime = QDateTime::fromMSecsSinceEpoch(1000 * entry.numberValue(KIO::UDSEntry::UDS_CREATION_TIME, -1));
+        QDateTime destmtime, destctime;
         const KIO::filesize_t destsize = entry.numberValue(KIO::UDSEntry::UDS_SIZE);
         const QString linkDest = entry.stringValue(KIO::UDSEntry::UDS_LINK_DEST);
 
@@ -1428,6 +1467,9 @@ void CopyJobPrivate::slotResultErrorCopyingFiles(KJob *job)
                 options = RenameDialog_OverwriteItself;
             } else {
                 options = RenameDialog_Overwrite;
+                // These timestamps are used only when RenameDialog_Overwrite is set.
+                destmtime = QDateTime::fromMSecsSinceEpoch(1000 * entry.numberValue(KIO::UDSEntry::UDS_MODIFICATION_TIME, -1), Qt::UTC);
+                destctime = QDateTime::fromMSecsSinceEpoch(1000 * entry.numberValue(KIO::UDSEntry::UDS_CREATION_TIME, -1), Qt::UTC);
             }
             isDir = false;
         }
@@ -1480,6 +1522,7 @@ void CopyJobPrivate::slotResultErrorCopyingFiles(KJob *job)
         newUrl.setPath(newPath);
         emit q->renamed(q, (*it).uDest, newUrl);   // for e.g. kpropsdlg
         (*it).uDest = newUrl;
+        m_bURLDirty = true;
 
         QList<CopyInfo> files;
         files.append(*it);
@@ -1556,7 +1599,7 @@ KIO::Job *CopyJobPrivate::linkNextFile(const QUrl &uSource, const QUrl &uDest, J
                 QString protocol = uSource.scheme();
                 if (protocol == QLatin1String("ftp")) {
                     config.writeEntry("Icon", QStringLiteral("folder-remote"));
-                } else if (protocol == QLatin1String("http")) {
+                } else if (protocol == QLatin1String("http") || protocol == QLatin1String("https")) {
                     config.writeEntry("Icon", QStringLiteral("text-html"));
                 } else if (protocol == QLatin1String("info")) {
                     config.writeEntry("Icon", QStringLiteral("text-x-texinfo"));
@@ -1606,6 +1649,16 @@ void CopyJobPrivate::copyNextFile()
         if (!bCopyFile) {
             it = files.erase(it);
         }
+
+        if (it != files.end() && (*it).size > ((1ul << 32) - 1)) { // ((1ul << 32) - 1) = 4 GB
+            const auto fileSystem = KFileSystemType::fileSystemType(m_globalDest.toLocalFile());
+            if (fileSystem == KFileSystemType::Fat) {
+                q->setError(ERR_FILE_TOO_LARGE_FOR_FAT32);
+                q->setErrorText((*it).uDest.toDisplayString());
+                q->emitResult();
+                return;
+            }
+        }
     }
 
     if (bCopyFile) { // any file to create, finally ?
@@ -1616,7 +1669,6 @@ void CopyJobPrivate::copyNextFile()
                 q->emitResult();
                 return;
             }
-            //TODO check if dst mount is msdos and (*it).size exceeds it's limits
         }
 
         const QUrl &uSource = (*it).uSource;
@@ -1692,13 +1744,18 @@ void CopyJobPrivate::copyNextFile()
             m_bURLDirty = true;
         }
         q->addSubjob(newjob);
-        q->connect(newjob, SIGNAL(processedSize(KJob*,qulonglong)),
-                   SLOT(slotProcessedSize(KJob*,qulonglong)));
-        q->connect(newjob, SIGNAL(totalSize(KJob*,qulonglong)),
-                   SLOT(slotTotalSize(KJob*,qulonglong)));
+        q->connect(newjob, &Job::processedSize, q, [this](KJob *job, qulonglong processedSize) {
+            slotProcessedSize(job, processedSize);
+        });
+        q->connect(newjob, &Job::totalSize, q, [this](KJob *job, qulonglong totalSize) {
+            slotTotalSize(job, totalSize);
+        });
     } else {
         // We're done
         qCDebug(KIO_COPYJOB_DEBUG) << "copyNextFile finished";
+        --m_processedFiles; // undo the "start at 1" hack
+        slotReport(); // display final numbers, important if progress dialog stays up
+
         deleteNextDir();
     }
 }
@@ -1719,7 +1776,7 @@ void CopyJobPrivate::deleteNextDir()
     } else {
         // This step is done, move on
         state = STATE_SETTING_DIR_ATTRIBUTES;
-        m_directoriesCopiedIterator = m_directoriesCopied.constBegin();
+        m_directoriesCopiedIterator = m_directoriesCopied.cbegin();
         setNextDirAttribute();
     }
 }
@@ -1727,11 +1784,11 @@ void CopyJobPrivate::deleteNextDir()
 void CopyJobPrivate::setNextDirAttribute()
 {
     Q_Q(CopyJob);
-    while (m_directoriesCopiedIterator != m_directoriesCopied.constEnd() &&
+    while (m_directoriesCopiedIterator != m_directoriesCopied.cend() &&
             !(*m_directoriesCopiedIterator).mtime.isValid()) {
         ++m_directoriesCopiedIterator;
     }
-    if (m_directoriesCopiedIterator != m_directoriesCopied.constEnd()) {
+    if (m_directoriesCopiedIterator != m_directoriesCopied.cend()) {
         const QUrl url = (*m_directoriesCopiedIterator).uDest;
         const QDateTime dt = (*m_directoriesCopiedIterator).mtime;
         ++m_directoriesCopiedIterator;
@@ -1766,8 +1823,6 @@ void CopyJobPrivate::setNextDirAttribute()
         if (m_reportTimer) {
             m_reportTimer->stop();
         }
-        --m_processedFiles; // undo the "start at 1" hack
-        slotReport(); // display final numbers, important if progress dialog stays up
 
         q->emitResult();
     }
@@ -1808,7 +1863,6 @@ void CopyJobPrivate::slotProcessedSize(KJob *, qulonglong data_size)
     Q_Q(CopyJob);
     qCDebug(KIO_COPYJOB_DEBUG) << data_size;
     m_fileProcessedSize = data_size;
-    q->setProcessedAmount(KJob::Bytes, m_processedSize + m_fileProcessedSize);
 
     if (m_processedSize + m_fileProcessedSize > m_totalSize) {
         // Example: download any attachment from bugs.kde.org
@@ -1817,7 +1871,6 @@ void CopyJobPrivate::slotProcessedSize(KJob *, qulonglong data_size)
         q->setTotalAmount(KJob::Bytes, m_totalSize); // safety
     }
     qCDebug(KIO_COPYJOB_DEBUG) << "emit processedSize" << (unsigned long) (m_processedSize + m_fileProcessedSize);
-    q->setProcessedAmount(KJob::Bytes, m_processedSize + m_fileProcessedSize);
 }
 
 void CopyJobPrivate::slotTotalSize(KJob *, qulonglong size)
@@ -1870,7 +1923,7 @@ void CopyJobPrivate::slotResultRenaming(KJob *job)
     int err = job->error();
     const QString errText = job->errorText();
     // Merge metadata from subjob
-    KIO::Job *kiojob = dynamic_cast<KIO::Job *>(job);
+    KIO::Job *kiojob = qobject_cast<KIO::Job *>(job);
     Q_ASSERT(kiojob);
     m_incomingMetaData += kiojob->metaData();
     q->removeSubjob(job);
@@ -1893,7 +1946,7 @@ void CopyJobPrivate::slotResultRenaming(KJob *job)
             if (_src != _dest && QString::compare(_src, _dest, Qt::CaseInsensitive) == 0) {
                 qCDebug(KIO_COPYJOB_DEBUG) << "Couldn't rename directly, dest already exists. Detected special case of lower/uppercase renaming in same dir, try with 2 rename calls";
                 const QString srcDir = QFileInfo(_src).absolutePath();
-                QTemporaryFile tmpFile(srcDir + "kio_XXXXXX");
+                QTemporaryFile tmpFile(srcDir + QLatin1String("/kio_XXXXXX"));
                 const bool openOk = tmpFile.open();
                 if (!openOk) {
                     qCWarning(KIO_CORE) << "Couldn't open temp file in" << srcDir;
@@ -1946,10 +1999,11 @@ void CopyJobPrivate::slotResultRenaming(KJob *job)
                 ; // nothing to do, stat+copy+del will overwrite
             } else if ((isDir && m_bAutoRenameDirs) || (!isDir && m_bAutoRenameFiles)) {
                 QUrl destDirectory = m_currentDestURL.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash); // m_currendDestURL includes filename
-                const QString newName = KIO::suggestName(destDirectory, m_currentDestURL.fileName());
+                const QString newName = KFileUtils::suggestName(destDirectory, m_currentDestURL.fileName());
 
                 m_dest = destDirectory;
                 m_dest.setPath(concatPaths(m_dest.path(), newName));
+                emit q->renamed(q, dest, m_dest);
                 KIO::Job *job = KIO::stat(m_dest, StatJob::DestinationSide, 2, KIO::HideProgressInfo);
                 state = STATE_STATING;
                 destinationState = DEST_NOT_STATED;
@@ -1976,7 +2030,7 @@ void CopyJobPrivate::slotResultRenaming(KJob *job)
                     QFileInfo info(m_currentSrcURL.toLocalFile());
                     if (info.exists()) {
                         sizeSrc = info.size();
-                        ctimeSrc = info.created();
+                        ctimeSrc = info.birthTime();
                         mtimeSrc = info.lastModified();
                         isDir = info.isDir();
                     }
@@ -1985,7 +2039,7 @@ void CopyJobPrivate::slotResultRenaming(KJob *job)
                     QFileInfo destInfo(dest.toLocalFile());
                     if (destInfo.exists()) {
                         sizeDest = destInfo.size();
-                        ctimeDest = destInfo.created();
+                        ctimeDest = destInfo.birthTime();
                         mtimeDest = destInfo.lastModified();
                         destIsDir = destInfo.isDir();
                     }
@@ -2041,6 +2095,7 @@ void CopyJobPrivate::slotResultRenaming(KJob *job)
                     // Set m_dest to the chosen destination
                     // This is only for this src url; the next one will revert to m_globalDest
                     m_dest.setPath(newPath);
+                    emit q->renamed(q, dest, m_dest); // for e.g. KPropertiesDialog
                     KIO::Job *job = KIO::stat(m_dest, StatJob::DestinationSide, 2, KIO::HideProgressInfo);
                     state = STATE_STATING;
                     destinationState = DEST_NOT_STATED;
@@ -2102,7 +2157,9 @@ void CopyJobPrivate::slotResultRenaming(KJob *job)
     } else {
         qCDebug(KIO_COPYJOB_DEBUG) << "Renaming succeeded, move on";
         ++m_processedFiles;
-        emit q->copyingDone(q, *m_currentStatSrc, finalDestUrl(*m_currentStatSrc, dest), QDateTime() /*mtime unknown, and not needed*/, true, true);
+        // Emit copyingDone for FileUndoManager to remember what we did.
+        // Use resolved URL m_currentSrcURL since that's what we just used for renaming. See bug 391606 and kio_desktop's testTrashAndUndo().
+        emit q->copyingDone(q, m_currentSrcURL, finalDestUrl(m_currentSrcURL, dest), QDateTime() /*mtime unknown, and not needed*/, m_bCurrentSrcIsDir, true);
         m_successSrcList.append(*m_currentStatSrc);
         statNextSrc();
     }
