@@ -20,6 +20,7 @@
 */
 
 #include "desktopexecparser.h"
+#include "kiofuse_interface.h"
 
 #include <kmacroexpander.h>
 #include <kshell.h>
@@ -34,6 +35,8 @@
 #include <QDir>
 #include <QUrl>
 #include <QStandardPaths>
+#include <QDBusConnection>
+#include <QDBusReply>
 
 #include <config-kiocore.h> // CMAKE_INSTALL_FULL_LIBEXECDIR_KF5
 
@@ -42,14 +45,18 @@
 class KRunMX1 : public KMacroExpanderBase
 {
 public:
-    KRunMX1(const KService &_service) :
-        KMacroExpanderBase('%'), hasUrls(false), hasSpec(false), service(_service) {}
+    explicit KRunMX1(const KService &_service)
+        : KMacroExpanderBase(QLatin1Char('%'))
+        , hasUrls(false)
+        , hasSpec(false)
+        , service(_service)
+    {}
 
     bool hasUrls;
     bool hasSpec;
 
 protected:
-    int expandEscapedMacro(const QString &str, int pos, QStringList &ret) Q_DECL_OVERRIDE;
+    int expandEscapedMacro(const QString &str, int pos, QStringList &ret) override;
 
 private:
     const KService &service;
@@ -60,13 +67,13 @@ int KRunMX1::expandEscapedMacro(const QString &str, int pos, QStringList &ret)
     uint option = str[pos + 1].unicode();
     switch (option) {
     case 'c':
-        ret << service.name().replace('%', QLatin1String("%%"));
+        ret << service.name().replace(QLatin1Char('%'), QLatin1String("%%"));
         break;
     case 'k':
-        ret << service.entryPath().replace('%', QLatin1String("%%"));
+        ret << service.entryPath().replace(QLatin1Char('%'), QLatin1String("%%"));
         break;
     case 'i':
-        ret << QStringLiteral("--icon") << service.icon().replace('%', QLatin1String("%%"));
+        ret << QStringLiteral("--icon") << service.icon().replace(QLatin1Char('%'), QLatin1String("%%"));
         break;
     case 'm':
 //       ret << "-miniicon" << service.icon().replace( '%', "%%" );
@@ -76,6 +83,7 @@ int KRunMX1::expandEscapedMacro(const QString &str, int pos, QStringList &ret)
     case 'u':
     case 'U':
         hasUrls = true;
+        Q_FALLTHROUGH();
     /* fallthrough */
     case 'f':
     case 'F':
@@ -85,6 +93,7 @@ int KRunMX1::expandEscapedMacro(const QString &str, int pos, QStringList &ret)
     case 'D':
     case 'v':
         hasSpec = true;
+        Q_FALLTHROUGH();
     /* fallthrough */
     default:
         return -2; // subst with same and skip
@@ -95,13 +104,16 @@ int KRunMX1::expandEscapedMacro(const QString &str, int pos, QStringList &ret)
 class KRunMX2 : public KMacroExpanderBase
 {
 public:
-    KRunMX2(const QList<QUrl> &_urls) :
-        KMacroExpanderBase('%'), ignFile(false), urls(_urls) {}
+    explicit KRunMX2(const QList<QUrl> &_urls)
+        : KMacroExpanderBase(QLatin1Char('%'))
+        , ignFile(false),
+        urls(_urls)
+    {}
 
     bool ignFile;
 
 protected:
-    int expandEscapedMacro(const QString &str, int pos, QStringList &ret) Q_DECL_OVERRIDE;
+    int expandEscapedMacro(const QString &str, int pos, QStringList &ret) override;
 
 private:
     void subst(int option, const QUrl &url, QStringList &ret);
@@ -158,8 +170,8 @@ int KRunMX2::expandEscapedMacro(const QString &str, int pos, QStringList &ret)
     case 'N':
     case 'D':
         option += 'a' - 'A';
-        for (QList<QUrl>::ConstIterator it = urls.begin(); it != urls.end(); ++it) {
-            subst(option, *it, ret);
+        for (const QUrl &url : urls) {
+            subst(option, url, ret);
         }
         break;
     case '%':
@@ -185,7 +197,7 @@ QStringList KIO::DesktopExecParser::supportedProtocols(const KService &service)
         if (supportedProtocols.isEmpty()) {
             // compat mode: assume KIO if not set and it's a KDE app (or a KDE service)
             const QStringList categories = service.property(QStringLiteral("Categories")).toStringList();
-            if (categories.contains(QStringLiteral("KDE"))
+            if (categories.contains(QLatin1String("KDE"))
                     || !service.isApplication()
                     || service.entryPath().isEmpty() /*temp service*/) {
                 supportedProtocols.append(QStringLiteral("KIO"));
@@ -196,29 +208,48 @@ QStringList KIO::DesktopExecParser::supportedProtocols(const KService &service)
             }
         }
     }
+
+    // add x-scheme-handler/<protocol>
+    const auto servicesTypes = service.serviceTypes();
+    for (const auto &mimeType : servicesTypes) {
+        if (mimeType.startsWith(QLatin1String("x-scheme-handler/"))) {
+            supportedProtocols << mimeType.mid(17);
+        }
+    }
+
     //qCDebug(KIO_CORE) << "supportedProtocols:" << supportedProtocols;
     return supportedProtocols;
 }
 
 bool KIO::DesktopExecParser::isProtocolInSupportedList(const QUrl &url, const QStringList &supportedProtocols)
 {
-    if (supportedProtocols.contains(QStringLiteral("KIO"))) {
+    if (supportedProtocols.contains(QLatin1String("KIO"))) {
         return true;
     }
     return url.isLocalFile() || supportedProtocols.contains(url.scheme().toLower());
 }
 
+// We have up to two sources of data, for protocols not handled by kioslaves (so called "helper") :
+// 1) the exec line of the .protocol file, if there's one
+// 2) the application associated with x-scheme-handler/<protocol> if there's one
+
+// If both exist, then:
+//  A) if the .protocol file says "launch an application", then the new-style handler-app has priority
+//  B) but if the .protocol file is for a kioslave (e.g. kio_http) then this has priority over
+//     firefox or chromium saying x-scheme-handler/http. Gnome people want to send all HTTP urls
+//     to a webbrowser, but we want mimetype-determination-in-calling-application by default
+//     (the user can configure a BrowserApplication though)
 bool KIO::DesktopExecParser::hasSchemeHandler(const QUrl &url)
 {
     if (KProtocolInfo::isHelperProtocol(url)) {
         return true;
     }
     if (KProtocolInfo::isKnownProtocol(url)) {
-        return false; // see schemeHandler()... this is case B, we prefer kioslaves over the competition
+        return false; // see KRun::init()... this is case B, we prefer kioslaves over the competition
     }
     const KService::Ptr service = KMimeTypeTrader::self()->preferredService(QLatin1String("x-scheme-handler/") + url.scheme());
     if (service) {
-        qCDebug(KIO_CORE) << "preferred service for x-scheme-handler/" + url.scheme() << service->desktopEntryName();
+        qCDebug(KIO_CORE) << QLatin1String("preferred service for x-scheme-handler/") + url.scheme() << service->desktopEntryName();
     }
     return service;
 }
@@ -256,9 +287,9 @@ void KIO::DesktopExecParser::setSuggestedFileName(const QString &suggestedFileNa
 
 static const QString kioexecPath()
 {
-    QString kioexec = QCoreApplication::applicationDirPath() + "/kioexec";
+    QString kioexec = QCoreApplication::applicationDirPath() + QLatin1String("/kioexec");
     if (!QFileInfo::exists(kioexec))
-        kioexec = CMAKE_INSTALL_FULL_LIBEXECDIR_KF5 "/kioexec";
+        kioexec = QStringLiteral(CMAKE_INSTALL_FULL_LIBEXECDIR_KF5 "/kioexec");
     Q_ASSERT(QFileInfo::exists(kioexec));
     return kioexec;
 }
@@ -296,25 +327,50 @@ QStringList KIO::DesktopExecParser::resultingArguments() const
         return result;
     }
 
-    // Check if we need kioexec
+    // Check if we need kioexec, or KIOFuse
     bool useKioexec = false;
+    org::kde::KIOFuse::VFS kiofuse_iface(QStringLiteral("org.kde.KIOFuse"),
+                                         QStringLiteral("/org/kde/KIOFuse"),
+                                         QDBusConnection::sessionBus());
+    struct MountRequest { QDBusPendingReply<QString> reply; int urlIndex; };
+    QVector<MountRequest> requests;
+    requests.reserve(d->urls.count());
+    QStringList appSupportedProtocols = supportedProtocols(d->service);
     if (!mx1.hasUrls) {
-        Q_FOREACH (const QUrl &url, d->urls)
+        for (int i = 0; i < d->urls.count(); ++i)  {
+            const QUrl url = d->urls.at(i);
             if (!url.isLocalFile() && !hasSchemeHandler(url)) {
-                useKioexec = true;
-                //qCDebug(KIO_CORE) << "non-local files, application does not support urls, using kioexec";
-                break;
+                // Lets try a KIOFuse mount instead.
+                requests.push_back({ kiofuse_iface.mountUrl(url.toString()), i });
             }
-    } else { // app claims to support %u/%U, check which protocols
-        QStringList appSupportedProtocols = supportedProtocols(d->service);
-        Q_FOREACH (const QUrl &url, d->urls) {
-            if (!isProtocolInSupportedList(url, appSupportedProtocols) && !hasSchemeHandler(url)) {
-                useKioexec = true;
-                //qCDebug(KIO_CORE) << "application does not support url, using kioexec:" << url;
-                break;
+        }
+    } else if (!appSupportedProtocols.contains(QLatin1String("KIO"))) {
+        for (int i = 0; i < d->urls.count(); ++i)  {
+            const QUrl url = d->urls.at(i);
+            const bool supported = isProtocolInSupportedList(url, appSupportedProtocols);
+            // NOTE: Some non-KIO apps may support the URLs (e.g. VLC supports smb://)
+            // but will not have the password if they are not in the URL itself.
+            // Hence convert URL to KIOFuse equivalent in case there is a password.
+            // @see https://pointieststick.com/2018/01/17/videos-on-samba-shares/
+            // @see https://bugs.kde.org/show_bug.cgi?id=330192
+            if (!supported || (!url.userName().isEmpty() && url.password().isEmpty())) {
+                requests.push_back({ kiofuse_iface.mountUrl(url.toString()), i });
             }
         }
     }
+
+    // Doesn't matter that we've blocked here to process the replies.
+    // Main thing that we want is to send the mount requests without blocking.
+    for (auto &request : requests) {
+        request.reply.waitForFinished();
+        if (request.reply.isError()) {
+            useKioexec = true;
+            // At this point we should just send the original urls to kioexec.
+            // There's no point sending urls to kioexec that go through kiofuse.
+            break;
+        }
+    }
+
     if (useKioexec) {
         // We need to run the app through kioexec
         result << kioexecPath();
@@ -330,6 +386,12 @@ QStringList KIO::DesktopExecParser::resultingArguments() const
         return result;
     }
 
+    // At this point we know we're not using kioexec, so feel free to replace
+    // KIO URLs with their KIOFuse local path.
+    for (const auto &request : qAsConst(requests)) {
+        d->urls[request.urlIndex] = QUrl::fromLocalFile(request.reply.value());
+    }
+
     if (appHasTempFileOption) {
         exec += QLatin1String(" --tempfile");
     }
@@ -365,13 +427,15 @@ QStringList KIO::DesktopExecParser::resultingArguments() const
         KConfigGroup cg(KSharedConfig::openConfig(), "General");
         QString terminal = cg.readPathEntry("TerminalApplication", QStringLiteral("konsole"));
         if (terminal == QLatin1String("konsole")) {
-            if (!d->service.path().isEmpty()) {
-                terminal += " --workdir " + KShell::quoteArg(d->service.path());
+            if (!d->service.workingDirectory().isEmpty()) {
+                terminal += QLatin1String(" --workdir ") + KShell::quoteArg(d->service.workingDirectory());
+            }
+            terminal += QLatin1String(" -qwindowtitle '%c'");
+            if(!d->service.icon().isEmpty()) {
+                terminal += QLatin1String(" -qwindowicon ") + KShell::quoteArg(d->service.icon().replace(QLatin1Char('%'), QLatin1String("%%")));
             }
-            terminal += QLatin1String(" -qwindowtitle '%c' %i");
         }
-        terminal += ' ';
-        terminal += d->service.terminalOptions();
+        terminal += QLatin1Char(' ') + d->service.terminalOptions();
         if (!mx1.expandMacrosShellQuote(terminal)) {
             qCWarning(KIO_CORE) << "KRun: syntax error in command" << terminal << ", service" << d->service.name();
             return QStringList();
@@ -413,7 +477,7 @@ QStringList KIO::DesktopExecParser::resultingArguments() const
 
         result << d->service.username() << QStringLiteral("-c");
         if (err == KShell::FoundMeta) {
-            exec = "/bin/sh -c " + KShell::quoteArg(exec);
+            exec = QLatin1String("/bin/sh -c ") + KShell::quoteArg(exec);
         } else {
             exec = KShell::joinArgs(execlist);
         }
@@ -433,7 +497,7 @@ QStringList KIO::DesktopExecParser::resultingArguments() const
 QString KIO::DesktopExecParser::executableName(const QString &execLine)
 {
     const QString bin = executablePath(execLine);
-    return bin.mid(bin.lastIndexOf('/') + 1);
+    return bin.mid(bin.lastIndexOf(QLatin1Char('/')) + 1);
 }
 
 //static
@@ -441,9 +505,9 @@ QString KIO::DesktopExecParser::executablePath(const QString &execLine)
 {
     // Remove parameters and/or trailing spaces.
     const QStringList args = KShell::splitArgs(execLine);
-    for (QStringList::ConstIterator it = args.begin(); it != args.end(); ++it) {
-        if (!(*it).contains('=')) {
-            return *it;
+    for (const QString &arg : args) {
+        if (!arg.contains(QLatin1Char('='))) {
+            return arg;
         }
     }
     return QString();
