@@ -20,6 +20,7 @@
 
 #include "filecopyjob.h"
 #include "job_p.h"
+#include <QFile>
 #include <QTimer>
 #include "kprotocolmanager.h"
 #include "scheduler.h"
@@ -41,7 +42,7 @@ public:
                        bool move, JobFlags flags)
         : m_sourceSize(filesize_t(-1)), m_src(src), m_dest(dest), m_moveJob(nullptr), m_copyJob(nullptr), m_delJob(nullptr),
           m_chmodJob(nullptr), m_getJob(nullptr), m_putJob(nullptr), m_permissions(permissions),
-          m_move(move), m_mustChmod(0), m_flags(flags)
+          m_move(move), m_mustChmod(0), m_bFileCopyInProgress(false), m_flags(flags)
     {
     }
     KIO::filesize_t m_sourceSize;
@@ -60,6 +61,7 @@ public:
     bool m_canResume: 1;
     bool m_resumeAnswerSent: 1;
     bool m_mustChmod: 1;
+    bool m_bFileCopyInProgress: 1;
     JobFlags m_flags;
 
     void startBestCopyMethod();
@@ -73,24 +75,6 @@ public:
     void slotData(KIO::Job *, const QByteArray &data);
     void slotDataReq(KIO::Job *, QByteArray &data);
     void slotMimetype(KIO::Job *, const QString &type);
-    /**
-     * Forward signal from subjob
-     * @param job the job that emitted this signal
-     * @param size the processed size in bytes
-     */
-    void slotProcessedSize(KJob *job, qulonglong size);
-    /**
-     * Forward signal from subjob
-     * @param job the job that emitted this signal
-     * @param size the total size
-     */
-    void slotTotalSize(KJob *job, qulonglong size);
-    /**
-     * Forward signal from subjob
-     * @param job the job that emitted this signal
-     * @param pct the percentage
-     */
-    void slotPercent(KJob *job, unsigned long pct);
     /**
      * Forward signal from subjob
      * @param job the job that emitted this signal
@@ -111,6 +95,10 @@ public:
         if (!(flags & HideProgressInfo)) {
             KIO::getJobTracker()->registerJob(job);
         }
+        if (!(flags & NoPrivilegeExecution)) {
+            job->d_func()->m_privilegeExecutionEnabled = true;
+            job->d_func()->m_operationType = move ? Move : Copy;
+        }
         return job;
     }
 };
@@ -125,8 +113,10 @@ public:
 FileCopyJob::FileCopyJob(FileCopyJobPrivate &dd)
     : Job(dd)
 {
-    //qDebug();
-    QTimer::singleShot(0, this, SLOT(slotStart()));
+    Q_D(FileCopyJob);
+    QTimer::singleShot(0, this, [d]() {
+        d->slotStart();
+    });
 }
 
 void FileCopyJobPrivate::slotStart()
@@ -216,14 +206,17 @@ void FileCopyJobPrivate::startCopyJob(const QUrl &slave_url)
     Q_Q(FileCopyJob);
     //qDebug();
     KIO_ARGS << m_src << m_dest << m_permissions << (qint8)(m_flags & Overwrite);
-    m_copyJob = new DirectCopyJob(slave_url, packedArgs);
+    auto job = new DirectCopyJob(slave_url, packedArgs);
+    m_copyJob = job;
+    m_copyJob->setParentJob(q);
     if (m_modificationTime.isValid()) {
         m_copyJob->addMetaData(QStringLiteral("modified"), m_modificationTime.toString(Qt::ISODate));     // #55804
     }
     q->addSubjob(m_copyJob);
     connectSubjob(m_copyJob);
-    q->connect(m_copyJob, SIGNAL(canResume(KIO::Job*,KIO::filesize_t)),
-               SLOT(slotCanResume(KIO::Job*,KIO::filesize_t)));
+    q->connect(job, &DirectCopyJob::canResume, q, [this](KIO::Job *job, KIO::filesize_t offset) {
+        slotCanResume(job, offset);
+    });
 }
 
 void FileCopyJobPrivate::startRenameJob(const QUrl &slave_url)
@@ -232,6 +225,7 @@ void FileCopyJobPrivate::startRenameJob(const QUrl &slave_url)
     m_mustChmod = true;  // CMD_RENAME by itself doesn't change permissions
     KIO_ARGS << m_src << m_dest << (qint8)(m_flags & Overwrite);
     m_moveJob = SimpleJobPrivate::newJobNoUi(slave_url, CMD_RENAME, packedArgs);
+    m_moveJob->setParentJob(q);
     if (m_modificationTime.isValid()) {
         m_moveJob->addMetaData(QStringLiteral("modified"), m_modificationTime.toString(Qt::ISODate));     // #55804
     }
@@ -242,14 +236,27 @@ void FileCopyJobPrivate::startRenameJob(const QUrl &slave_url)
 void FileCopyJobPrivate::connectSubjob(SimpleJob *job)
 {
     Q_Q(FileCopyJob);
-    q->connect(job, SIGNAL(totalSize(KJob*,qulonglong)),
-               SLOT(slotTotalSize(KJob*,qulonglong)));
+    q->connect(job, &KJob::totalSize, q, [q](KJob *job, qulonglong totalSize) {
+        Q_UNUSED(job);
+        if (totalSize != q->totalAmount(KJob::Bytes)) {
+            q->setTotalAmount(KJob::Bytes, totalSize);
+        }
+    });
 
-    q->connect(job, SIGNAL(processedSize(KJob*,qulonglong)),
-               SLOT(slotProcessedSize(KJob*,qulonglong)));
+    q->connect(job, &KJob::processedSize, q, [q, this](KJob *job, qulonglong processedSize) {
+        if (job == m_copyJob) {
+            m_bFileCopyInProgress = processedSize > 0;
+        }
+        q->setProcessedAmount(KJob::Bytes, processedSize);
+    });
+
+    q->connect(job, QOverload<KJob*,ulong>::of(&KJob::percent), q, [q](KJob *job, ulong percent) {
+        Q_UNUSED(job);
+        if (percent > q->percent()) {
+            q->setPercent(percent);
+        }
+    });
 
-    q->connect(job, SIGNAL(percent(KJob*,ulong)),
-               SLOT(slotPercent(KJob*,ulong)));
     if (q->isSuspended()) {
         job->suspend();
     }
@@ -301,28 +308,6 @@ bool FileCopyJob::doResume()
     return true;
 }
 
-void FileCopyJobPrivate::slotProcessedSize(KJob *, qulonglong size)
-{
-    Q_Q(FileCopyJob);
-    q->setProcessedAmount(KJob::Bytes, size);
-}
-
-void FileCopyJobPrivate::slotTotalSize(KJob *, qulonglong size)
-{
-    Q_Q(FileCopyJob);
-    if (size != q->totalAmount(KJob::Bytes)) {
-        q->setTotalAmount(KJob::Bytes, size);
-    }
-}
-
-void FileCopyJobPrivate::slotPercent(KJob *, unsigned long pct)
-{
-    Q_Q(FileCopyJob);
-    if (pct > q->percent()) {
-        q->setPercent(pct);
-    }
-}
-
 void FileCopyJobPrivate::startDataPump()
 {
     Q_Q(FileCopyJob);
@@ -332,6 +317,7 @@ void FileCopyJobPrivate::startDataPump()
     m_resumeAnswerSent = false;
     m_getJob = nullptr; // for now
     m_putJob = put(m_dest, m_permissions, (m_flags | HideProgressInfo) /* no GUI */);
+    m_putJob->setParentJob(q);
     //qDebug() << "m_putJob=" << m_putJob << "m_dest=" << m_dest;
     if (m_modificationTime.isValid()) {
         m_putJob->setModificationTime(m_modificationTime);
@@ -339,10 +325,12 @@ void FileCopyJobPrivate::startDataPump()
 
     // The first thing the put job will tell us is whether we can
     // resume or not (this is always emitted)
-    q->connect(m_putJob, SIGNAL(canResume(KIO::Job*,KIO::filesize_t)),
-               SLOT(slotCanResume(KIO::Job*,KIO::filesize_t)));
-    q->connect(m_putJob, SIGNAL(dataReq(KIO::Job*,QByteArray&)),
-               SLOT(slotDataReq(KIO::Job*,QByteArray&)));
+    q->connect(m_putJob, &KIO::TransferJob::canResume, q, [this](KIO::Job *job, KIO::filesize_t offset) {
+        slotCanResume(job, offset);
+    });
+    q->connect(m_putJob, &KIO::TransferJob::dataReq, q, [this](KIO::Job *job, QByteArray &data) {
+        slotDataReq(job, data);
+    });
     q->addSubjob(m_putJob);
 }
 
@@ -388,6 +376,7 @@ void FileCopyJobPrivate::slotCanResume(KIO::Job *job, KIO::filesize_t offset)
 
         if (job == m_putJob) {
             m_getJob = KIO::get(m_src, NoReload, HideProgressInfo /* no GUI */);
+            m_getJob->setParentJob(q);
             //qDebug() << "m_getJob=" << m_getJob << m_src;
             m_getJob->addMetaData(QStringLiteral("errorPage"), QStringLiteral("false"));
             m_getJob->addMetaData(QStringLiteral("AllowCompressedPage"), QStringLiteral("false"));
@@ -400,8 +389,9 @@ void FileCopyJobPrivate::slotCanResume(KIO::Job *job, KIO::filesize_t offset)
                 m_getJob->addMetaData(QStringLiteral("range-start"), KIO::number(offset));
 
                 // Might or might not get emitted
-                q->connect(m_getJob, SIGNAL(canResume(KIO::Job*,KIO::filesize_t)),
-                           SLOT(slotCanResume(KIO::Job*,KIO::filesize_t)));
+                q->connect(m_getJob, &KIO::TransferJob::canResume, q, [this](KIO::Job *job, KIO::filesize_t offset) {
+                    slotCanResume(job, offset);
+                });
             }
             jobSlave(m_putJob)->setOffset(offset);
 
@@ -410,10 +400,12 @@ void FileCopyJobPrivate::slotCanResume(KIO::Job *job, KIO::filesize_t offset)
             connectSubjob(m_getJob);   // Progress info depends on get
             m_getJob->d_func()->internalResume(); // Order a beer
 
-            q->connect(m_getJob, SIGNAL(data(KIO::Job*,QByteArray)),
-                       SLOT(slotData(KIO::Job*,QByteArray)));
-            q->connect(m_getJob, SIGNAL(mimetype(KIO::Job*,QString)),
-                       SLOT(slotMimetype(KIO::Job*,QString)));
+            q->connect(m_getJob, &KIO::TransferJob::data, q, [this](KIO::Job *job, const QByteArray &data) {
+                slotData(job, data);
+            });
+            q->connect(m_getJob, QOverload<KIO::Job *, const QString &>::of(&KIO::TransferJob::mimetype), q, [this](KIO::Job *job, const QString &type) {
+                slotMimetype(job, type);
+            });
         } else { // copyjob
             jobSlave(m_copyJob)->sendResumeAnswer(offset != 0);
         }
@@ -481,6 +473,12 @@ void FileCopyJob::slotResult(KJob *job)
     Q_D(FileCopyJob);
     //qDebug() << "this=" << this << "job=" << job;
     removeSubjob(job);
+
+    // If result comes from copyjob then we are not writing anymore.
+    if (job == d->m_copyJob) {
+        d->m_bFileCopyInProgress = false;
+    }
+
     // Did job have an error ?
     if (job->error()) {
         if ((job == d->m_moveJob) && (job->error() == ERR_UNSUPPORTED_ACTION)) {
@@ -503,6 +501,18 @@ void FileCopyJob::slotResult(KJob *job)
                 d->m_getJob->kill(Quietly);
                 removeSubjob(d->m_getJob);
             }
+        } else if (job == d->m_chmodJob) {
+            d->m_chmodJob = nullptr;
+            if (d->m_delJob) {
+                d->m_delJob->kill(Quietly);
+                removeSubjob(d->m_delJob);
+            }
+        } else if (job == d->m_delJob) {
+            d->m_delJob = nullptr;
+            if (d->m_chmodJob) {
+                d->m_chmodJob->kill(Quietly);
+                removeSubjob(d->m_chmodJob);
+            }
         }
         setError(job->error());
         setErrorText(job->errorText());
@@ -566,6 +576,32 @@ void FileCopyJob::slotResult(KJob *job)
     }
 }
 
+bool FileCopyJob::doKill()
+{
+#ifdef Q_OS_WIN
+//TODO Use SetConsoleCtrlHandler on Windows or similar behaviour.
+// https://stackoverflow.com/questions/2007516/is-there-a-posix-sigterm-alternative-on-windows-a-gentle-kill-for-console-ap
+// https://danielkaes.wordpress.com/2009/06/04/how-to-catch-kill-events-with-python/
+// https://phabricator.kde.org/D25117#566107
+
+    Q_D(FileCopyJob);
+
+    // If we are interrupted in the middle of file copying,
+    // we may end up with corrupted file at the destination.
+    // It is better to clean up this file. If a copy is being
+    // made as part of move operation then delete the dest only if
+    // source file is intact (m_delJob == NULL).
+    if (d->m_bFileCopyInProgress && d->m_copyJob && d->m_dest.isLocalFile()) {
+        if (d->m_flags & Overwrite) {
+            QFile::remove(d->m_dest.toLocalFile() + QStringLiteral(".part"));
+        } else {
+            QFile::remove(d->m_dest.toLocalFile());
+        }
+    }
+#endif
+    return Job::doKill();
+}
+
 FileCopyJob *KIO::file_copy(const QUrl &src, const QUrl &dest, int permissions,
                             JobFlags flags)
 {
