@@ -24,11 +24,9 @@
 
 #include <time.h>
 
-#include <QtCore/QTimer>
-#include <QtCore/QFile>
-#include <QLinkedList>
 
 #include <klocalizedstring.h>
+#include <kstringhandler.h>
 
 #include <kio/jobuidelegateextension.h>
 #include "slave.h"
@@ -78,16 +76,17 @@ bool Job::addSubjob(KJob *jobBase)
     //qDebug() << "addSubjob(" << jobBase << ") this=" << this;
 
     bool ok = KCompositeJob::addSubjob(jobBase);
-    KIO::Job *job = dynamic_cast<KIO::Job *>(jobBase);
+    KIO::Job *job = qobject_cast<KIO::Job *>(jobBase);
     if (ok && job) {
         // Copy metadata into subjob (e.g. window-id, user-timestamp etc.)
         Q_D(Job);
         job->mergeMetaData(d->m_outgoingMetaData);
 
         // Forward information from that subjob.
-        connect(job, SIGNAL(speed(KJob*,ulong)),
-                SLOT(slotSpeed(KJob*,ulong)));
-
+        connect(job, &KJob::speed, this, [this](KJob *job, ulong speed) {
+            Q_UNUSED(job);
+            emitSpeed(speed);
+        });
         job->setProperty("window", property("window")); // see KJobWidgets
         job->setProperty("userTimestamp", property("userTimestamp")); // see KJobWidgets
         job->setUiDelegateExtension(d->m_uiDelegateExtension);
@@ -101,46 +100,66 @@ bool Job::removeSubjob(KJob *jobBase)
     return KCompositeJob::removeSubjob(jobBase);
 }
 
+static QString url_description_string(const QUrl& url)
+{
+    return url.scheme() == QLatin1String("data") ? QStringLiteral("data:[...]") :
+                                    KStringHandler::csqueeze(url.toDisplayString(QUrl::PreferLocalFile), 100);
+}
+
 KIO::JobPrivate::~JobPrivate()
 {
 }
 
 void JobPrivate::emitMoving(KIO::Job *job, const QUrl &src, const QUrl &dest)
 {
-    emit job->description(job, i18nc("@title job", "Moving"),
-                          qMakePair(i18nc("The source of a file operation", "Source"), src.toDisplayString(QUrl::PreferLocalFile)),
-                          qMakePair(i18nc("The destination of a file operation", "Destination"), dest.toDisplayString(QUrl::PreferLocalFile)));
+    static const QString s_title = i18nc("@title job", "Moving");
+    static const QString s_source = i18nc("The source of a file operation", "Source");
+    static const QString s_destination = i18nc("The destination of a file operation", "Destination");
+    emit job->description(job, s_title,
+                          qMakePair(s_source, url_description_string(src)),
+                          qMakePair(s_destination, url_description_string(dest)));
 }
 
 void JobPrivate::emitCopying(KIO::Job *job, const QUrl &src, const QUrl &dest)
 {
-    emit job->description(job, i18nc("@title job", "Copying"),
-                          qMakePair(i18nc("The source of a file operation", "Source"), src.toDisplayString(QUrl::PreferLocalFile)),
-                          qMakePair(i18nc("The destination of a file operation", "Destination"), dest.toDisplayString(QUrl::PreferLocalFile)));
+    static const QString s_title = i18nc("@title job", "Copying");
+    static const QString s_source = i18nc("The source of a file operation", "Source");
+    static const QString s_destination = i18nc("The destination of a file operation", "Destination");
+    emit job->description(job, s_title,
+                          qMakePair(s_source, url_description_string(src)),
+                          qMakePair(s_destination, url_description_string(dest)));
 }
 
 void JobPrivate::emitCreatingDir(KIO::Job *job, const QUrl &dir)
 {
-    emit job->description(job, i18nc("@title job", "Creating directory"),
-                          qMakePair(i18n("Directory"), dir.toDisplayString(QUrl::PreferLocalFile)));
+    static const QString s_title = i18nc("@title job", "Creating directory");
+    static const QString s_directory = i18n("Directory");
+    emit job->description(job, s_title,
+                          qMakePair(s_directory, url_description_string(dir)));
 }
 
 void JobPrivate::emitDeleting(KIO::Job *job, const QUrl &url)
 {
-    emit job->description(job, i18nc("@title job", "Deleting"),
-                          qMakePair(i18n("File"), url.toDisplayString(QUrl::PreferLocalFile)));
+    static const QString s_title = i18nc("@title job", "Deleting");
+    static const QString s_file = i18n("File");
+    emit job->description(job, s_title,
+                          qMakePair(s_file, url_description_string(url)));
 }
 
 void JobPrivate::emitStating(KIO::Job *job, const QUrl &url)
 {
-    emit job->description(job, i18nc("@title job", "Examining"),
-                          qMakePair(i18n("File"), url.toDisplayString(QUrl::PreferLocalFile)));
+    static const QString s_title = i18nc("@title job", "Examining");
+    static const QString s_file = i18n("File");
+    emit job->description(job, s_title,
+                          qMakePair(s_file, url_description_string(url)));
 }
 
 void JobPrivate::emitTransferring(KIO::Job *job, const QUrl &url)
 {
-    emit job->description(job, i18nc("@title job", "Transferring"),
-                          qMakePair(i18nc("The source of a file operation", "Source"), url.toDisplayString(QUrl::PreferLocalFile)));
+    static const QString s_title = i18nc("@title job", "Transferring");
+    static const QString s_source = i18nc("The source of a file operation", "Source");
+    emit job->description(job, s_title,
+                          qMakePair(s_source, url_description_string(url)));
 }
 
 void JobPrivate::emitMounting(KIO::Job *job, const QString &dev, const QString &point)
@@ -159,8 +178,8 @@ void JobPrivate::emitUnmounting(KIO::Job *job, const QString &point)
 bool Job::doKill()
 {
     // kill all subjobs, without triggering their result slot
-    Q_FOREACH (KJob *it, subjobs()) {
-        it->kill(KJob::Quietly);
+    for (KJob *job : subjobs()) {
+        job->kill(KJob::Quietly);
     }
     clearSubjobs();
 
@@ -169,8 +188,8 @@ bool Job::doKill()
 
 bool Job::doSuspend()
 {
-    Q_FOREACH (KJob *it, subjobs()) {
-        if (!it->suspend()) {
+    for (KJob *job : subjobs()) {
+        if (!job->suspend()) {
             return false;
         }
     }
@@ -180,8 +199,8 @@ bool Job::doSuspend()
 
 bool Job::doResume()
 {
-    Q_FOREACH (KJob *it, subjobs()) {
-        if (!it->resume()) {
+    for (KJob *job : subjobs()) {
+        if (!job->resume()) {
             return false;
         }
     }
@@ -189,12 +208,6 @@ bool Job::doResume()
     return true;
 }
 
-void JobPrivate::slotSpeed(KJob *, unsigned long speed)
-{
-    //qDebug() << speed;
-    q_func()->emitSpeed(speed);
-}
-
 //Job::errorString is implemented in job_error.cpp
 
 void Job::setParentJob(Job *job)
@@ -256,6 +269,76 @@ MetaData Job::outgoingMetaData() const
     return d_func()->m_outgoingMetaData;
 }
 
+QByteArray JobPrivate::privilegeOperationData()
+{
+    PrivilegeOperationStatus status = OperationNotAllowed;
+
+    if (m_parentJob) {
+        QByteArray jobData = m_parentJob->d_func()->privilegeOperationData();
+        // Copy meta-data from parent job
+        m_incomingMetaData.insert(QStringLiteral("TestData"), m_parentJob->queryMetaData(QStringLiteral("TestData")));
+        return jobData;
+    } else {
+        if (m_privilegeExecutionEnabled) {
+            status = OperationAllowed;
+            switch (m_operationType) {
+            case ChangeAttr:
+                m_caption = i18n("Change Attribute");
+                m_message = i18n("Root privileges are required to change file attributes. "
+                                 "Do you want to continue?");
+                break;
+            case Copy:
+                m_caption = i18n("Copy Files");
+                m_message = i18n("Root privileges are required to complete the copy operation. "
+                                 "Do you want to continue?");
+                break;
+            case Delete:
+                m_caption = i18n("Delete Files");
+                m_message = i18n("Root privileges are required to complete the delete operation. "
+                                 "However, doing so may damage your system. Do you want to continue?");
+                break;
+            case MkDir:
+                m_caption = i18n("Create Folder");
+                m_message = i18n("Root privileges are required to create this folder. "
+                                 "Do you want to continue?");
+                break;
+            case Move:
+                m_caption = i18n("Move Items");
+                m_message = i18n("Root privileges are required to complete the move operation. "
+                                 "Do you want to continue?");
+                break;
+            case Rename:
+                m_caption = i18n("Rename");
+                m_message = i18n("Root privileges are required to complete renaming. "
+                                 "Do you want to continue?");
+                break;
+            case Symlink:
+                m_caption = i18n("Create Symlink");
+                m_message = i18n("Root privileges are required to create a symlink. "
+                                 "Do you want to continue?");
+                break;
+            case Transfer:
+                m_caption = i18n("Transfer data");
+                m_message = i18n("Root privileges are required to complete transferring data. "
+                                 "Do you want to continue?");
+                Q_FALLTHROUGH();
+            default:
+                break;
+            }
+
+            if (m_outgoingMetaData.value(QStringLiteral("UnitTesting")) == QLatin1String("true")) {
+                // Set meta-data for the top-level job
+                m_incomingMetaData.insert(QStringLiteral("TestData"), QStringLiteral("PrivilegeOperationAllowed"));
+            }
+        }
+    }
+
+    QByteArray parentJobData;
+    QDataStream ds(&parentJobData, QIODevice::WriteOnly);
+    ds << status << m_caption << m_message;
+    return parentJobData;
+}
+
 //////////////////////////
 
 class KIO::DirectCopyJobPrivate: public KIO::SimpleJobPrivate
@@ -271,7 +354,7 @@ public:
      * work on this job.
      * @param slave the slave that starts working on this job
      */
-    void start(Slave *slave) Q_DECL_OVERRIDE;
+    void start(Slave *slave) override;
 
     Q_DECLARE_PUBLIC(DirectCopyJob)
 };
@@ -289,8 +372,8 @@ DirectCopyJob::~DirectCopyJob()
 void DirectCopyJobPrivate::start(Slave *slave)
 {
     Q_Q(DirectCopyJob);
-    q->connect(slave, SIGNAL(canResume(KIO::filesize_t)),
-               SLOT(slotCanResume(KIO::filesize_t)));
+    q->connect(slave, &SlaveInterface::canResume,
+               q, &DirectCopyJob::slotCanResume);
     SimpleJobPrivate::start(slave);
 }
 
