@@ -47,7 +47,7 @@ public:
 #endif
     {}
 #if HAVE_POSIX_ACL
-    KACLPrivate(acl_t acl)
+    explicit KACLPrivate(acl_t acl)
         : m_acl(acl) {}
 #endif
 #if HAVE_POSIX_ACL
@@ -213,7 +213,7 @@ static void printACL(acl_t acl, const QString &comment)
 
 static int getUidForName(const QString &name)
 {
-    struct passwd *user = getpwnam(name.toLocal8Bit());
+    struct passwd *user = getpwnam(name.toLocal8Bit().constData());
     if (user) {
         return user->pw_uid;
     } else {
@@ -223,7 +223,7 @@ static int getUidForName(const QString &name)
 
 static int getGidForName(const QString &name)
 {
-    struct group *group = getgrnam(name.toLocal8Bit());
+    struct group *group = getgrnam(name.toLocal8Bit().constData());
     if (group) {
         return group->gr_gid;
     } else {
@@ -641,7 +641,7 @@ bool KACL::setACL(const QString &aclStr)
 {
     bool ret = false;
 #if HAVE_POSIX_ACL
-    acl_t temp = acl_from_text(aclStr.toLatin1());
+    acl_t temp = acl_from_text(aclStr.toLatin1().constData());
     if (acl_valid(temp) != 0) {
         // TODO errno is set, what to do with it here?
         acl_free(temp);
@@ -676,28 +676,30 @@ QString KACL::asString() const
 #if HAVE_POSIX_ACL
 QString KACL::KACLPrivate::getUserName(uid_t uid) const
 {
-    if (!m_usercache.contains(uid)) {
+    auto it = m_usercache.find(uid);
+    if (it == m_usercache.end()) {
         struct passwd *user = getpwuid(uid);
         if (user) {
-            m_usercache.insert(uid, QString::fromLatin1(user->pw_name));
+            it = m_usercache.insert(uid, QString::fromLatin1(user->pw_name));
         } else {
             return QString::number(uid);
         }
     }
-    return m_usercache[uid];
+    return *it;
 }
 
 QString KACL::KACLPrivate::getGroupName(gid_t gid) const
 {
-    if (!m_groupcache.contains(gid)) {
+    auto it = m_groupcache.find(gid);
+    if (it == m_groupcache.end()) {
         struct group *grp = getgrgid(gid);
         if (grp) {
-            m_groupcache.insert(gid, QString::fromLatin1(grp->gr_name));
+            it = m_groupcache.insert(gid, QString::fromLatin1(grp->gr_name));
         } else {
             return QString::number(gid);
         }
     }
-    return m_groupcache[gid];
+    return *it;
 }
 #endif
 
