@@ -30,14 +30,15 @@
 #include "kiocoredebug.h"
 #include "../pathhelpers_p.h"
 
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QFileInfo>
 #include <QFile>
 #include <QTextStream>
 #include <QDir>
-#include <QLinkedList>
 #include <qmimedatabase.h>
 
+#include <list>
+
 #include <QLoggingCategory>
 Q_DECLARE_LOGGING_CATEGORY(KIO_CORE_DIRLISTER)
 Q_LOGGING_CATEGORY(KIO_CORE_DIRLISTER, "kf5.kio.core.dirlister", QtWarningMsg)
@@ -282,7 +283,7 @@ bool KCoreDirListerCache::listDir(KCoreDirLister *lister, const QUrl &_u,
 
 KCoreDirLister::Private::CachedItemsJob *KCoreDirLister::Private::cachedItemsJobForUrl(const QUrl &url) const
 {
-    Q_FOREACH (CachedItemsJob *job, m_cachedItemsJobs) {
+    for (CachedItemsJob *job : m_cachedItemsJobs) {
         if (job->url() == url) {
             return job;
         }
@@ -517,9 +518,8 @@ void KCoreDirListerCache::forgetDirs(KCoreDirLister *lister)
     lister->d->lstDirs.clear();
 
     qCDebug(KIO_CORE_DIRLISTER) << "Iterating over dirs" << lstDirsCopy;
-    for (QList<QUrl>::const_iterator it = lstDirsCopy.begin();
-            it != lstDirsCopy.end(); ++it) {
-        forgetDirs(lister, *it, false);
+    for (const QUrl &dir : lstDirsCopy) {
+        forgetDirs(lister, dir, false);
     }
 }
 
@@ -533,7 +533,7 @@ static bool manually_mounted(const QString &path, const KMountPoint::List &possi
         return true;
     }
     // noauto -> manually mounted. Otherwise, mounted at boot time, won't be unmounted any time soon hopefully.
-    return mp->mountOptions().contains(QStringLiteral("noauto"));
+    return mp->mountOptions().contains(QLatin1String("noauto"));
 }
 
 void KCoreDirListerCache::forgetDirs(KCoreDirLister *lister, const QUrl &_url, bool notify)
@@ -644,12 +644,6 @@ void KCoreDirListerCache::updateDirectory(const QUrl &_dir)
 
     const QUrl dir = _dir.adjusted(QUrl::StripTrailingSlash);
     if (!checkUpdate(dir)) {
-        if (dir.isLocalFile() && !(findByUrl(nullptr, dir).isNull())) {
-            pendingUpdates.insert(dir.toLocalFile());
-            if (!pendingUpdateTimer.isActive()) {
-                pendingUpdateTimer.start(500);
-            }
-        }
         return;
     }
 
@@ -659,8 +653,8 @@ void KCoreDirListerCache::updateDirectory(const QUrl &_dir)
     //   - update a currently running listing: the listers are in both
 
     KCoreDirListerCacheDirectoryData &dirData = directoryData[dir];
-    QList<KCoreDirLister *> listers = dirData.listersCurrentlyListing;
-    QList<KCoreDirLister *> holders = dirData.listersCurrentlyHolding;
+    const QList<KCoreDirLister *> listers = dirData.listersCurrentlyListing;
+    const QList<KCoreDirLister *> holders = dirData.listersCurrentlyHolding;
 
     qCDebug(KIO_CORE_DIRLISTER) << dir << "listers=" << listers << "holders=" << holders;
 
@@ -674,7 +668,7 @@ void KCoreDirListerCache::updateDirectory(const QUrl &_dir)
     } else {
         // Emit any cached items.
         // updateDirectory() is about the diff compared to the cached items...
-        Q_FOREACH (KCoreDirLister *kdl, listers) {
+        for (const KCoreDirLister *kdl : listers) {
             KCoreDirLister::Private::CachedItemsJob *cachedItemsJob = kdl->d->cachedItemsJobForUrl(dir);
             if (cachedItemsJob) {
                 cachedItemsJob->setEmitCompleted(false);
@@ -693,7 +687,7 @@ void KCoreDirListerCache::updateDirectory(const QUrl &_dir)
         qCWarning(KIO_CORE) << "The unexpected happened.";
         qCWarning(KIO_CORE) << "listers for" << dir << "=" << listers;
         qCWarning(KIO_CORE) << "job=" << job;
-        Q_FOREACH(KCoreDirLister *kdl, listers) {
+        for (const KCoreDirLister *kdl : listers) {
             qCDebug(KIO_CORE_DIRLISTER) << "lister" << kdl << "m_cachedItemsJobs=" << kdl->d->m_cachedItemsJobs;
         }
 #ifndef NDEBUG
@@ -710,18 +704,18 @@ void KCoreDirListerCache::updateDirectory(const QUrl &_dir)
 
     qCDebug(KIO_CORE_DIRLISTER) << "update started in" << dir;
 
-    foreach (KCoreDirLister *kdl, listers) {
+    for (KCoreDirLister *kdl : listers) {
         kdl->jobStarted(job);
     }
 
     if (!holders.isEmpty()) {
         if (!killed) {
-            foreach (KCoreDirLister *kdl, holders) {
+            for (KCoreDirLister *kdl : holders) {
                 kdl->jobStarted(job);
                 emit kdl->started(dir);
             }
         } else {
-            foreach (KCoreDirLister *kdl, holders) {
+            for (KCoreDirLister *kdl : holders) {
                 kdl->jobStarted(job);
             }
         }
@@ -736,7 +730,6 @@ bool KCoreDirListerCache::checkUpdate(const QUrl &_dir)
             item->complete = false;
             item->watchedWhileInCache = false;
             item->decAutoUpdate();
-            // Hmm, this debug output might include login/password from the _dir URL.
             qCDebug(KIO_CORE_DIRLISTER) << "directory " << _dir << " not in use, marked dirty.";
         }
         //else
@@ -829,8 +822,9 @@ void KCoreDirListerCache::slotFilesAdded(const QString &dir /*url*/)   // from K
 
 void KCoreDirListerCache::itemsAddedInDirectory(const QUrl &urlDir)
 {
-    qCDebug(KIO_CORE_DIRLISTER) << urlDir; // output urls, not qstrings, since they might contain a password
-    Q_FOREACH (const QUrl &u, directoriesForCanonicalPath(urlDir)) {
+    qCDebug(KIO_CORE_DIRLISTER) << urlDir;
+    const QList<QUrl> urls = directoriesForCanonicalPath(urlDir);
+    for (const QUrl &u : urls) {
         updateDirectory(u);
     }
 }
@@ -850,8 +844,7 @@ void KCoreDirListerCache::slotFilesRemoved(const QList<QUrl> &fileList)
     QMap<QUrl, KFileItemList> removedItemsByDir;
     QList<QUrl> deletedSubdirs;
 
-    for (auto it = fileList.cbegin(), cend = fileList.end(); it != cend; ++it) {
-        QUrl url(*it);
+    for (const QUrl &url : fileList) {
         DirItem *dirItem = dirItemForUrl(url); // is it a listed directory?
         if (dirItem) {
             deletedSubdirs.append(url);
@@ -888,7 +881,7 @@ void KCoreDirListerCache::slotFilesRemoved(const QList<QUrl> &fileList)
         }
     }
 
-    Q_FOREACH (const QUrl &url, deletedSubdirs) {
+    for (const QUrl &url : qAsConst(deletedSubdirs)) {
         // in case of a dir, check if we have any known children, there's much to do in that case
         // (stopping jobs, removing dirs from cache etc.)
         deleteDir(url);
@@ -899,9 +892,8 @@ void KCoreDirListerCache::slotFilesChanged(const QStringList &fileList)   // fro
 {
     qCDebug(KIO_CORE_DIRLISTER) << fileList;
     QList<QUrl> dirsToUpdate;
-    QStringList::const_iterator it = fileList.begin();
-    for (; it != fileList.end(); ++it) {
-        QUrl url(*it);
+    for (const QString &fileUrl : fileList) {
+        const QUrl url(fileUrl);
         const KFileItem &fileitem = findByUrl(nullptr, url);
         if (fileitem.isNull()) {
             qCDebug(KIO_CORE_DIRLISTER) << "item not found for" << url;
@@ -920,9 +912,8 @@ void KCoreDirListerCache::slotFilesChanged(const QStringList &fileList)   // fro
         }
     }
 
-    QList<QUrl>::const_iterator itdir = dirsToUpdate.constBegin();
-    for (; itdir != dirsToUpdate.constEnd(); ++itdir) {
-        updateDirectory(*itdir);
+    for (const QUrl &dirUrl : qAsConst(dirsToUpdate)) {
+        updateDirectory(dirUrl);
     }
     // ## TODO problems with current jobs listing/updating that dir
     // ( see kde-2.2.2's kdirlister )
@@ -994,8 +985,8 @@ void KCoreDirListerCache::slotFileRenamed(const QString &_src, const QString &_d
         fileitem.determineMimeType();
         reinsert(fileitem, itemOldUrl);
 
-        QSet<KCoreDirLister *> listers = emitRefreshItem(oldItem, fileitem);
-        Q_FOREACH (KCoreDirLister *kdl, listers) {
+        const QSet<KCoreDirLister *> listers = emitRefreshItem(oldItem, fileitem);
+        for (KCoreDirLister *kdl : listers) {
             kdl->d->emitItems();
         }
     }
@@ -1025,7 +1016,7 @@ QSet<KCoreDirLister *> KCoreDirListerCache::emitRefreshItem(const KFileItem &old
         }
     }
     QSet<KCoreDirLister *> listersToRefresh;
-    Q_FOREACH (KCoreDirLister *kdl, listers) {
+    for (KCoreDirLister *kdl : qAsConst(listers)) {
         // For a directory, look for dirlisters where it's the root item.
         QUrl directoryUrl(oldItem.url());
         if (oldItem.isDir() && kdl->d->rootFileItem == oldItem) {
@@ -1045,7 +1036,7 @@ QList<QUrl> KCoreDirListerCache::directoriesForCanonicalPath(const QUrl &dir) co
 {
     QList<QUrl> dirs;
     dirs << dir;
-    dirs << canonicalUrls.value(dir).toSet().toList(); /* make unique; there are faster ways, but this is really small anyway */
+    dirs << canonicalUrls.value(dir).toSet().values(); /* make unique; there are faster ways, but this is really small anyway */
 
     if (dirs.count() > 1) {
         qCDebug(KIO_CORE_DIRLISTER) << dir << "known as" << dirs;
@@ -1076,12 +1067,14 @@ void KCoreDirListerCache::slotFileDirty(const QString &path)
     }
 
     if (isDir) {
-        Q_FOREACH (const QUrl &dir, directoriesForCanonicalPath(url)) {
+        const QList<QUrl> urls = directoriesForCanonicalPath(url);
+        for (const QUrl &dir : urls) {
             handleFileDirty(dir); // e.g. for permission changes
             handleDirDirty(dir);
         }
     } else {
-        Q_FOREACH (const QUrl &dir, directoriesForCanonicalPath(url.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash))) {
+        const QList<QUrl> urls = directoriesForCanonicalPath(url.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash));
+        for (const QUrl &dir : urls) {
             QUrl aliasUrl(dir);
             aliasUrl.setPath(concatPaths(aliasUrl.path(), url.fileName()));
             handleFileDirty(aliasUrl);
@@ -1155,7 +1148,8 @@ void KCoreDirListerCache::slotFileDeleted(const QString &path)   // from KDirWat
     const QString fileName = QFileInfo(path).fileName();
     QUrl dirUrl(QUrl::fromLocalFile(path));
     QStringList fileUrls;
-    Q_FOREACH (const QUrl &url, directoriesForCanonicalPath(dirUrl.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash))) {
+    const QList<QUrl> urls = directoriesForCanonicalPath(dirUrl.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash));
+    for (const QUrl &url : urls) {
         QUrl urlInfo(url);
         urlInfo.setPath(concatPaths(urlInfo.path(), fileName));
         fileUrls << urlInfo.toString();
@@ -1184,18 +1178,19 @@ void KCoreDirListerCache::slotEntries(KIO::Job *job, const KIO::UDSEntryList &en
         return;
     }
     KCoreDirListerCacheDirectoryData &dirData = *dit;
-    if (dirData.listersCurrentlyListing.isEmpty()) {
+    const QList<KCoreDirLister *> listers = dirData.listersCurrentlyListing;
+    if (listers.isEmpty()) {
         qCWarning(KIO_CORE) << "Internal error: job is listing" << url << "but directoryData says no listers are currently listing " << url;
 #ifndef NDEBUG
         printDebug();
 #endif
-        Q_ASSERT(!dirData.listersCurrentlyListing.isEmpty());
+        Q_ASSERT(!listers.isEmpty());
         return;
     }
 
     // check if anyone wants the mimetypes immediately
     bool delayedMimeTypes = true;
-    foreach (KCoreDirLister *kdl, dirData.listersCurrentlyListing) {
+    for  (const KCoreDirLister *kdl : listers) {
         delayedMimeTypes &= kdl->d->delayedMimeTypes;
     }
 
@@ -1211,7 +1206,7 @@ void KCoreDirListerCache::slotEntries(KIO::Job *job, const KIO::UDSEntryList &en
             continue;
         }
 
-        if (name == QLatin1String(".")) {
+        if (name == QLatin1Char('.')) {
             Q_ASSERT(dir->rootItem.isNull());
             // Try to reuse an existing KFileItem (if we listed the parent dir)
             // rather than creating a new one. There are many reasons:
@@ -1225,10 +1220,11 @@ void KCoreDirListerCache::slotEntries(KIO::Job *job, const KIO::UDSEntryList &en
                 dir->rootItem = KFileItem(*it, url, delayedMimeTypes, true);
             }
 
-            foreach (KCoreDirLister *kdl, dirData.listersCurrentlyListing)
+            for (KCoreDirLister *kdl : listers) {
                 if (kdl->d->rootFileItem.isNull() && kdl->d->url == url) {
                     kdl->d->rootFileItem = dir->rootItem;
                 }
+            }
         } else if (name != QLatin1String("..")) {
             KFileItem item(*it, url, delayedMimeTypes, true);
 
@@ -1251,13 +1247,13 @@ void KCoreDirListerCache::slotEntries(KIO::Job *job, const KIO::UDSEntryList &en
             // Add the items sorted by url, needed by findByUrl
             dir->insert(item);
 
-            foreach (KCoreDirLister *kdl, dirData.listersCurrentlyListing) {
+            for (KCoreDirLister *kdl : listers) {
                 kdl->d->addNewItem(url, item);
             }
         }
     }
 
-    foreach (KCoreDirLister *kdl, dirData.listersCurrentlyListing) {
+    for (KCoreDirLister *kdl : listers) {
         kdl->d->emitItems();
     }
 }
@@ -1295,7 +1291,7 @@ void KCoreDirListerCache::slotResult(KJob *j)
 #endif
         Q_ASSERT(!dirData.listersCurrentlyListing.isEmpty());
     }
-    QList<KCoreDirLister *> listers = dirData.listersCurrentlyListing;
+    const QList<KCoreDirLister *> listers = dirData.listersCurrentlyListing;
 
     // move all listers to the holding list, do it before emitting
     // the signals to make sure it exists in KCoreDirListerCache in case someone
@@ -1304,7 +1300,7 @@ void KCoreDirListerCache::slotResult(KJob *j)
     dirData.moveListersWithoutCachedItemsJob(jobUrl);
 
     if (job->error()) {
-        foreach (KCoreDirLister *kdl, listers) {
+        for (KCoreDirLister *kdl : listers) {
             kdl->d->jobDone(job);
             if (job->error() != KJob::KilledJobError) {
                 kdl->handleError(job);
@@ -1326,7 +1322,7 @@ void KCoreDirListerCache::slotResult(KJob *j)
         Q_ASSERT(dir);
         dir->complete = true;
 
-        foreach (KCoreDirLister *kdl, listers) {
+        for (KCoreDirLister *kdl : listers) {
             kdl->d->jobDone(job);
             emit kdl->completed(jobUrl);
             if (kdl->d->numJobs() == 0) {
@@ -1393,14 +1389,14 @@ void KCoreDirListerCache::slotRedirection(KIO::Job *j, const QUrl &url)
     const QList<KCoreDirLister *> listers = oldDirData.listersCurrentlyListing;
     Q_ASSERT(!listers.isEmpty());
 
-    foreach (KCoreDirLister *kdl, listers) {
+    for (KCoreDirLister *kdl : listers) {
         kdl->d->redirect(oldUrl, newUrl, false /*clear items*/);
     }
 
     // when a lister was stopped before the job emits the redirection signal, the old url will
     // also be in listersCurrentlyHolding
     const QList<KCoreDirLister *> holders = oldDirData.listersCurrentlyHolding;
-    foreach (KCoreDirLister *kdl, holders) {
+    for (KCoreDirLister *kdl : holders) {
         kdl->jobStarted(job);
         // do it like when starting a new list-job that will redirect later
         // TODO: maybe don't emit started if there's an update running for newUrl already?
@@ -1409,6 +1405,8 @@ void KCoreDirListerCache::slotRedirection(KIO::Job *j, const QUrl &url)
         kdl->d->redirect(oldUrl, newUrl, false /*clear items*/);
     }
 
+    const QList<KCoreDirLister *> allListers = listers + holders;
+
     DirItem *newDir = itemsInUse.value(newUrl);
     if (newDir) {
         qCDebug(KIO_CORE_DIRLISTER) << newUrl << "already in use";
@@ -1430,7 +1428,7 @@ void KCoreDirListerCache::slotRedirection(KIO::Job *j, const QUrl &url)
 
             Q_ASSERT(oldJob);    // ?!
 
-            foreach (KCoreDirLister *kdl, curListers) {   // listers of newUrl
+            for (KCoreDirLister *kdl : qAsConst(curListers)) {   // listers of newUrl
                 kdl->d->jobDone(oldJob);
 
                 kdl->jobStarted(job);
@@ -1438,7 +1436,7 @@ void KCoreDirListerCache::slotRedirection(KIO::Job *j, const QUrl &url)
             }
 
             // append listers of oldUrl with newJob to listers of newUrl with oldJob
-            foreach (KCoreDirLister *kdl, listers) {
+            for (KCoreDirLister *kdl : listers) {
                 curListers.append(kdl);
             }
         } else {
@@ -1454,13 +1452,13 @@ void KCoreDirListerCache::slotRedirection(KIO::Job *j, const QUrl &url)
         if (!curHolders.isEmpty()) {
             qCDebug(KIO_CORE_DIRLISTER) << "and it is currently held.";
 
-            foreach (KCoreDirLister *kdl, curHolders) {    // holders of newUrl
+            for (KCoreDirLister *kdl : qAsConst(curHolders)) {    // holders of newUrl
                 kdl->jobStarted(job);
                 emit kdl->started(newUrl);
             }
 
             // append holders of oldUrl to holders of newUrl
-            foreach (KCoreDirLister *kdl, holders) {
+            for (KCoreDirLister *kdl : holders) {
                 curHolders.append(kdl);
             }
         } else {
@@ -1469,7 +1467,7 @@ void KCoreDirListerCache::slotRedirection(KIO::Job *j, const QUrl &url)
 
         // emit old items: listers, holders. NOT: newUrlListers/newUrlHolders, they already have them listed
         // TODO: make this a separate method?
-        foreach (KCoreDirLister *kdl, listers + holders) {
+        for (KCoreDirLister *kdl : allListers) {
             if (kdl->d->rootFileItem.isNull() && kdl->d->url == newUrl) {
                 kdl->d->rootFileItem = newDir->rootItem;
             }
@@ -1487,7 +1485,7 @@ void KCoreDirListerCache::slotRedirection(KIO::Job *j, const QUrl &url)
         newDirData.listersCurrentlyHolding = holders;
 
         // emit old items: listers, holders
-        foreach (KCoreDirLister *kdl, listers + holders) {
+        for (KCoreDirLister *kdl : allListers) {
             if (kdl->d->rootFileItem.isNull() && kdl->d->url == newUrl) {
                 kdl->d->rootFileItem = newDir->rootItem;
             }
@@ -1545,13 +1543,13 @@ void KCoreDirListerCache::renameDir(const QUrl &oldUrl, const QUrl &newUrl)
     //DirItem *dir = itemsInUse.take( oldUrlStr );
     //emitRedirections( oldUrl, url );
 
-    QLinkedList<ItemInUseChange> itemsToChange;
+    std::vector<ItemInUseChange> itemsToChange;
     QSet<KCoreDirLister *> listers;
 
     // Look at all dirs being listed/shown
     for (auto itu = itemsInUse.begin(), ituend = itemsInUse.end(); itu != ituend; ++itu) {
         DirItem *dir = itu.value();
-        QUrl oldDirUrl(itu.key());
+        const QUrl &oldDirUrl = itu.key();
         qCDebug(KIO_CORE_DIRLISTER) << "itemInUse:" << oldDirUrl;
         // Check if this dir is oldUrl, or a subfolder of it
         if (oldDirUrl == oldUrl || oldUrl.isParentOf(oldDirUrl)) {
@@ -1567,14 +1565,14 @@ void KCoreDirListerCache::renameDir(const QUrl &oldUrl, const QUrl &newUrl)
             // Update URL in dir item and in itemsInUse
             dir->redirect(newDirUrl);
 
-            itemsToChange.append(ItemInUseChange(oldDirUrl.adjusted(QUrl::StripTrailingSlash),
-                                                 newDirUrl.adjusted(QUrl::StripTrailingSlash),
-                                                 dir));
+            itemsToChange.emplace_back(oldDirUrl.adjusted(QUrl::StripTrailingSlash),
+                                       newDirUrl.adjusted(QUrl::StripTrailingSlash),
+                                       dir);
             // Rename all items under that dir
             // If all items of the directory change the same part of their url, the order is not
             // changed, therefore just change it in the list.
-            for (auto kit = dir->lstItems.begin(), kend = dir->lstItems.end(); kit != kend; ++kit) {
-                const KFileItem oldItem = *kit;
+            for (KFileItem &item : dir->lstItems) {
+                const KFileItem oldItem = item;
                 KFileItem newItem = oldItem;
                 const QUrl &oldItemUrl = oldItem.url();
                 QUrl newItemUrl(oldItemUrl);
@@ -1584,23 +1582,23 @@ void KCoreDirListerCache::renameDir(const QUrl &oldUrl, const QUrl &newUrl)
 
                 listers |= emitRefreshItem(oldItem, newItem);
                 // Change the item
-                kit->setUrl(newItemUrl);
+                item.setUrl(newItemUrl);
             }
         }
     }
 
-    Q_FOREACH (KCoreDirLister *kdl, listers) {
+    for (KCoreDirLister *kdl : qAsConst(listers)) {
         kdl->d->emitItems();
     }
 
     // Do the changes to itemsInUse out of the loop to avoid messing up iterators,
     // and so that emitRefreshItem can find the stuff in the hash.
-    foreach (const ItemInUseChange &i, itemsToChange) {
+    for (const ItemInUseChange &i : itemsToChange) {
         itemsInUse.remove(i.oldUrl);
         itemsInUse.insert(i.newUrl, i.dirItem);
     }
     //Now that all the caches are updated and consistent, emit the redirection.
-    foreach(const ItemInUseChange& i, itemsToChange) {
+    for (const ItemInUseChange& i : itemsToChange) {
         emitRedirections(QUrl(i.oldUrl), QUrl(i.newUrl));
     }
     // Is oldUrl a directory in the cache?
@@ -1632,7 +1630,7 @@ void KCoreDirListerCache::emitRedirections(const QUrl &_oldUrl, const QUrl &_new
     KCoreDirListerCacheDirectoryData &newDirData = directoryData[newUrl];
 
     // Tell the world that the job listing the old url is dead.
-    foreach (KCoreDirLister *kdl, listers) {
+    for (KCoreDirLister *kdl : listers) {
         if (job) {
             kdl->d->jobDone(job);
         }
@@ -1642,7 +1640,7 @@ void KCoreDirListerCache::emitRedirections(const QUrl &_oldUrl, const QUrl &_new
     newDirData.listersCurrentlyListing += listers;
 
     // Check if we are currently displaying this directory (odds opposite wrt above)
-    foreach (KCoreDirLister *kdl, holders) {
+    for (KCoreDirLister *kdl : holders) {
         if (job) {
             kdl->d->jobDone(job);
         }
@@ -1654,13 +1652,13 @@ void KCoreDirListerCache::emitRedirections(const QUrl &_oldUrl, const QUrl &_new
         updateDirectory(newUrl);
 
         // Tell the world about the new url
-        foreach (KCoreDirLister *kdl, listers) {
+        for (KCoreDirLister *kdl : listers) {
             emit kdl->started(newUrl);
         }
     }
 
     // And notify the dirlisters of the redirection
-    foreach (KCoreDirLister *kdl, holders) {
+    for (KCoreDirLister *kdl : holders) {
         kdl->d->redirect(oldUrl, newUrl, true /*keep items*/);
     }
 }
@@ -1695,14 +1693,14 @@ void KCoreDirListerCache::slotUpdateResult(KJob *j)
     // Collect the dirlisters which were listing the URL using that ListJob
     // plus those that were already holding that URL - they all get updated.
     dirData.moveListersWithoutCachedItemsJob(jobUrl);
-    QList<KCoreDirLister *> listers = dirData.listersCurrentlyHolding;
-    listers += dirData.listersCurrentlyListing;
+    const QList<KCoreDirLister *> listers = dirData.listersCurrentlyHolding +
+                                            dirData.listersCurrentlyListing;
 
     // once we are updating dirs that are only in the cache this will fail!
     Q_ASSERT(!listers.isEmpty());
 
     if (job->error()) {
-        foreach (KCoreDirLister *kdl, listers) {
+        for (KCoreDirLister *kdl : listers) {
             kdl->d->jobDone(job);
 
             //don't bother the user
@@ -1741,7 +1739,7 @@ void KCoreDirListerCache::slotUpdateResult(KJob *j)
 
     // check if anyone wants the mimetypes immediately
     bool delayedMimeTypes = true;
-    foreach (KCoreDirLister *kdl, listers) {
+    for (const KCoreDirLister *kdl : listers) {
         delayedMimeTypes &= kdl->d->delayedMimeTypes;
     }
 
@@ -1750,8 +1748,8 @@ void KCoreDirListerCache::slotUpdateResult(KJob *j)
 
     // Fill the hash from the old list of items. We'll remove entries as we see them
     // in the new listing, and the resulting hash entries will be the deleted items.
-    for (auto kit = dir->lstItems.begin(), kend = dir->lstItems.end(); kit != kend; ++kit) {
-        fileItems.insert((*kit).name(), *kit);
+    for (const KFileItem &item : qAsConst(dir->lstItems)) {
+        fileItems.insert(item.name(), item);
     }
 
     QSet<QString> filesToHide;
@@ -1772,16 +1770,17 @@ void KCoreDirListerCache::slotUpdateResult(KJob *j)
             continue;
         }
 
-        if (name == QLatin1String(".")) {
+        if (name == QLatin1Char('.')) {
             // if the update was started before finishing the original listing
             // there is no root item yet
             if (dir->rootItem.isNull()) {
                 dir->rootItem = item;
 
-                foreach (KCoreDirLister *kdl, listers)
+                for (KCoreDirLister *kdl : listers) {
                     if (kdl->d->rootFileItem.isNull() && kdl->d->url == jobUrl) {
                         kdl->d->rootFileItem = dir->rootItem;
                     }
+                }
             }
             continue;
         } else {
@@ -1818,7 +1817,7 @@ void KCoreDirListerCache::slotUpdateResult(KJob *j)
                 qCDebug(KIO_CORE_DIRLISTER) << "file changed:" << tmp.name();
 
                 reinsert(item, tmp.url());
-                foreach (KCoreDirLister *kdl, listers) {
+                for (KCoreDirLister *kdl : listers) {
                     kdl->d->addRefreshItem(jobUrl, tmp, item);
                 }
             }
@@ -1828,7 +1827,7 @@ void KCoreDirListerCache::slotUpdateResult(KJob *j)
             qCDebug(KIO_CORE_DIRLISTER) << "new file:" << name;
             dir->insert(item);
 
-            foreach (KCoreDirLister *kdl, listers) {
+            for (KCoreDirLister *kdl : listers) {
                 kdl->d->addNewItem(jobUrl, item);
             }
         }
@@ -1840,7 +1839,7 @@ void KCoreDirListerCache::slotUpdateResult(KJob *j)
         deleteUnmarkedItems(listers, dir->lstItems, fileItems);
     }
 
-    foreach (KCoreDirLister *kdl, listers) {
+    for (KCoreDirLister *kdl : listers) {
         kdl->d->emitItems();
 
         kdl->d->jobDone(job);
@@ -1917,7 +1916,7 @@ void KCoreDirListerCache::deleteUnmarkedItems(const QList<KCoreDirLister *> &lis
 
 void KCoreDirListerCache::itemsDeleted(const QList<KCoreDirLister *> &listers, const KFileItemList &deletedItems)
 {
-    Q_FOREACH (KCoreDirLister *kdl, listers) {
+    for (KCoreDirLister *kdl : listers) {
         kdl->d->emitItemsDeleted(deletedItems);
     }
 
@@ -1943,27 +1942,27 @@ void KCoreDirListerCache::deleteDir(const QUrl &_dirUrl)
     auto itu = itemsInUse.begin();
     const auto ituend = itemsInUse.end();
     for (; itu != ituend; ++itu) {
-        const QUrl deletedUrl(itu.key());
+        const QUrl &deletedUrl = itu.key();
         if (dirUrl == deletedUrl || dirUrl.isParentOf(deletedUrl)) {
             affectedItems.append(deletedUrl);
         }
     }
 
-    foreach (const QUrl &deletedUrl, affectedItems) {
+    for (const QUrl &deletedUrl : qAsConst(affectedItems)) {
         // stop all jobs for deletedUrlStr
         DirectoryDataHash::iterator dit = directoryData.find(deletedUrl);
         if (dit != directoryData.end()) {
             // we need a copy because stop modifies the list
-            QList<KCoreDirLister *> listers = (*dit).listersCurrentlyListing;
-            foreach (KCoreDirLister *kdl, listers) {
+            const QList<KCoreDirLister *> listers = (*dit).listersCurrentlyListing;
+            for (KCoreDirLister *kdl : listers) {
                 stopListingUrl(kdl, deletedUrl);
             }
             // tell listers holding deletedUrl to forget about it
             // this will stop running updates for deletedUrl as well
 
             // we need a copy because forgetDirs modifies the list
-            QList<KCoreDirLister *> holders = (*dit).listersCurrentlyHolding;
-            foreach (KCoreDirLister *kdl, holders) {
+            const QList<KCoreDirLister *> holders = (*dit).listersCurrentlyHolding;
+            for (KCoreDirLister *kdl : holders) {
                 // lister's root is the deleted item
                 if (kdl->d->url == deletedUrl) {
                     // tell the view first. It might need the subdirs' items (which forgetDirs will delete)
@@ -2001,7 +2000,7 @@ void KCoreDirListerCache::deleteDir(const QUrl &_dirUrl)
 void KCoreDirListerCache::processPendingUpdates()
 {
     QSet<KCoreDirLister *> listers;
-    foreach (const QString &file, pendingUpdates) { // always a local path
+    for (const QString &file : qAsConst(pendingUpdates)) { // always a local path
         qCDebug(KIO_CORE_DIRLISTER) << file;
         QUrl u = QUrl::fromLocalFile(file);
         KFileItem item = findByUrl(nullptr, u);   // search all items
@@ -2017,12 +2016,12 @@ void KCoreDirListerCache::processPendingUpdates()
         }
     }
     pendingUpdates.clear();
-    Q_FOREACH (KCoreDirLister *kdl, listers) {
+    for (KCoreDirLister *kdl : qAsConst(listers)) {
         kdl->d->emitItems();
     }
 
     // Directories in need of updating
-    foreach (const QString &dir, pendingDirectoryUpdates) {
+    for (const QString &dir : qAsConst(pendingDirectoryUpdates)) {
         updateDirectory(QUrl::fromLocalFile(dir));
     }
     pendingDirectoryUpdates.clear();
@@ -2047,11 +2046,12 @@ void KCoreDirListerCache::printDebug()
     DirectoryDataHash::const_iterator dit = directoryData.constBegin();
     for (; dit != directoryData.constEnd(); ++dit) {
         QString list;
-        foreach (KCoreDirLister *listit, (*dit).listersCurrentlyListing) {
+        const QList<KCoreDirLister *> listers = (*dit).listersCurrentlyListing;
+        for (KCoreDirLister *listit : listers) {
             list += QLatin1String(" 0x") + QString::number(reinterpret_cast<qlonglong>(listit), 16);
         }
-        qCDebug(KIO_CORE_DIRLISTER) << "  " << dit.key() << (*dit).listersCurrentlyListing.count() << "listers:" << list;
-        foreach (KCoreDirLister *listit, (*dit).listersCurrentlyListing) {
+        qCDebug(KIO_CORE_DIRLISTER) << "  " << dit.key() << listers.count() << "listers:" << list;
+        for (KCoreDirLister *listit : listers) {
             if (!listit->d->m_cachedItemsJobs.isEmpty()) {
                 qCDebug(KIO_CORE_DIRLISTER) << "  Lister" << listit << "has CachedItemsJobs" << listit->d->m_cachedItemsJobs;
             } else if (KIO::ListJob *listJob = jobForUrl(dit.key())) {
@@ -2062,10 +2062,11 @@ void KCoreDirListerCache::printDebug()
         }
 
         list.clear();
-        foreach (KCoreDirLister *listit, (*dit).listersCurrentlyHolding) {
+        const QList<KCoreDirLister *> holders = (*dit).listersCurrentlyHolding;
+        for (KCoreDirLister *listit : holders) {
             list += QLatin1String(" 0x") + QString::number(reinterpret_cast<qlonglong>(listit), 16);
         }
-        qCDebug(KIO_CORE_DIRLISTER) << "  " << dit.key() << (*dit).listersCurrentlyHolding.count() << "holders:" << list;
+        qCDebug(KIO_CORE_DIRLISTER) << "  " << dit.key() << holders.count() << "holders:" << list;
     }
 
     QMap< KIO::ListJob *, KIO::UDSEntryList >::Iterator jit = runningListJobs.begin();
@@ -2084,7 +2085,7 @@ void KCoreDirListerCache::printDebug()
     }
 
     // Abort on listers without jobs -after- showing the full dump. Easier debugging.
-    Q_FOREACH (KCoreDirLister *listit, listersWithoutJob) {
+    for (KCoreDirLister *listit : qAsConst(listersWithoutJob)) {
         qCWarning(KIO_CORE) << "Fatal Error: HUH? Lister" << listit << "is supposed to be listing, but has no job!";
         abort();
     }
@@ -2213,13 +2214,13 @@ void KCoreDirLister::Private::emitChanges()
 
     // Fill hash with all items that are currently visible
     QSet<QString> oldVisibleItems;
-    Q_FOREACH (const QUrl &dir, lstDirs) {
-        QList<KFileItem> *itemList = kDirListerCache()->itemsForDir(dir);
+    for (const QUrl &dir : qAsConst(lstDirs)) {
+        const QList<KFileItem> *itemList = kDirListerCache()->itemsForDir(dir);
         if (!itemList) {
             continue;
         }
 
-        foreach (const KFileItem &item, *itemList) {
+        for (const KFileItem &item : *itemList) {
             if (isItemVisible(item) && m_parent->matchesMimeFilter(item)) {
                 oldVisibleItems.insert(item.name());
             }
@@ -2228,10 +2229,11 @@ void KCoreDirLister::Private::emitChanges()
 
     settings = newSettings;
 
-    Q_FOREACH (const QUrl &dir, lstDirs) {
+    const QList<QUrl> dirs = lstDirs;
+    for (const QUrl &dir : dirs) {
         KFileItemList deletedItems;
 
-        QList<KFileItem> *itemList = kDirListerCache()->itemsForDir(dir);
+        const QList<KFileItem> *itemList = kDirListerCache()->itemsForDir(dir);
         if (!itemList) {
             continue;
         }
@@ -2239,9 +2241,9 @@ void KCoreDirLister::Private::emitChanges()
         auto kit = itemList->begin();
         const auto kend = itemList->end();
         for (; kit != kend; ++kit) {
-            KFileItem &item = *kit;
+            const KFileItem &item = *kit;
             const QString text = item.text();
-            if (text == QLatin1String(".") || text == QLatin1String("..")) {
+            if (text == QLatin1Char('.') || text == QLatin1String("..")) {
                 continue;
             }
             const bool wasVisible = oldVisibleItems.contains(item.name());
@@ -2298,9 +2300,15 @@ void KCoreDirLister::setNameFilter(const QString &nameFilter)
     d->settings.lstFilters.clear();
     d->nameFilter = nameFilter;
     // Split on white space
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
     const QStringList list = nameFilter.split(QLatin1Char(' '), QString::SkipEmptyParts);
-    for (QStringList::const_iterator it = list.begin(); it != list.end(); ++it) {
-        d->settings.lstFilters.append(QRegExp(*it, Qt::CaseInsensitive, QRegExp::Wildcard));
+#else
+    const QStringList list = nameFilter.split(QLatin1Char(' '), Qt::SkipEmptyParts);
+#endif
+    for (const QString &filter : list) {
+        d->settings.lstFilters.append(
+            QRegularExpression(QRegularExpression::wildcardToRegularExpression(filter),
+                               QRegularExpression::CaseInsensitiveOption));
     }
 }
 
@@ -2316,7 +2324,7 @@ void KCoreDirLister::setMimeFilter(const QStringList &mimeFilter)
     }
 
     d->prepareForSettingsChange();
-    if (mimeFilter.contains(QStringLiteral("application/octet-stream")) || mimeFilter.contains(QStringLiteral("all/allfiles"))) { // all files
+    if (mimeFilter.contains(QLatin1String("application/octet-stream")) || mimeFilter.contains(QLatin1String("all/allfiles"))) { // all files
         d->settings.mimeFilter.clear();
     } else {
         d->settings.mimeFilter = mimeFilter;
@@ -2347,7 +2355,13 @@ QStringList KCoreDirLister::mimeFilters() const
 
 bool KCoreDirLister::matchesFilter(const QString &name) const
 {
-    return doNameFilter(name, d->settings.lstFilters);
+    for (const QRegularExpression &filter : d->settings.lstFilters) {
+        if (filter.match(name).hasMatch()) {
+            return true;
+        }
+    }
+
+    return false;
 }
 
 bool KCoreDirLister::matchesMimeFilter(const QString &mime) const
@@ -2389,10 +2403,11 @@ bool KCoreDirLister::matchesMimeFilter(const KFileItem &item) const
 
 bool KCoreDirLister::doNameFilter(const QString &name, const QList<QRegExp> &filters) const
 {
-    for (QList<QRegExp>::const_iterator it = filters.begin(); it != filters.end(); ++it)
-        if ((*it).exactMatch(name)) {
+    for (const QRegExp &filter : filters) {
+        if (filter.exactMatch(name)) {
             return true;
         }
+    }
 
     return false;
 }
