@@ -28,7 +28,6 @@
 #include <QDir>
 #include <QDirIterator>
 #include <QFile>
-#include <QMap>
 #include <QDataStream>
 #include <QMimeDatabase>
 #include <QDebug>
@@ -51,7 +50,8 @@ public:
                      mode_t mode, mode_t permissions,
                      const QUrl &itemOrDirUrl,
                      bool urlIsDirectory,
-                     bool delayedMimeTypes)
+                     bool delayedMimeTypes,
+                     KFileItem::MimeTypeDetermination mimeTypeDetermination)
         : m_entry(entry),
           m_url(itemOrDirUrl),
           m_strName(),
@@ -67,7 +67,9 @@ public:
           m_delayedMimeTypes(delayedMimeTypes),
           m_useIconNameCache(false),
           m_hidden(Auto),
-          m_slow(SlowUnknown)
+          m_slow(SlowUnknown),
+          m_bSkipMimeTypeFromContent(mimeTypeDetermination == KFileItem::SkipMimeTypeFromContent),
+          m_bInitCalled(false)
     {
         if (entry.count() != 0) {
             readUDSEntry(urlIsDirectory);
@@ -76,16 +78,17 @@ public:
             m_strName = itemOrDirUrl.fileName();
             m_strText = KIO::decodeFileName(m_strName);
         }
-        init();
     }
 
     /**
-     * Computes the text and mode from the UDSEntry
-     * Called by constructor, but can be called again later
-     * Nothing does that anymore though (I guess some old KonqFileItem did)
-     * so it's not a protected method of the public class anymore.
+     * Call init() if not yet done.
      */
-    void init();
+    void ensureInitialized() const;
+
+    /**
+     * Computes the text and mode from the UDSEntry.
+     */
+    void init() const;
 
     QString localPath() const;
     KIO::filesize_t size() const;
@@ -106,6 +109,11 @@ public:
      */
     QString parsePermissions(mode_t perm) const;
 
+    /**
+     * Mime type helper
+     */
+    void determineMimeTypeHelper(const QUrl &url) const;
+
     /**
      * The UDSEntry that contains the data for this fileitem, if it came from a directory listing.
      */
@@ -144,16 +152,16 @@ public:
     /**
      * The file mode
      */
-    mode_t m_fileMode;
+    mutable mode_t m_fileMode;
     /**
      * The permissions
      */
-    mode_t m_permissions;
+    mutable mode_t m_permissions;
 
     /**
      * Whether the file is a link
      */
-    bool m_bLink: 1;
+    mutable bool m_bLink: 1;
     /**
      * True if local file
      */
@@ -171,18 +179,35 @@ public:
     // Slow? (nfs/smb/ssh)
     mutable enum { SlowUnknown, Fast, Slow } m_slow: 3;
 
+    /**
+     * True if mime type determination by content should be skipped
+     */
+    bool m_bSkipMimeTypeFromContent: 1;
+
+    /**
+     * True if init() was called on demand
+     */
+    mutable bool m_bInitCalled: 1;
+
     // For special case like link to dirs over FTP
     QString m_guessedMimeType;
     mutable QString m_access;
+
 };
 
-void KFileItemPrivate::init()
+void KFileItemPrivate::ensureInitialized() const
+{
+    if (!m_bInitCalled) {
+        init();
+    }
+}
+
+void KFileItemPrivate::init() const
 {
     m_access.clear();
     //  metaInfo = KFileMetaInfo();
 
     // stat() local files if needed
-    // TODO: delay this until requested
     if (m_fileMode == KFileItem::Unknown || m_permissions == KFileItem::Unknown || m_entry.count() == 0) {
         if (m_url.isLocalFile()) {
             /* directories may not have a slash at the end if
@@ -229,7 +254,10 @@ void KFileItemPrivate::init()
             }
         }
     }
+
+    m_bInitCalled = true;
 }
+
 void KFileItemPrivate::readUDSEntry(bool _urlIsDirectory)
 {
     // extract fields from the KIO::UDS Entry
@@ -276,6 +304,8 @@ void KFileItemPrivate::readUDSEntry(bool _urlIsDirectory)
 inline //because it is used only in one place
 KIO::filesize_t KFileItemPrivate::size() const
 {
+    ensureInitialized();
+
     // Extract it from the KIO::UDSEntry
     long long fieldVal = m_entry.numberValue(KIO::UDSEntry::UDS_SIZE, -1);
     if (fieldVal != -1) {
@@ -310,11 +340,13 @@ void KFileItemPrivate::setTime(KFileItem::FileTimes mappedWhich, uint time_t_val
 void KFileItemPrivate::setTime(KFileItem::FileTimes mappedWhich, const QDateTime &val) const
 {
     const QDateTime dt = val.toLocalTime(); // #160979
-    setTime(mappedWhich, dt.toTime_t());
+    setTime(mappedWhich, dt.toSecsSinceEpoch());
 }
 
 QDateTime KFileItemPrivate::time(KFileItem::FileTimes mappedWhich) const
 {
+    ensureInitialized();
+
     // Extract it from the KIO::UDSEntry
     const uint uds = udsFieldForTime(mappedWhich);
     if (uds > 0) {
@@ -330,6 +362,14 @@ QDateTime KFileItemPrivate::time(KFileItem::FileTimes mappedWhich) const
 inline //because it is used only in one place
 bool KFileItemPrivate::cmp(const KFileItemPrivate &item) const
 {
+    if (item.m_bInitCalled) {
+        ensureInitialized();
+    }
+
+    if (m_bInitCalled) {
+        item.ensureInitialized();
+    }
+
 #if 0
     //qDebug() << "Comparing" << m_url << "and" << item.m_url;
     //qDebug() << " name" << (m_strName == item.m_strName);
@@ -368,6 +408,8 @@ bool KFileItemPrivate::cmp(const KFileItemPrivate &item) const
 inline //because it is used only in one place
 QString KFileItemPrivate::parsePermissions(mode_t perm) const
 {
+    ensureInitialized();
+
     static char buffer[ 12 ];
 
     char uxbit, gxbit, oxbit;
@@ -448,6 +490,20 @@ QString KFileItemPrivate::parsePermissions(mode_t perm) const
     return QString::fromLatin1(buffer);
 }
 
+void KFileItemPrivate::determineMimeTypeHelper(const QUrl &url) const
+{
+    QMimeDatabase db;
+    if (m_bSkipMimeTypeFromContent) {
+        const QString scheme = url.scheme();
+        if (scheme.startsWith(QLatin1String("http")) || scheme == QLatin1String("mailto"))
+            m_mimeType = db.mimeTypeForName(QLatin1String("application/octet-stream"));
+        else
+            m_mimeType = db.mimeTypeForFile(url.path(), QMimeDatabase::MatchMode::MatchExtension);
+    } else {
+        m_mimeType = db.mimeTypeForUrl(url);
+    }
+}
+
 ///////
 
 KFileItem::KFileItem()
@@ -457,19 +513,19 @@ KFileItem::KFileItem()
 
 KFileItem::KFileItem(const KIO::UDSEntry &entry, const QUrl &itemOrDirUrl,
                      bool delayedMimeTypes, bool urlIsDirectory)
-    : d(new KFileItemPrivate(entry, KFileItem::Unknown, KFileItem::Unknown, itemOrDirUrl, urlIsDirectory, delayedMimeTypes))
+    : d(new KFileItemPrivate(entry, KFileItem::Unknown, KFileItem::Unknown, itemOrDirUrl, urlIsDirectory, delayedMimeTypes, KFileItem::NormalMimeTypeDetermination))
 {
 }
 
 KFileItem::KFileItem(mode_t mode, mode_t permissions, const QUrl &url, bool delayedMimeTypes)
     : d(new KFileItemPrivate(KIO::UDSEntry(), mode, permissions,
-                             url, false, delayedMimeTypes))
+                             url, false, delayedMimeTypes, KFileItem::NormalMimeTypeDetermination))
 {
 }
 
 KFileItem::KFileItem(const QUrl &url, const QString &mimeType, mode_t mode)
     : d(new KFileItemPrivate(KIO::UDSEntry(), mode, KFileItem::Unknown,
-                             url, false, false))
+                             url, false, false, KFileItem::NormalMimeTypeDetermination))
 {
     d->m_bMimeTypeKnown = !mimeType.isEmpty();
     if (d->m_bMimeTypeKnown) {
@@ -478,6 +534,13 @@ KFileItem::KFileItem(const QUrl &url, const QString &mimeType, mode_t mode)
     }
 }
 
+KFileItem::KFileItem(const QUrl &url, KFileItem::MimeTypeDetermination mimeTypeDetermination)
+    : d(new KFileItemPrivate(KIO::UDSEntry(), KFileItem::Unknown, KFileItem::Unknown,
+                             url, false, false, mimeTypeDetermination))
+{
+}
+
+
 // Default implementations for:
 // - Copy constructor
 // - Move constructor
@@ -558,6 +621,8 @@ void KFileItem::setName(const QString &name)
         return;
     }
 
+    d->ensureInitialized();
+
     d->m_strName = name;
     if (!d->m_strName.isEmpty()) {
         d->m_strText = KIO::decodeFileName(d->m_strName);
@@ -574,6 +639,8 @@ QString KFileItem::linkDest() const
         return QString();
     }
 
+    d->ensureInitialized();
+
     // Extract it from the KIO::UDSEntry
     const QString linkStr = d->m_entry.stringValue(KIO::UDSEntry::UDS_LINK_DEST);
     if (!linkStr.isEmpty()) {
@@ -593,6 +660,8 @@ QString KFileItemPrivate::localPath() const
         return m_url.toLocalFile();
     }
 
+    ensureInitialized();
+
     // Extract the local path from the KIO::UDSEntry
     return m_entry.stringValue(KIO::UDSEntry::UDS_LOCAL_PATH);
 }
@@ -622,7 +691,7 @@ bool KFileItem::hasExtendedACL() const
     }
 
     // Check if the field exists; its value doesn't matter
-    return d->m_entry.contains(KIO::UDSEntry::UDS_EXTENDED_ACL);
+    return entry().contains(KIO::UDSEntry::UDS_EXTENDED_ACL);
 }
 
 KACL KFileItem::ACL() const
@@ -638,6 +707,7 @@ KACL KFileItem::ACL() const
             return KACL(fieldVal);
         }
     }
+
     // create one from the basic permissions
     return KACL(d->m_permissions);
 }
@@ -649,7 +719,7 @@ KACL KFileItem::defaultACL() const
     }
 
     // Extract it from the KIO::UDSEntry
-    const QString fieldVal = d->m_entry.stringValue(KIO::UDSEntry::UDS_DEFAULT_ACL_STRING);
+    const QString fieldVal = entry().stringValue(KIO::UDSEntry::UDS_DEFAULT_ACL_STRING);
     if (!fieldVal.isEmpty()) {
         return KACL(fieldVal);
     } else {
@@ -672,7 +742,7 @@ QString KFileItem::user() const
         return QString();
     }
 
-    return d->m_entry.stringValue(KIO::UDSEntry::UDS_USER);
+    return entry().stringValue(KIO::UDSEntry::UDS_USER);
 }
 
 QString KFileItem::group() const
@@ -681,7 +751,7 @@ QString KFileItem::group() const
         return QString();
     }
 
-    return d->m_entry.stringValue(KIO::UDSEntry::UDS_GROUP);
+    return entry().stringValue(KIO::UDSEntry::UDS_GROUP);
 }
 
 bool KFileItemPrivate::isSlow() const
@@ -730,7 +800,8 @@ QMimeType KFileItem::determineMimeType() const
         } else {
             bool isLocalUrl;
             const QUrl url = mostLocalUrl(&isLocalUrl);
-            d->m_mimeType = db.mimeTypeForUrl(url);
+            d->determineMimeTypeHelper(url);
+
             // was:  d->m_mimeType = KMimeType::findByUrl( url, d->m_fileMode, isLocalUrl );
             // => we are no longer using d->m_fileMode for remote URLs.
             Q_ASSERT(d->m_mimeType.isValid());
@@ -956,9 +1027,9 @@ QString KFileItem::iconName() const
 static bool checkDesktopFile(const KFileItem &item, bool _determineMimeType)
 {
     // only local files
-    bool isLocal;
-    const QUrl url = item.mostLocalUrl(&isLocal);
-    if (!isLocal) {
+    bool isLocalUrl;
+    item.mostLocalUrl(&isLocalUrl);
+    if (!isLocalUrl) {
         return false;
     }
 
@@ -983,7 +1054,13 @@ QStringList KFileItem::overlays() const
         return QStringList();
     }
 
+    d->ensureInitialized();
+
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
     QStringList names = d->m_entry.stringValue(KIO::UDSEntry::UDS_ICON_OVERLAY_NAMES).split(QLatin1Char(','), QString::SkipEmptyParts);
+#else
+    QStringList names = d->m_entry.stringValue(KIO::UDSEntry::UDS_ICON_OVERLAY_NAMES).split(QLatin1Char(','), Qt::SkipEmptyParts);
+#endif
 
     if (d->m_bLink) {
         names.append(QStringLiteral("emblem-symbolic-link"));
@@ -1049,6 +1126,8 @@ bool KFileItem::isReadable() const
         return false;
     }
 
+    d->ensureInitialized();
+
     /*
       struct passwd * user = getpwuid( geteuid() );
       bool isMyFile = (QString::fromLocal8Bit(user->pw_name) == d->m_user);
@@ -1084,6 +1163,8 @@ bool KFileItem::isWritable() const
         return false;
     }
 
+    d->ensureInitialized();
+
     /*
       struct passwd * user = getpwuid( geteuid() );
       bool isMyFile = (QString::fromLocal8Bit(user->pw_name) == d->m_user);
@@ -1139,6 +1220,12 @@ bool KFileItem::isDir() const
         return false;
     }
 
+    if (d->m_bSkipMimeTypeFromContent) {
+        return false;
+    }
+
+    d->ensureInitialized();
+
     if (d->m_fileMode == KFileItem::Unknown) {
         // Probably the file was deleted already, and KDirLister hasn't told the world yet.
         //qDebug() << d << url() << "can't say -> false";
@@ -1156,7 +1243,6 @@ bool KFileItem::isFile() const
     return !isDir();
 }
 
-#ifndef KIOCORE_NO_DEPRECATED
 bool KFileItem::acceptsDrops() const
 {
     // A directory ?
@@ -1180,7 +1266,6 @@ bool KFileItem::acceptsDrops() const
 
     return false;
 }
-#endif
 
 QString KFileItem::getStatusBarInfo() const
 {
@@ -1260,7 +1345,7 @@ bool KFileItem::operator<(const QUrl &other) const
 
 KFileItem::operator QVariant() const
 {
-    return qVariantFromValue(*this);
+    return QVariant::fromValue(*this);
 }
 
 QString KFileItem::permissionsString() const
@@ -1269,6 +1354,8 @@ QString KFileItem::permissionsString() const
         return QString();
     }
 
+    d->ensureInitialized();
+
     if (d->m_access.isNull() && d->m_permissions != KFileItem::Unknown) {
         d->m_access = d->parsePermissions(d->m_permissions);
     }
@@ -1286,7 +1373,6 @@ QString KFileItem::timeString(FileTimes which) const
     return d->time(which).toString();
 }
 
-#ifndef KIOCORE_NO_DEPRECATED
 QString KFileItem::timeString(unsigned int which) const
 {
     if (!d) {
@@ -1303,14 +1389,11 @@ QString KFileItem::timeString(unsigned int which) const
         return timeString(ModificationTime);
     }
 }
-#endif
 
-#ifndef KIOCORE_NO_DEPRECATED
 void KFileItem::assign(const KFileItem &item)
 {
     *this = item;
 }
-#endif
 
 QUrl KFileItem::mostLocalUrl(bool *local) const
 {
@@ -1393,6 +1476,8 @@ mode_t KFileItem::permissions() const
         return 0;
     }
 
+    d->ensureInitialized();
+
     return d->m_permissions;
 }
 
@@ -1402,6 +1487,8 @@ mode_t KFileItem::mode() const
         return 0;
     }
 
+    d->ensureInitialized();
+
     return d->m_fileMode;
 }
 
@@ -1411,6 +1498,8 @@ bool KFileItem::isLink() const
         return false;
     }
 
+    d->ensureInitialized();
+
     return d->m_bLink;
 }
 
@@ -1499,7 +1588,7 @@ QMimeType KFileItem::currentMimeType() const
             }
         } else {
             // ## d->m_fileMode isn't used anymore (for remote urls)
-            d->m_mimeType = db.mimeTypeForUrl(url);
+            d->determineMimeTypeHelper(url);
             d->m_bMimeTypeKnown = true;
         }
     }
@@ -1512,6 +1601,8 @@ KIO::UDSEntry KFileItem::entry() const
         return KIO::UDSEntry();
     }
 
+    d->ensureInitialized();
+
     return d->m_entry;
 }
 
@@ -1586,6 +1677,8 @@ bool KFileItem::isRegularFile() const
         return false;
     }
 
+    d->ensureInitialized();
+
     return (d->m_fileMode & QT_STAT_MASK) == QT_STAT_REG;
 }
 
