@@ -20,9 +20,10 @@
 
 #include "ksslcertificatemanager.h"
 #include "ksslcertificatemanager_p.h"
+#include "ksslerror_p.h"
 
 #include "ktcpsocket.h"
-#include "ktcpsocket_p.h"
+#include "ksslerroruidata_p.h"
 #include <kconfig.h>
 #include <kconfiggroup.h>
 #include <klocalizedstring.h>
@@ -32,7 +33,6 @@
 #include <QFile>
 #include <QDir>
 #include <QDBusConnection>
-#include <QDBusReply>
 #include <QDBusConnectionInterface>
 
 #include "kssld_interface.h"
@@ -57,16 +57,6 @@ CertificatePEM = <PEM-encoded certificate> #host entries are all lowercase, thus
 
 // TODO GUI for managing exception rules
 
-class KSslCertificateRulePrivate
-{
-public:
-    QSslCertificate certificate;
-    QString hostName;
-    bool isRejected;
-    QDateTime expiryDateTime;
-    QList<KSslError::Error> ignoredErrors;
-};
-
 KSslCertificateRule::KSslCertificateRule(const QSslCertificate &cert, const QString &hostName)
     : d(new KSslCertificateRulePrivate())
 {
@@ -124,42 +114,70 @@ bool KSslCertificateRule::isRejected() const
 
 bool KSslCertificateRule::isErrorIgnored(KSslError::Error error) const
 {
-    foreach (KSslError::Error ignoredError, d->ignoredErrors)
-        if (error == ignoredError) {
-            return true;
-        }
+    return d->ignoredErrors.contains(KSslErrorPrivate::errorFromKSslError(error));
+}
 
-    return false;
+bool KSslCertificateRule::isErrorIgnored(QSslError::SslError error) const
+{
+    return d->ignoredErrors.contains(error);
 }
 
 void KSslCertificateRule::setIgnoredErrors(const QList<KSslError::Error> &errors)
 {
     d->ignoredErrors.clear();
     //### Quadratic runtime, woohoo! Use a QSet if that should ever be an issue.
-    foreach (KSslError::Error e, errors)
-        if (!isErrorIgnored(e)) {
-            d->ignoredErrors.append(e);
+    for (KSslError::Error e : errors) {
+        QSslError::SslError error = KSslErrorPrivate::errorFromKSslError(e);
+        if (!isErrorIgnored(error)) {
+            d->ignoredErrors.append(error);
         }
+    }
 }
 
 void KSslCertificateRule::setIgnoredErrors(const QList<KSslError> &errors)
 {
     QList<KSslError::Error> el;
-    foreach (const KSslError &e, errors) {
+    el.reserve(errors.size());
+    for (const KSslError &e : errors) {
         el.append(e.error());
     }
     setIgnoredErrors(el);
 }
 
+void KSslCertificateRule::setIgnoredErrors(const QList<QSslError> &errors)
+{
+    d->ignoredErrors.clear();
+    for (const QSslError &error : errors) {
+        if (!isErrorIgnored(error.error())) {
+            d->ignoredErrors.append(error.error());
+        }
+    }
+}
+
+void KSslCertificateRule::setIgnoredErrors(const QList<QSslError::SslError> &errors)
+{
+    d->ignoredErrors.clear();
+    for (QSslError::SslError error : errors) {
+        if (!isErrorIgnored(error)) {
+            d->ignoredErrors.append(error);
+        }
+    }
+}
+
 QList<KSslError::Error> KSslCertificateRule::ignoredErrors() const
 {
-    return d->ignoredErrors;
+    // TODO KF6: return d->ignoredErrors
+    // return d->ignoredErrors;
+    QList<KSslError::Error> errors;
+    errors.reserve(d->ignoredErrors.size());
+    std::transform(d->ignoredErrors.cbegin(), d->ignoredErrors.cend(), std::back_inserter(errors), KSslErrorPrivate::errorFromQSslError);
+    return errors;
 }
 
 QList<KSslError::Error> KSslCertificateRule::filterErrors(const QList<KSslError::Error> &errors) const
 {
     QList<KSslError::Error> ret;
-    foreach (KSslError::Error error, errors) {
+    for (KSslError::Error error : errors) {
         if (!isErrorIgnored(error)) {
             ret.append(error);
         }
@@ -170,7 +188,18 @@ QList<KSslError::Error> KSslCertificateRule::filterErrors(const QList<KSslError:
 QList<KSslError> KSslCertificateRule::filterErrors(const QList<KSslError> &errors) const
 {
     QList<KSslError> ret;
-    foreach (const KSslError &error, errors) {
+    for (const KSslError &error : errors) {
+        if (!isErrorIgnored(error.error())) {
+            ret.append(error);
+        }
+    }
+    return ret;
+}
+
+QList<QSslError> KSslCertificateRule::filterErrors(const QList<QSslError> &errors) const
+{
+    QList<QSslError> ret;
+    for (const QSslError &error : errors) {
         if (!isErrorIgnored(error.error())) {
             ret.append(error);
         }
@@ -184,7 +213,7 @@ static QList<QSslCertificate> deduplicate(const QList<QSslCertificate> &certs)
 {
     QSet<QByteArray> digests;
     QList<QSslCertificate> ret;
-    foreach (const QSslCertificate &cert, certs) {
+    for (const QSslCertificate &cert : certs) {
         QByteArray digest = cert.digest();
         if (!digests.contains(digest)) {
             digests.insert(digest);
@@ -195,35 +224,33 @@ static QList<QSslCertificate> deduplicate(const QList<QSslCertificate> &certs)
 }
 
 KSslCertificateManagerPrivate::KSslCertificateManagerPrivate()
-    : config(QLatin1String("ksslcertificatemanager"), KConfig::SimpleConfig),
-      iface(new org::kde::KSSLDInterface(QLatin1String("org.kde.kiod5"),
-                                         QLatin1String("/modules/kssld"),
+    : config(QStringLiteral("ksslcertificatemanager"), KConfig::SimpleConfig),
+      iface(new org::kde::KSSLDInterface(QStringLiteral("org.kde.kssld5"),
+                                         QStringLiteral("/modules/kssld"),
                                          QDBusConnection::sessionBus())),
       isCertListLoaded(false),
-      userCertDir(QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1String("/kssl/userCaCertificates/"))
+      userCertDir(QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QStringLiteral("/kssl/userCaCertificates/"))
 {
-    // set Qt's set to empty; this is protected by the lock in K_GLOBAL_STATIC.
-    QSslSocket::setDefaultCaCertificates(QList<QSslCertificate>());
 }
 
 KSslCertificateManagerPrivate::~KSslCertificateManagerPrivate()
 {
     delete iface;
-    iface = 0;
+    iface = nullptr;
 }
 
 void KSslCertificateManagerPrivate::loadDefaultCaCertificates()
 {
     defaultCaCertificates.clear();
 
-    QList<QSslCertificate> certs = deduplicate(QSslSocket::systemCaCertificates());
+    QList<QSslCertificate> certs = deduplicate(QSslConfiguration::systemCaCertificates());
 
-    KConfig config(QLatin1String("ksslcablacklist"), KConfig::SimpleConfig);
+    KConfig config(QStringLiteral("ksslcablacklist"), KConfig::SimpleConfig);
     KConfigGroup group = config.group("Blacklist of CA Certificates");
 
-    certs.append(QSslCertificate::fromPath(userCertDir + QLatin1String("*"), QSsl::Pem,
+    certs.append(QSslCertificate::fromPath(userCertDir + QLatin1Char('*'), QSsl::Pem,
                                            QRegExp::Wildcard));
-    foreach (const QSslCertificate &cert, certs) {
+    for (const QSslCertificate &cert : qAsConst(certs)) {
         const QByteArray digest = cert.digest().toHex();
         if (!group.hasKey(digest.constData())) {
             defaultCaCertificates += cert;
@@ -247,9 +274,9 @@ bool KSslCertificateManagerPrivate::addCertificate(const KSslCaCertificate &in)
     }
 
     QString certFilename = userCertDir + QString::fromLatin1(in.certHash);
-    //qDebug() << certFilename;
+
     QFile certFile(certFilename);
-    if (certFile.open(QIODevice::ReadOnly)) {
+    if (!QDir().mkpath(userCertDir) || certFile.open(QIODevice::ReadOnly)) {
         return false;
     }
     if (!certFile.open(QIODevice::WriteOnly)) {
@@ -286,7 +313,8 @@ bool KSslCertificateManagerPrivate::removeCertificate(const KSslCaCertificate &o
 
         bool removed = false;
         QDir dir(userCertDir);
-        foreach (const QString &certFilename, dir.entryList(QDir::Files)) {
+        const QStringList dirList = dir.entryList(QDir::Files);
+        for (const QString &certFilename : dirList) {
             const QString certPath = userCertDir + certFilename;
             QList<QSslCertificate> certs = QSslCertificate::fromPath(certPath);
 
@@ -330,8 +358,8 @@ void KSslCertificateManagerPrivate::setAllCertificates(const QList<KSslCaCertifi
     Q_ASSERT(knownCerts.isEmpty());
     QList<KSslCaCertificate> in = certsIn;
     QList<KSslCaCertificate> old = allCertificates();
-    qSort(in.begin(), in.end(), certLessThan);
-    qSort(old.begin(), old.end(), certLessThan);
+    std::sort(in.begin(), in.end(), certLessThan);
+    std::sort(old.begin(), old.end(), certLessThan);
 
     for (int ii = 0, oi = 0; ii < in.size() || oi < old.size(); ++ii, ++oi) {
         // look at all elements in both lists, even if we reach the end of one early.
@@ -372,20 +400,22 @@ QList<KSslCaCertificate> KSslCertificateManagerPrivate::allCertificates() const
 {
     //qDebug() << Q_FUNC_INFO;
     QList<KSslCaCertificate> ret;
-    foreach (const QSslCertificate &cert, deduplicate(QSslSocket::systemCaCertificates())) {
+    const QList<QSslCertificate> list = deduplicate(QSslConfiguration::systemCaCertificates());
+    for (const QSslCertificate &cert : list) {
         ret += KSslCaCertificate(cert, KSslCaCertificate::SystemStore, false);
     }
 
-    foreach (const QSslCertificate &cert, QSslCertificate::fromPath(userCertDir + QLatin1String("*"),
-             QSsl::Pem, QRegExp::Wildcard)) {
+    const QList<QSslCertificate> userList = QSslCertificate::fromPath(userCertDir + QLatin1Char('*'), QSsl::Pem,
+                                                                   QRegExp::Wildcard);
+    for (const QSslCertificate &cert : userList) {
         ret += KSslCaCertificate(cert, KSslCaCertificate::UserStore, false);
     }
 
-    KConfig config(QLatin1String("ksslcablacklist"), KConfig::SimpleConfig);
+    KConfig config(QStringLiteral("ksslcablacklist"), KConfig::SimpleConfig);
     KConfigGroup group = config.group("Blacklist of CA Certificates");
-    for (int i = 0; i < ret.size(); i++) {
-        if (group.hasKey(ret[i].certHash.constData())) {
-            ret[i].isBlacklisted = true;
+    for (KSslCaCertificate &cert : ret) {
+        if (group.hasKey(cert.certHash.constData())) {
+            cert.isBlacklisted = true;
             //qDebug() << "is blacklisted";
         }
     }
@@ -402,7 +432,7 @@ bool KSslCertificateManagerPrivate::setCertificateBlacklisted(const QByteArray &
         bool isBlacklisted)
 {
     //qDebug() << Q_FUNC_INFO << isBlacklisted;
-    KConfig config(QLatin1String("ksslcablacklist"), KConfig::SimpleConfig);
+    KConfig config(QStringLiteral("ksslcablacklist"), KConfig::SimpleConfig);
     KConfigGroup group = config.group("Blacklist of CA Certificates");
     if (isBlacklisted) {
         // TODO check against certificate list ?
@@ -472,18 +502,34 @@ QList<QSslCertificate> KSslCertificateManager::caCertificates() const
 }
 
 //static
-QList<KSslError> KSslCertificateManager::nonIgnorableErrors(const QList<KSslError> &/*e*/)
+QList<KSslError> KSslCertificateManager::nonIgnorableErrors(const QList<KSslError> &errors)
 {
     QList<KSslError> ret;
-    // ### add filtering here...
+    // errors not handled in KSSLD
+    std::copy_if(errors.begin(), errors.end(), std::back_inserter(ret), [](const KSslError &e) {
+        return e.error() == KSslError::NoPeerCertificate || e.error() == KSslError::PathLengthExceeded;
+    });
     return ret;
 }
 
 //static
-QList<KSslError::Error> KSslCertificateManager::nonIgnorableErrors(const QList<KSslError::Error> &/*e*/)
+QList<KSslError::Error> KSslCertificateManager::nonIgnorableErrors(const QList<KSslError::Error> &errors)
 {
     QList<KSslError::Error> ret;
-    // ### add filtering here...
+    // errors not handled in KSSLD
+    std::copy_if(errors.begin(), errors.end(), std::back_inserter(ret), [](const KSslError::Error &e) {
+        return e == KSslError::NoPeerCertificate || e == KSslError::PathLengthExceeded;
+    });
+    return ret;
+}
+
+QList<QSslError> KSslCertificateManager::nonIgnorableErrors(const QList<QSslError> &errors)
+{
+    QList<QSslError> ret;
+    // errors not handled in KSSLD
+    std::copy_if(errors.begin(), errors.end(), std::back_inserter(ret), [](const QSslError &e) {
+        return e.error() == QSslError::NoPeerCertificate || e.error() == QSslError::PathLengthExceeded || e.error() == QSslError::NoSslSupport;
+    });
     return ret;
 }
 
