@@ -34,11 +34,19 @@ public:
     MkpathJobPrivate(const QUrl &url, const QUrl &baseUrl, JobFlags flags)
         : JobPrivate(),
           m_url(url),
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
           m_pathComponents(url.path().split(QLatin1Char('/'), QString::SkipEmptyParts)),
+#else
+          m_pathComponents(url.path().split(QLatin1Char('/'), Qt::SkipEmptyParts)),
+#endif
           m_pathIterator(),
           m_flags(flags)
     {
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
         const QStringList basePathComponents = baseUrl.path().split(QLatin1Char('/'), QString::SkipEmptyParts);
+#else
+        const QStringList basePathComponents = baseUrl.path().split(QLatin1Char('/'), Qt::SkipEmptyParts);
+#endif
         m_url.setPath(QStringLiteral("/"));
         int i = 0;
         for (; i < basePathComponents.count() && i < m_pathComponents.count(); ++i) {
