@@ -141,19 +141,6 @@ public:
     QString m_warningMessage;
     int m_privilegeOperationStatus;
 
-    PrivilegeOperationStatus askConfirmation()
-    {
-        int status = q->messageBox(SlaveBase::WarningContinueCancel, m_warningMessage, m_warningCaption, QStringLiteral("Continue"), QStringLiteral("Cancel"));
-        switch (status) {
-        case SlaveBase::Continue:
-            return OperationAllowed;
-        case SlaveBase::Cancel:
-            return OperationCanceled;
-        default:
-            return OperationNotAllowed;
-        }
-    }
-
     void updateTempAuthStatus()
     {
 #ifdef Q_OS_UNIX
@@ -334,7 +321,7 @@ void SlaveBase::dispatchLoop()
 
         int ms = -1;
         if (d->nextTimeout.isValid()) {
-            ms = qMax<int>(d->nextTimeout.elapsed() - d->nextTimeoutMsecs, 1);
+            ms = qMax<int>(d->nextTimeoutMsecs - d->nextTimeout.elapsed(), 1);
         }
 
         int ret = -1;
@@ -666,6 +653,12 @@ void SlaveBase::position(KIO::filesize_t _pos)
     send(INF_POSITION, data);
 }
 
+void SlaveBase::truncated(KIO::filesize_t _length)
+{
+    KIO_DATA << KIO_FILESIZE_T(_length);
+    send(INF_TRUNCATED, data);
+}
+
 void SlaveBase::processedPercent(float /* percent */)
 {
     //qDebug() << "STUB";
@@ -1405,6 +1398,13 @@ void SlaveBase::dispatchOpenCommand(int command, const QByteArray &data)
         seek(offset);
         break;
     }
+    case CMD_TRUNCATE: {
+        KIO::filesize_t length;
+        stream >> length;
+        void *data = static_cast<void *>(&length);
+        virtual_hook(Truncate, data);
+        break;
+    }
     case CMD_NONE:
         break;
     case CMD_CLOSE:
@@ -1499,6 +1499,9 @@ void SlaveBase::virtual_hook(int id, void *data)
     case GetFileSystemFreeSpace: {
         error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_FILESYSTEMFREESPACE));
     } break;
+    case Truncate: {
+        error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_TRUNCATE));
+    } break;
     }
 }
 
@@ -1535,8 +1538,9 @@ int SlaveBase::waitForHostInfo(QHostInfo &info)
     return result;
 }
 
-PrivilegeOperationStatus SlaveBase::requestPrivilegeOperation()
+PrivilegeOperationStatus SlaveBase::requestPrivilegeOperation(const QString &operationDetails)
 {
+
     if (d->m_privilegeOperationStatus == OperationNotAllowed) {
         QByteArray buffer;
         send(MSG_PRIVILEGE_EXEC);
@@ -1548,7 +1552,13 @@ PrivilegeOperationStatus SlaveBase::requestPrivilegeOperation()
     if (metaData(QStringLiteral("UnitTesting")) != QLatin1String("true") &&
             d->m_privilegeOperationStatus == OperationAllowed &&
             !d->m_confirmationAsked) {
-        d->m_privilegeOperationStatus = d->askConfirmation();
+        //KF6 TODO Remove. We don't want to pass details as meta-data. Pass it as a parameter in messageBox().
+        setMetaData(QStringLiteral("privilege_conf_details"), operationDetails);
+        sendMetaData();
+
+        int result = messageBox(d->m_warningMessage, WarningContinueCancelDetailed,
+                                d->m_warningCaption, QString(), QString(), QString());
+        d->m_privilegeOperationStatus = result == Continue ? OperationAllowed : OperationCanceled;
         d->m_confirmationAsked = true;
     }
 
@@ -1559,3 +1569,10 @@ void SlaveBase::addTemporaryAuthorization(const QString &action)
 {
     d->m_tempAuths.insert(action);
 }
+
+#if KIOCORE_BUILD_DEPRECATED_SINCE(5, 66)
+PrivilegeOperationStatus SlaveBase::requestPrivilegeOperation()
+{
+        return KIO::OperationNotAllowed;
+}
+#endif
