@@ -20,7 +20,7 @@
 
 #include "slaveconfig.h"
 
-#include <QtCore/QHash>
+#include <QHash>
 #include <QThreadStorage>
 
 #include <kconfig.h>
@@ -42,6 +42,9 @@ public:
         delete configFile;
     }
 
+    SlaveConfigProtocol(const SlaveConfigProtocol &) = delete;
+    SlaveConfigProtocol &operator=(const SlaveConfigProtocol &) = delete;
+
 public:
     MetaData global;
     QHash<QString, MetaData> host;
@@ -105,7 +108,7 @@ void SlaveConfigPrivate::readConfigProtocolHost(const QString &, SlaveConfigProt
     // Break host into domains
     QString domain = host;
 
-    if (!domain.contains('.')) {
+    if (!domain.contains(QLatin1Char('.'))) {
         // Host without domain.
         if (scp->configFile->hasGroup("<local>")) {
             readConfig(scp->configFile, QStringLiteral("<local>"), &metaData);
@@ -115,7 +118,7 @@ void SlaveConfigPrivate::readConfigProtocolHost(const QString &, SlaveConfigProt
 
     int pos = 0;
     do {
-        pos = host.lastIndexOf('.', pos - 1);
+        pos = host.lastIndexOf(QLatin1Char('.'), pos - 1);
 
         if (pos < 0) {
             domain = host;
@@ -217,7 +220,7 @@ MetaData SlaveConfig::configData(const QString &protocol, const QString &host)
 
 QString SlaveConfig::configData(const QString &protocol, const QString &host, const QString &key)
 {
-    return configData(protocol, host)[key];
+    return configData(protocol, host).value(key);
 }
 
 void SlaveConfig::reset()
