@@ -44,6 +44,18 @@ KIOCORE_EXPORT QDataStream &operator>> (QDataStream &s, KIO::UDSEntry &a);
  */
 KIOCORE_EXPORT QDebug operator<<(QDebug stream, const KIO::UDSEntry &entry);
 
+/**
+ * Returns true if the entry contains the same data as the other
+ * @since 5.63
+ */
+KIOCORE_EXPORT bool operator== (const KIO::UDSEntry &entry, const KIO::UDSEntry &other);
+
+/**
+ * Returns true if the entry does not contain the same data as the other
+ * @since 5.63
+ */
+KIOCORE_EXPORT bool operator!= (const KIO::UDSEntry &entry, const KIO::UDSEntry &other);
+
 namespace KIO
 {
 class UDSEntryPrivate;
@@ -139,21 +151,27 @@ public:
      */
     void reserve(int size);
 
+#if KIOCORE_ENABLE_DEPRECATED_SINCE(5, 48)
     /**
      * insert field with string value
      * @param field numeric field id
      * @param value to set
      * @deprecated since 5.48 in favor of fastInsert or replace
      */
-    KIOCORE_DEPRECATED void insert(uint field, const QString &value);
+    KIOCORE_DEPRECATED_VERSION(5, 48, "Use UDSEntry::fastInsert(uint, const QString &) or UDSEntry::replace(uint, const QString &)")
+    void insert(uint field, const QString &value);
+#endif
 
+#if KIOCORE_ENABLE_DEPRECATED_SINCE(5, 48)
     /**
      * insert field with numeric value
      * @param field numeric field id
      * @param l value to set
      * @deprecated since 5.48 in favor of fastInsert or replace
      */
-    KIOCORE_DEPRECATED void insert(uint field, long long l);
+    KIOCORE_DEPRECATED_VERSION(5, 48, "Use UDSEntry::fastInsert(uint, long long) or UDSEntry::replace(uint, long long)")
+    void insert(uint field, long long l);
+#endif
 
     /**
      * insert field with string value, it will assert if the field is already inserted. In that case, use replace() instead.
@@ -183,13 +201,14 @@ public:
      */
     bool contains(uint field) const;
 
+#if KIOCORE_ENABLE_DEPRECATED_SINCE(5, 8)
     /**
      * List all fields.
      * @return all fields.
      * @deprecated since 5.8. Use fields() instead.
      */
-#ifndef KIOCORE_NO_DEPRECATED
-    KIOCORE_DEPRECATED QList<uint> listFields() const;
+    KIOCORE_DEPRECATED_VERSION(5, 8, "Use UDSEntry::fields()")
+    QList<uint> listFields() const;
 #endif
 
     /**
@@ -337,6 +356,7 @@ private:
     friend KIOCORE_EXPORT QDataStream& ::operator<< (QDataStream &s, const KIO::UDSEntry &a);
     friend KIOCORE_EXPORT QDataStream& ::operator>> (QDataStream &s, KIO::UDSEntry &a);
     friend KIOCORE_EXPORT QDebug (::operator<<) (QDebug stream, const KIO::UDSEntry &entry);
+
 public:
     /**
      * Replace or insert field with string value
