@@ -18,9 +18,6 @@
 
 */
 
-// TODO: remove me
-#undef QT_NO_CAST_FROM_ASCII
-
 #include "kfileplacesmodel.h"
 #include "kfileplacesitem_p.h"
 
@@ -167,7 +164,7 @@ public:
           bookmarkManager(nullptr),
           fileIndexingEnabled(isFileIndexingEnabled()),
           tags(),
-          tagsLister(new KCoreDirLister())
+          tagsLister(new KCoreDirLister(q))
     {
         if (KProtocolInfo::isKnownProtocol(QStringLiteral("tags"))) {
             connect(tagsLister, &KCoreDirLister::itemsAdded, q, [this](const QUrl&, const KFileItemList& items) {
@@ -184,7 +181,9 @@ public:
                     }
 
                     if (!existingBookmarks.contains(QUrl(tagsUrlBase))) {
-                        KBookmark alltags = KFilePlacesItem::createSystemBookmark(bookmarkManager, QStringLiteral("All tags"), i18n("All tags"), QUrl(tagsUrlBase), QStringLiteral("tag"));
+                        KBookmark alltags = KFilePlacesItem::createSystemBookmark(bookmarkManager, "All tags"
+                                                                                  , i18n("All tags").toUtf8().data()
+                                                                                  , QUrl(tagsUrlBase), QStringLiteral("tag"));
                     }
                 }
 
@@ -276,17 +275,11 @@ KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QO
     };
 
     if (root.first().isNull() || !QFile::exists(file)) {
-
-        // NOTE: The context for these I18N_NOOP2 calls has to be "KFile System Bookmarks".
+        // NOTE: The context for these I18NC_NOOP calls has to be "KFile System Bookmarks".
         // The real i18nc call is made later, with this context, so the two must match.
-        //
-        // createSystemBookmark actually does nothing with its third argument,
-        // but we have to give it something so the I18N_NOOP2 calls stay here for now.
-        //
-        // (coles, 13th May 2009)
-
+        // createSystemBookmark actually does nothing with its second argument, the context,
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              QStringLiteral("Home"), I18N_NOOP2("KFile System Bookmarks", "Home"),
+                                              I18NC_NOOP("KFile System Bookmarks", "Home"),
                                               QUrl::fromLocalFile(QDir::homePath()), QStringLiteral("user-home"));
 
         // Some distros may not create various standard XDG folders by default
@@ -294,28 +287,28 @@ KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QO
         const QString desktopFolder = QStandardPaths::writableLocation(QStandardPaths::DesktopLocation);
         if (QDir(desktopFolder).exists()) {
             KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                                QStringLiteral("Desktop"), I18N_NOOP2("KFile System Bookmarks", "Desktop"),
+                                                I18NC_NOOP("KFile System Bookmarks", "Desktop"),
                                                 QUrl::fromLocalFile(desktopFolder), QStringLiteral("user-desktop"));
         }
         const QString documentsFolder = QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation);
         if (QDir(documentsFolder).exists()) {
             KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                                QStringLiteral("Documents"), I18N_NOOP2("KFile System Bookmarks", "Documents"),
+                                                I18NC_NOOP("KFile System Bookmarks", "Documents"),
                                                 QUrl::fromLocalFile(documentsFolder), QStringLiteral("folder-documents"));
         }
         const QString downloadFolder = QStandardPaths::writableLocation(QStandardPaths::DownloadLocation);
         if (QDir(downloadFolder).exists()) {
             KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                                QStringLiteral("Downloads"), I18N_NOOP2("KFile System Bookmarks", "Downloads"),
+                                                I18NC_NOOP("KFile System Bookmarks", "Downloads"),
                                                 QUrl::fromLocalFile(downloadFolder), QStringLiteral("folder-downloads"));
         }
 
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              QStringLiteral("Network"), I18N_NOOP2("KFile System Bookmarks", "Network"),
+                                              I18NC_NOOP("KFile System Bookmarks", "Network"),
                                               QUrl(QStringLiteral("remote:/")), QStringLiteral("folder-network"));
 
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              QStringLiteral("Trash"), I18N_NOOP2("KFile System Bookmarks", "Trash"),
+                                              I18NC_NOOP("KFile System Bookmarks", "Trash"),
                                               QUrl(QStringLiteral("trash:/")), QStringLiteral("user-trash"));
 
         setDefaultMetadataItemForGroup(PlacesType);
@@ -337,17 +330,17 @@ KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QO
         root.setMetaDataItem(QStringLiteral("withRecentlyUsed"), QStringLiteral("true"));
 
         KBookmark recentFilesBookmark = KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                            QStringLiteral("Recent Files"), I18N_NOOP2("KFile System Bookmarks", "Recent Files"),
+                                            I18NC_NOOP("KFile System Bookmarks", "Recent Files"),
                                             QUrl(QStringLiteral("recentlyused:/files")), QStringLiteral("document-open-recent"));
 
         KBookmark recentDirectoriesBookmark = KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                            QStringLiteral("Recent Locations"), I18N_NOOP2("KFile System Bookmarks", "Recent Locations"),
+                                            I18NC_NOOP("KFile System Bookmarks", "Recent Locations"),
                                             QUrl(QStringLiteral("recentlyused:/locations")), QStringLiteral("folder-open-recent"));
 
         setDefaultMetadataItemForGroup(RecentlySavedType);
 
         // Move The recently used bookmarks below the trash, making it the first element in the Recent group
-        KBookmark trashBookmark = bookmarkForUrl(QUrl("trash:/"));
+        KBookmark trashBookmark = bookmarkForUrl(QUrl(QStringLiteral("trash:/")));
         if (!trashBookmark.isNull()) {
             root.moveBookmark(recentFilesBookmark, trashBookmark);
             root.moveBookmark(recentDirectoriesBookmark, recentFilesBookmark);
@@ -361,23 +354,28 @@ KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QO
         root.metaDataItem(QStringLiteral("withBaloo")) != QLatin1String("true")) {
 
         root.setMetaDataItem(QStringLiteral("withBaloo"), QStringLiteral("true"));
+
+        // don't add by default "Modified Today" and "Modified Yesterday" when recentlyused:/ is present
+        if (root.metaDataItem(QStringLiteral("withRecentlyUsed")) != QLatin1String("true")) {
+            KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
+                                                  I18NC_NOOP("KFile System Bookmarks", "Modified Today"),
+                                                  QUrl(QStringLiteral("timeline:/today")),  QStringLiteral("go-jump-today"));
+            KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
+                                                  I18NC_NOOP("KFile System Bookmarks", "Modified Yesterday"),
+                                                  QUrl(QStringLiteral("timeline:/yesterday")),  QStringLiteral("view-calendar-day"));
+        }
+
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              QStringLiteral("Modified Today"), I18N_NOOP2("KFile System Bookmarks", "Today"),
-                                              QUrl(QStringLiteral("timeline:/today")),  QStringLiteral("go-jump-today"));
-        KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              QStringLiteral("Modified Yesterday"), I18N_NOOP2("KFile System Bookmarks", "Yesterday"),
-                                              QUrl(QStringLiteral("timeline:/yesterday")),  QStringLiteral("view-calendar-day"));
-        KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              QStringLiteral("Documents"), I18N_NOOP2("KFile System Bookmarks", "Documents"),
+                                              I18NC_NOOP("KFile System Bookmarks", "Documents"),
                                              QUrl(QStringLiteral("search:/documents")),  QStringLiteral("folder-text"));
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              QStringLiteral("Images"), I18N_NOOP2("KFile System Bookmarks", "Images"),
+                                              I18NC_NOOP("KFile System Bookmarks", "Images"),
                                               QUrl(QStringLiteral("search:/images")),  QStringLiteral("folder-images"));
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              QStringLiteral("Audio"), I18N_NOOP2("KFile System Bookmarks", "Audio"),
+                                              I18NC_NOOP("KFile System Bookmarks", "Audio"),
                                               QUrl(QStringLiteral("search:/audio")),  QStringLiteral("folder-sound"));
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              QStringLiteral("Videos"), I18N_NOOP2("KFile System Bookmarks", "Videos"),
+                                              I18NC_NOOP("KFile System Bookmarks", "Videos"),
                                               QUrl(QStringLiteral("search:/videos")),  QStringLiteral("folder-videos"));
 
         setDefaultMetadataItemForGroup(SearchForType);
@@ -761,15 +759,19 @@ QList<KFilePlacesItem *> KFilePlacesModel::Private::loadBookmarkList()
                 bool isSupportedUrl = isBalooUrl(url) ? fileIndexingEnabled : true;
                 bool isSupportedScheme = supportedSchemes.isEmpty() || supportedSchemes.contains(url.scheme());
 
-                if (isSupportedScheme && ((isSupportedUrl && udi.isEmpty() && allowedHere) || deviceAvailable)) {
-
-                    KFilePlacesItem *item;
-                    if (deviceAvailable) {
-                        item = new KFilePlacesItem(bookmarkManager, bookmark.address(), udi);
-                        // TODO: Update bookmark internal element
-                    } else {
-                        item = new KFilePlacesItem(bookmarkManager, bookmark.address());
+                KFilePlacesItem *item = nullptr;
+                if (deviceAvailable) {
+                    item = new KFilePlacesItem(bookmarkManager, bookmark.address(), udi);
+                    if (!item->hasSupportedScheme(supportedSchemes)) {
+                        delete item;
+                        item = nullptr;
                     }
+                } else if (isSupportedScheme && isSupportedUrl && udi.isEmpty() && allowedHere) {
+                    // TODO: Update bookmark internal element
+                    item = new KFilePlacesItem(bookmarkManager, bookmark.address());
+                }
+
+                if (item) {
                     connect(item, SIGNAL(itemChanged(QString)),
                             q, SLOT(_k_itemChanged(QString)));
 
