@@ -37,7 +37,6 @@
 #include <QAbstractItemView>
 #include <QAbstractProxyModel>
 #include <QClipboard>
-#include <QColor>
 #include <QHash>
 #include <QList>
 #include <QListView>
@@ -46,7 +45,6 @@
 #include <QPixmap>
 #include <QPointer>
 #include <QTimer>
-#include <QScrollBar>
 #include <QIcon>
 
 #if HAVE_X11 && HAVE_XRENDER
@@ -490,15 +488,15 @@ KFilePreviewGenerator::Private::Private(KFilePreviewGenerator *parent,
     connect(m_changedItemsTimer, SIGNAL(timeout()),
             q, SLOT(delayedIconUpdate()));
 
-    KConfigGroup globalConfig(KSharedConfig::openConfig(), "PreviewSettings");
-    m_enabledPlugins = globalConfig.readEntry("Plugins", QStringList()
-                       << QStringLiteral("directorythumbnail")
-                       << QStringLiteral("imagethumbnail")
-                       << QStringLiteral("jpegthumbnail"));
+    KConfigGroup globalConfig(KSharedConfig::openConfig(QStringLiteral("dolphinrc")), "PreviewSettings");
+    m_enabledPlugins = globalConfig.readEntry("Plugins", QStringList{
+                                                              QStringLiteral("directorythumbnail"),
+                                                              QStringLiteral("imagethumbnail"),
+                                                              QStringLiteral("jpegthumbnail")});
 
     // Compatibility update: in 4.7, jpegrotatedthumbnail was merged into (or
     // replaced with?) jpegthumbnail
-    if (m_enabledPlugins.contains(QStringLiteral("jpegrotatedthumbnail"))) {
+    if (m_enabledPlugins.contains(QLatin1String("jpegrotatedthumbnail"))) {
         m_enabledPlugins.removeAll(QStringLiteral("jpegrotatedthumbnail"));
         m_enabledPlugins.append(QStringLiteral("jpegthumbnail"));
         globalConfig.writeEntry("Plugins", m_enabledPlugins);
@@ -546,7 +544,8 @@ void KFilePreviewGenerator::Private::updateIcons(const KFileItemList &items)
     KFileItemList orderedItems = items;
     orderItems(orderedItems);
 
-    foreach (const KFileItem &item, orderedItems) {
+    m_pendingItems.reserve(m_pendingItems.size() + orderedItems.size());
+    for (const KFileItem &item : qAsConst(orderedItems)) {
         m_pendingItems.append(item);
     }
 
@@ -637,7 +636,8 @@ void KFilePreviewGenerator::Private::addToPreviewQueue(const KFileItem &item, co
 
     const QUrl itemParentDir = item.url().adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash);
 
-    foreach (const QUrl &dir, dirModel->dirLister()->directories()) {
+    const QList<QUrl> dirs = dirModel->dirLister()->directories();
+    for (const QUrl &dir : dirs) {
         if (dir == itemParentDir || dir.path().isEmpty()) {
             isOldPreview = false;
             break;
@@ -651,7 +651,7 @@ void KFilePreviewGenerator::Private::addToPreviewQueue(const KFileItem &item, co
 
     const QString mimeType = item.mimetype();
     const int slashIndex = mimeType.indexOf(QLatin1Char('/'));
-    const QString mimeTypeGroup = mimeType.left(slashIndex);
+    const QStringRef mimeTypeGroup = mimeType.leftRef(slashIndex);
     if ((mimeTypeGroup != QLatin1String("image")) || !applyImageFrame(icon)) {
         limitToSize(icon, m_viewAdapter->iconSize());
     }
@@ -683,7 +683,7 @@ void KFilePreviewGenerator::Private::slotPreviewJobFinished(KJob *job)
     m_previewJobs.removeAt(index);
 
     if (m_previewJobs.isEmpty()) {
-        foreach (const KFileItem &item, m_pendingItems) {
+        for (const KFileItem &item : qAsConst(m_pendingItems)) {
             if (item.isMimeTypeKnown()) {
                 m_resolvedMimeTypes.append(item);
             }
@@ -712,7 +712,8 @@ void KFilePreviewGenerator::Private::updateCutItems()
     KFileItemList items;
     KDirLister *dirLister = dirModel->dirLister();
     const QList<QUrl> dirs = dirLister->directories();
-    foreach (const QUrl &url, dirs) {
+    items.reserve(dirs.size());
+    for (const QUrl &url : dirs) {
         items << dirLister->itemsForDir(url);
     }
     applyCutItemEffect(items);
@@ -729,8 +730,8 @@ void KFilePreviewGenerator::Private::clearCutItemsCache()
     KFileItemList previews;
     // Reset the icons of all items that are stored in the cache
     // to use their default MIME type icon.
-    foreach (const QUrl &url, m_cutItemsCache.keys()) {
-        const QModelIndex index = dirModel->indexForUrl(url);
+    for (auto it = m_cutItemsCache.cbegin(); it != m_cutItemsCache.cend(); ++it) {
+        const QModelIndex index = dirModel->indexForUrl(it.key());
         if (index.isValid()) {
             dirModel->setData(index, QIcon(), Qt::DecorationRole);
             if (m_previewShown) {
@@ -740,7 +741,7 @@ void KFilePreviewGenerator::Private::clearCutItemsCache()
     }
     m_cutItemsCache.clear();
 
-    if (previews.size() > 0) {
+    if (!previews.isEmpty()) {
         // assure that the previews gets restored
         Q_ASSERT(m_previewShown);
         orderItems(previews);
@@ -762,7 +763,7 @@ void KFilePreviewGenerator::Private::dispatchIconUpdateQueue()
 
         if (m_previewShown) {
             // dispatch preview queue
-            foreach (const ItemInfo &preview, m_previews) {
+            for (const ItemInfo &preview : qAsConst(m_previews)) {
                 const QModelIndex idx = dirModel->indexForUrl(preview.url);
                 if (idx.isValid() && (idx.column() == 0)) {
                     dirModel->setData(idx, QIcon(preview.pixmap), Qt::DecorationRole);
@@ -772,7 +773,7 @@ void KFilePreviewGenerator::Private::dispatchIconUpdateQueue()
         }
 
         // dispatch mime type queue
-        foreach (const KFileItem &item, m_resolvedMimeTypes) {
+        for (const KFileItem &item : qAsConst(m_resolvedMimeTypes)) {
             const QModelIndex idx = dirModel->indexForItem(item);
             dirModel->itemChanged(idx);
         }
@@ -795,7 +796,7 @@ void KFilePreviewGenerator::Private::dispatchIconUpdateQueue()
 void KFilePreviewGenerator::Private::pauseIconUpdates()
 {
     m_iconUpdatesPaused = true;
-    foreach (KJob *job, m_previewJobs) {
+    for (KJob *job : qAsConst(m_previewJobs)) {
         Q_ASSERT(job != nullptr);
         job->suspend();
     }
@@ -812,7 +813,7 @@ void KFilePreviewGenerator::Private::resumeIconUpdates()
     // queue is usually equal. So even when having a lot of elements the
     // nested loop is no performance bottle neck, as the inner loop is only
     // entered once in most cases.
-    foreach (const KFileItem &item, m_dispatchedItems) {
+    for (const KFileItem &item : qAsConst(m_dispatchedItems)) {
         KFileItemList::iterator begin = m_pendingItems.begin();
         KFileItemList::iterator end   = m_pendingItems.end();
         for (KFileItemList::iterator it = begin; it != end; ++it) {
@@ -916,7 +917,7 @@ void KFilePreviewGenerator::Private::applyCutItemEffect(const KFileItemList &ite
 
     DataChangeObtainer obt(this);
     KIconEffect *iconEffect = KIconLoader::global()->iconEffect();
-    foreach (const KFileItem &item, items) {
+    for (const KFileItem &item : items) {
         if (cutUrls.contains(item.url())) {
             const QModelIndex index = dirModel->indexForItem(item);
             const QVariant value = dirModel->data(index, Qt::DecorationRole);
@@ -940,16 +941,9 @@ void KFilePreviewGenerator::Private::applyCutItemEffect(const KFileItemList &ite
 bool KFilePreviewGenerator::Private::applyImageFrame(QPixmap &icon)
 {
     const QSize maxSize = m_viewAdapter->iconSize();
-
-    // The original size of an image is not exported by the thumbnail mechanism.
-    // Still it would be helpful to not apply an image frame for e. g. icons that
-    // fit into the given boundaries:
-    const bool isIconCandidate = (icon.width() == icon.height()) &&
-                                 ((icon.width() & 0x7) == 0);
-
     const bool applyFrame = (maxSize.width()  > KIconLoader::SizeSmallMedium) &&
                             (maxSize.height() > KIconLoader::SizeSmallMedium) &&
-                            !isIconCandidate;
+                            !icon.hasAlpha();
     if (!applyFrame) {
         // the maximum size or the image itself is too small for a frame
         return false;
@@ -1017,14 +1011,14 @@ void KFilePreviewGenerator::Private::limitToSize(QPixmap &icon, const QSize &max
             icon = icon.scaled(maxSize, Qt::KeepAspectRatio, Qt::FastTransformation);
         }
 #else
-        icon = icon.scaled(maxSize, Qt::KeepAspectRatio, Qt::FastTransformation);
+        icon = icon.scaled(maxSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
 #endif
     }
 }
 
 void KFilePreviewGenerator::Private::createPreviews(const KFileItemList &items)
 {
-    if (items.count() == 0) {
+    if (items.isEmpty()) {
         return;
     }
 
@@ -1039,11 +1033,10 @@ void KFilePreviewGenerator::Private::createPreviews(const KFileItemList &items)
     KFileItemList imageItems;
     KFileItemList otherItems;
     QString mimeType;
-    QString mimeTypeGroup;
-    foreach (const KFileItem &item, items) {
+    for (const KFileItem &item : items) {
         mimeType = item.mimetype();
         const int slashIndex = mimeType.indexOf(QLatin1Char('/'));
-        mimeTypeGroup = mimeType.left(slashIndex);
+        const QStringRef mimeTypeGroup = mimeType.leftRef(slashIndex);
         if (mimeTypeGroup == QLatin1String("image")) {
             imageItems.append(item);
         } else {
@@ -1061,7 +1054,7 @@ void KFilePreviewGenerator::Private::createPreviews(const KFileItemList &items)
 
 void KFilePreviewGenerator::Private::startPreviewJob(const KFileItemList &items, int width, int height)
 {
-    if (items.count() > 0) {
+    if (!items.isEmpty()) {
         KIO::PreviewJob *job = KIO::filePreview(items, QSize(width, height), &m_enabledPlugins);
 
         // Set the sequence index to the target. We only need to check if items.count() == 1,
@@ -1083,7 +1076,7 @@ void KFilePreviewGenerator::Private::startPreviewJob(const KFileItemList &items,
 
 void KFilePreviewGenerator::Private::killPreviewJobs()
 {
-    foreach (KJob *job, m_previewJobs) {
+    for (KJob *job : qAsConst(m_previewJobs)) {
         Q_ASSERT(job != nullptr);
         job->kill();
     }
@@ -1103,7 +1096,7 @@ void KFilePreviewGenerator::Private::orderItems(KFileItemList &items)
     }
 
     // Order the items in a way that the preview for the visible items
-    // is generated first, as this improves the feeled performance a lot.
+    // is generated first, as this improves the felt performance a lot.
     const bool hasProxy = (m_proxyModel != nullptr);
     const int itemCount = items.count();
     const QRect visibleArea = m_viewAdapter->visibleArea();
