@@ -60,9 +60,9 @@ KUrlNavigatorButton::KUrlNavigatorButton(const QUrl &url, QWidget *parent) :
     m_openSubDirsTimer = new QTimer(this);
     m_openSubDirsTimer->setSingleShot(true);
     m_openSubDirsTimer->setInterval(300);
-    connect(m_openSubDirsTimer, SIGNAL(timeout()), this, SLOT(startSubDirsJob()));
+    connect(m_openSubDirsTimer, &QTimer::timeout, this, &KUrlNavigatorButton::startSubDirsJob);
 
-    connect(this, SIGNAL(pressed()), this, SLOT(requestSubDirs()));
+    connect(this, &QAbstractButton::pressed, this, &KUrlNavigatorButton::requestSubDirs);
 }
 
 KUrlNavigatorButton::~KUrlNavigatorButton()
@@ -77,11 +77,15 @@ void KUrlNavigatorButton::setUrl(const QUrl &url)
     // URLs leads to problems for protocols where a limit is given for
     // the number of parallel connections. A black-list
     // is given where KIO::stat() should not be used:
-    static const QSet<QString> protocolBlacklist = QSet<QString>()
-        << QStringLiteral("nfs") << QStringLiteral("fish")
-        << QStringLiteral("ftp") << QStringLiteral("sftp")
-        << QStringLiteral("smb") << QStringLiteral("webdav")
-        << QStringLiteral("mtp");
+    static const QSet<QString> protocolBlacklist = QSet<QString>{
+        QStringLiteral("nfs"),
+        QStringLiteral("fish"),
+        QStringLiteral("ftp"),
+        QStringLiteral("sftp"),
+        QStringLiteral("smb"),
+        QStringLiteral("webdav"),
+        QStringLiteral("mtp"),
+    };
 
     const bool startTextResolving = m_url.isValid() && !m_url.isLocalFile()
                                 && !protocolBlacklist.contains(m_url.scheme());
@@ -89,11 +93,11 @@ void KUrlNavigatorButton::setUrl(const QUrl &url)
     if (startTextResolving) {
         m_pendingTextChange = true;
         KIO::StatJob *job = KIO::stat(m_url, KIO::HideProgressInfo);
-        connect(job, SIGNAL(result(KJob*)),
-                this, SLOT(statFinished(KJob*)));
+        connect(job, &KJob::result,
+                this, &KUrlNavigatorButton::statFinished);
         emit startedTextResolving();
     } else {
-        setText(m_url.fileName().replace('&', QLatin1String("&&")));
+        setText(m_url.fileName().replace(QLatin1Char('&'), QLatin1String("&&")));
     }
 }
 
@@ -139,7 +143,7 @@ QSize KUrlNavigatorButton::sizeHint() const
     adjustedFont.setBold(m_subDir.isEmpty());
     // the minimum size is textWidth + arrowWidth() + 2 * BorderWidth; for the
     // preferred size we add the BorderWidth 2 times again for having an uncluttered look
-    const int width = QFontMetrics(adjustedFont).width(plainText()) + arrowWidth() + 4 * BorderWidth;
+    const int width = QFontMetrics(adjustedFont).boundingRect(plainText()).width() + arrowWidth() + 4 * BorderWidth;
     return QSize(width, KUrlNavigatorButtonBase::sizeHint().height());
 }
 
@@ -279,7 +283,7 @@ void KUrlNavigatorButton::keyPressEvent(QKeyEvent *event)
     switch (event->key()) {
     case Qt::Key_Enter:
     case Qt::Key_Return:
-        emit clicked(m_url, Qt::LeftButton);
+        emit clicked(m_url, Qt::LeftButton, event->modifiers());
         break;
     case Qt::Key_Down:
     case Qt::Key_Space:
@@ -362,7 +366,7 @@ void KUrlNavigatorButton::mouseReleaseEvent(QMouseEvent *event)
     if (!isAboveArrow(event->x()) || (event->button() != Qt::LeftButton)) {
         // the mouse has been released above the text area and not
         // above the [>] button
-        emit clicked(m_url, event->button());
+        emit clicked(m_url, event->button(), event->modifiers());
         cancelSubDirsRequest();
     }
     KUrlNavigatorButtonBase::mouseReleaseEvent(event);
@@ -381,8 +385,8 @@ void KUrlNavigatorButton::mouseMoveEvent(QMouseEvent *event)
 
 void KUrlNavigatorButton::wheelEvent(QWheelEvent *event)
 {
-    if (event->orientation() == Qt::Vertical) {
-        m_wheelSteps = event->delta() / 120;
+    if (event->angleDelta().y() != 0) {
+        m_wheelSteps = event->angleDelta().y() / 120;
         m_replaceButton = true;
         startSubDirsJob();
     }
@@ -407,13 +411,13 @@ void KUrlNavigatorButton::startSubDirsJob()
     m_subDirsJob = KIO::listDir(url, KIO::HideProgressInfo, false /*no hidden files*/);
     m_subDirs.clear(); // just to be ++safe
 
-    connect(m_subDirsJob, SIGNAL(entries(KIO::Job*,KIO::UDSEntryList)),
-            this, SLOT(addEntriesToSubDirs(KIO::Job*,KIO::UDSEntryList)));
+    connect(m_subDirsJob, &KIO::ListJob::entries,
+            this, &KUrlNavigatorButton::addEntriesToSubDirs);
 
     if (m_replaceButton) {
-        connect(m_subDirsJob, SIGNAL(result(KJob*)), this, SLOT(replaceButton(KJob*)));
+        connect(m_subDirsJob, &KJob::result, this, &KUrlNavigatorButton::replaceButton);
     } else {
-        connect(m_subDirsJob, SIGNAL(result(KJob*)), this, SLOT(openSubDirsMenu(KJob*)));
+        connect(m_subDirsJob, &KJob::result, this, &KUrlNavigatorButton::openSubDirsMenu);
     }
 }
 
@@ -422,7 +426,7 @@ void KUrlNavigatorButton::addEntriesToSubDirs(KIO::Job *job, const KIO::UDSEntry
     Q_ASSERT(job == m_subDirsJob);
     Q_UNUSED(job);
 
-    foreach (const KIO::UDSEntry &entry, entries) {
+    for (const KIO::UDSEntry &entry : entries) {
         if (entry.isDir()) {
             const QString name = entry.stringValue(KIO::UDSEntry::UDS_NAME);
             QString displayName = entry.stringValue(KIO::UDSEntry::UDS_DISPLAY_NAME);
@@ -441,7 +445,7 @@ void KUrlNavigatorButton::urlsDropped(QAction *action, QDropEvent *event)
     const int result = action->data().toInt();
     QUrl url(m_url);
     url.setPath(concatPaths(url.path(), m_subDirs.at(result).first));
-    urlsDropped(url, event);
+    emit urlsDropped(url, event);
 }
 
 void KUrlNavigatorButton::slotMenuActionClicked(QAction *action, Qt::MouseButton button)
@@ -449,7 +453,7 @@ void KUrlNavigatorButton::slotMenuActionClicked(QAction *action, Qt::MouseButton
     const int result = action->data().toInt();
     QUrl url(m_url);
     url.setPath(concatPaths(url.path(), m_subDirs.at(result).first));
-    emit clicked(url, button);
+    emit clicked(url, button, Qt::NoModifier);
 }
 
 void KUrlNavigatorButton::statFinished(KJob *job)
@@ -522,7 +526,7 @@ void KUrlNavigatorButton::openSubDirsMenu(KJob *job)
     m_subDirsMenu->exec(popupPos);
 
     // If 'this' has been deleted in the menu's nested event loop, we have to return
-    // immediatedely because any access to a member variable might cause a crash.
+    // immediately because any access to a member variable might cause a crash.
     if (!guard) {
         return;
     }
@@ -569,7 +573,7 @@ void KUrlNavigatorButton::replaceButton(KJob *job)
 
     QUrl url(KIO::upUrl(m_url));
     url.setPath(concatPaths(url.path(), m_subDirs[targetIndex].first));
-    emit clicked(url, Qt::LeftButton);
+    emit clicked(url, Qt::LeftButton, Qt::NoModifier);
 
     m_subDirs.clear();
 }
@@ -591,7 +595,7 @@ QString KUrlNavigatorButton::plainText() const
     const int sourceLength = source.length();
 
     QString dest;
-    dest.reserve(sourceLength);
+    dest.resize(sourceLength);
 
     int sourceIndex = 0;
     int destIndex = 0;
@@ -607,6 +611,8 @@ QString KUrlNavigatorButton::plainText() const
         ++destIndex;
     }
 
+    dest.resize(destIndex);
+
     return dest;
 }
 
@@ -639,7 +645,7 @@ bool KUrlNavigatorButton::isTextClipped() const
 
     QFont adjustedFont(font());
     adjustedFont.setBold(m_subDir.isEmpty());
-    return QFontMetrics(adjustedFont).width(plainText()) >= availableWidth;
+    return QFontMetrics(adjustedFont).boundingRect(plainText()).width() >= availableWidth;
 }
 
 void KUrlNavigatorButton::updateMinimumWidth()
@@ -660,10 +666,10 @@ void KUrlNavigatorButton::updateMinimumWidth()
 
 void KUrlNavigatorButton::initMenu(KUrlNavigatorMenu *menu, int startIndex)
 {
-    connect(menu, SIGNAL(mouseButtonClicked(QAction*, Qt::MouseButton)),
-            this, SLOT(slotMenuActionClicked(QAction*, Qt::MouseButton)));
-    connect(menu, SIGNAL(urlsDropped(QAction*,QDropEvent*)),
-            this, SLOT(urlsDropped(QAction*,QDropEvent*)));
+    connect(menu, &KUrlNavigatorMenu::mouseButtonClicked,
+            this, &KUrlNavigatorButton::slotMenuActionClicked);
+    connect(menu, &KUrlNavigatorMenu::urlsDropped,
+            this, QOverload<QAction*,QDropEvent*>::of(&KUrlNavigatorButton::urlsDropped));
 
     menu->setLayoutDirection(Qt::LeftToRight);
 
