@@ -31,6 +31,7 @@
 #include <QDir>
 #include <qplatformdefs.h>
 #include <QStandardPaths>
+#include <QThread>
 
 #include <QDebug>
 #include <kconfiggroup.h>
@@ -78,10 +79,60 @@ static const QString socketPath()
     return QStringLiteral("%1/filehelper%2%3").arg(runtimeDir, KRandom::randomString(6)).arg(getpid());
 }
 
+static QString actionDetails(ActionType actionType, const QVariantList &args)
+{
+    QString action, detail;
+    switch (actionType) {
+    case CHMOD:
+        action = i18n("Change File Permissions");
+        detail = i18n("New Permissions: %1", args[1].toInt());
+        break;
+    case CHOWN:
+        action = i18n("Change File Owner");
+        detail = i18n("New Owner: UID=%1, GID=%2", args[1].toInt(), args[2].toInt());
+        break;
+    case DEL:
+        action = i18n("Remove File");
+        break;
+    case RMDIR:
+        action = i18n("Remove Directory");
+        break;
+    case MKDIR:
+        action = i18n("Create Directory");
+        detail = i18n("Directory Permissions: %1", args[1].toInt());
+        break;
+    case OPEN:
+        action = i18n("Open File");
+        break;
+    case OPENDIR:
+        action = i18n("Open Directory");
+        break;
+    case RENAME:
+        action = i18n("Rename");
+        detail = i18n("New Filename: %1", args[1].toString());
+        break;
+    case SYMLINK:
+        action = i18n("Create Symlink");
+        detail = i18n("Target: %1", args[1].toString());
+        break;
+    case UTIME:
+        action = i18n("Change Timestamp");
+        break;
+    default:
+        action = i18n("Unknown Action");
+        break;
+    }
+
+    const QString metadata = i18n("Action: %1\n"
+                                  "Source: %2\n"
+                                  "%3", action, args[0].toString(), detail);
+    return metadata;
+}
+
 bool FileProtocol::privilegeOperationUnitTestMode()
 {
     return (metaData(QStringLiteral("UnitTesting")) == QLatin1String("true"))
-            && (requestPrivilegeOperation() == KIO::OperationAllowed);
+            && (requestPrivilegeOperation(QStringLiteral("Test Call")) == KIO::OperationAllowed);
 }
 
 PrivilegeOperationReturnValue FileProtocol::tryOpen(QFile &f, const QByteArray &path, int flags, int mode, int errcode)
@@ -261,7 +312,12 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
     bool use_sendfile = buff_src.st_size < 0x7FFFFFFF;
 #endif
     bool existing_dest_delete_attempted = false;
-    while (1) {
+    while (!wasKilled()) {
+
+        if (testMode && dest_file.fileName().contains(QLatin1String("slow"))) {
+            QThread::usleep(500);
+        }
+
 #ifdef USE_SENDFILE
         if (use_sendfile) {
             off_t sf = processed_size;
@@ -348,6 +404,15 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
     src_file.close();
     dest_file.close();
 
+    if (wasKilled()) {
+        qCDebug(KIO_FILE) << "Clean dest file after ioslave was killed:" << dest;
+        if (!QFile::remove(dest)) {  // don't keep partly copied file
+            execWithElevatedPrivilege(DEL, {_dest}, errno);
+        }
+        error(KIO::ERR_USER_CANCELED, dest);
+        return;
+    }
+
     if (dest_file.error() != QFile::NoError) {
         qCWarning(KIO_FILE) << "Error when closing file descriptor[2]:" << dest_file.errorString();
         error(KIO::ERR_CANNOT_WRITE, dest);
@@ -494,7 +559,7 @@ void FileProtocol::listDir(const QUrl &url)
 {
     if (!isLocalFileSameHost(url)) {
         QUrl redir(url);
-        redir.setScheme(config()->readEntry("DefaultRemoteProtocol", "smb"));
+        redir.setScheme(configValue(QStringLiteral("DefaultRemoteProtocol"), QStringLiteral("smb")));
         redirection(redir);
         // qDebug() << "redirecting to " << redir;
         finished();
@@ -879,7 +944,8 @@ PrivilegeOperationReturnValue FileProtocol::execWithElevatedPrivilege(ActionType
         return PrivilegeOperationReturnValue::failure(errcode);
     }
 
-    KIO::PrivilegeOperationStatus opStatus = requestPrivilegeOperation();
+    const QString operationDetails = actionDetails(action, args);
+    KIO::PrivilegeOperationStatus opStatus = requestPrivilegeOperation(operationDetails);
     if (opStatus != KIO::OperationAllowed) {
         if (opStatus == KIO::OperationCanceled) {
             error(KIO::ERR_USER_CANCELED, QString());
