@@ -27,9 +27,9 @@
 
 #include <qt_windows.h>
 
-#include <QtCore/QDir>
-#include <QtCore/QDirIterator>
-#include <QtCore/QFileInfo>
+#include <QDir>
+#include <QDirIterator>
+#include <QFileInfo>
 
 #include <kconfiggroup.h>
 #include <QDebug>
@@ -100,10 +100,10 @@ static UDSEntry createUDSEntryWin(const QFileInfo &fileInfo)
         entry.insert(KIO::UDSEntry::UDS_HIDDEN, true);
     }
 
-    entry.insert(KIO::UDSEntry::UDS_MODIFICATION_TIME, fileInfo.lastModified().toTime_t());
+    entry.insert(KIO::UDSEntry::UDS_MODIFICATION_TIME, fileInfo.lastModified().toSecsSinceEpoch());
     entry.insert(KIO::UDSEntry::UDS_USER, fileInfo.owner());
     entry.insert(KIO::UDSEntry::UDS_GROUP, fileInfo.group());
-    entry.insert(KIO::UDSEntry::UDS_ACCESS_TIME, fileInfo.lastRead().toTime_t());
+    entry.insert(KIO::UDSEntry::UDS_ACCESS_TIME, fileInfo.lastRead().toSecsSinceEpoch());
 
     return entry;
 }
@@ -191,14 +191,18 @@ void FileProtocol::listDir(const QUrl &url)
 
     if (!url.isLocalFile()) {
         QUrl redir(url);
-        redir.setScheme(config()->readEntry("DefaultRemoteProtocol", "smb"));
+        redir.setScheme(configValue(QStringLiteral("DefaultRemoteProtocol"), QStringLiteral("smb")));
         redirection(redir);
         // qDebug() << "redirecting to " << redir;
         finished();
         return;
     }
 
-    const QString path = url.toLocalFile();
+    QString path = url.toLocalFile();
+    // C: means current directory, a concept which makes no sense in a GUI
+    // KCoreDireLister strips trailing slashes, let's put it back again here for C:/
+    if (path.length() == 2 && path.at(1) == QLatin1Char(':'))
+        path += QLatin1Char('/');
     const QFileInfo info(path);
     if (info.isFile()) {
         error(KIO::ERR_IS_FILE, path);
@@ -355,11 +359,7 @@ void FileProtocol::chown(const QUrl &url, const QString &, const QString &)
 void FileProtocol::stat(const QUrl &url)
 {
     if (!url.isLocalFile()) {
-        QUrl redir(url);
-        redir.setScheme(config()->readEntry("DefaultRemoteProtocol", "smb"));
-        redirection(redir);
-        // qDebug() << "redirecting to " << redir;
-        finished();
+        redirect(url);
         return;
     }
 
