@@ -28,13 +28,13 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <cstring>
 #include <stdlib.h>
 
-#include <QtCore/QDateTime>
-#include <QtCore/QDir>
-#include <QtCore/QString>
-#include <QtCore/QTime>
+#include <QDateTime>
+#include <QDir>
+#include <QString>
+#include <QElapsedTimer>
 #include <QDBusConnection>
-#include <QtNetwork/QLocalServer>
-#include <QtNetwork/QLocalSocket>
+#include <QLocalServer>
+#include <QLocalSocket>
 
 #include <QDebug>
 #include <klocalizedstring.h>
@@ -128,7 +128,7 @@ struct CacheFileInfo : MiniCacheFileInfo {
         out << "\n encoded URL      " << url;
         out << "\n mimetype         " << mimeType;
         out << "\nResponse headers follow...\n";
-        Q_FOREACH (const QString &h, responseHeaders) {
+        for (const QString &h : qAsConst(responseHeaders)) {
             out << h << '\n';
         }
     }
@@ -172,9 +172,11 @@ static bool readBinaryHeader(const QByteArray &d, CacheFileInfo *fi)
 
     stream >> fi->useCount;
 
-    stream >> fi->servedDate;
-    stream >> fi->lastModifiedDate;
-    stream >> fi->expireDate;
+    SerializedCacheFileInfo serialized;
+
+    stream >> serialized.servedDate; fi->servedDate.setSecsSinceEpoch(serialized.servedDate);
+    stream >> serialized.lastModifiedDate; fi->lastModifiedDate.setSecsSinceEpoch(serialized.lastModifiedDate);
+    stream >> serialized.expireDate; fi->expireDate.setSecsSinceEpoch(serialized.expireDate);
 
     stream >> fi->bytesCached;
     return true;
@@ -189,14 +191,14 @@ static QString filenameFromUrl(const QByteArray &url)
 
 static QString cacheDir()
 {
-    return QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + "/kio_http";
+    return QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + QLatin1String("/kio_http");
 }
 
 static QString filePath(const QString &baseName)
 {
     QString cacheDirName = cacheDir();
-    if (!cacheDirName.endsWith('/')) {
-        cacheDirName.append('/');
+    if (!cacheDirName.endsWith(QLatin1Char('/'))) {
+        cacheDirName.append(QLatin1Char('/'));
     }
     return cacheDirName + baseName;
 }
@@ -380,7 +382,7 @@ static CacheCleanerCommand readCommand(const QByteArray &cmd, CacheFileInfo *fi)
     return static_cast<CacheCleanerCommand>(ret);
 }
 
-// never istantiated, on-disk format only
+// never instantiated, on-disk format only
 struct ScoreboardEntry {
 // from scoreboard file
     quint8 index[s_hashedUrlBytes];
@@ -433,8 +435,8 @@ public:
             stream.writeRawData(indexData, s_hashedUrlBytes);
 
             stream << it.value().useCount;
-            stream << it.value().lastUsedDate;
-            stream << it.value().sizeOnDisk;
+            stream << it.value().lastUsedDate.toSecsSinceEpoch();
+            stream << qint32(it.value().sizeOnDisk);
         }
     }
 
@@ -530,7 +532,7 @@ public:
         }
         // qDebug() << "we have too many fake/stale entries, cleaning up...";
         QSet<CacheIndex> realFiles;
-        Q_FOREACH (CacheFileInfo *fi, fiList) {
+        for (CacheFileInfo *fi : fiList) {
             realFiles.insert(CacheIndex(fi->baseName));
         }
         QHash<CacheIndex, MiniCacheFileInfo>::Iterator it = m_scoreboard.begin();
@@ -549,8 +551,12 @@ private:
         QDataStream stream(rawData);
         stream >> mcfi->useCount;
         // check those against filesystem
-        stream >> mcfi->lastUsedDate;
-        stream >> mcfi->sizeOnDisk;
+        qint64 lastUsedDate;
+        stream >> lastUsedDate; mcfi->lastUsedDate.setSecsSinceEpoch(lastUsedDate);
+
+        qint32 sizeOnDisk;
+        stream >> sizeOnDisk; mcfi->sizeOnDisk = sizeOnDisk;
+        //qDebug() << basename << "sizeOnDisk" << mcfi->sizeOnDisk;
 
         QFileInfo fileInfo(filePath(basename));
         if (!fileInfo.exists()) {
@@ -597,7 +603,7 @@ static void removeOldFiles()
         QString dirName = QString::fromLatin1(&oldDirs[i], 1);
         // delete files in directory...
         Q_FOREACH (const QString &baseName, QDir(filePath(dirName)).entryList()) {
-            QFile::remove(filePath(dirName + '/' + baseName));
+            QFile::remove(filePath(dirName + QLatin1Char('/') + baseName));
         }
         // delete the (now hopefully empty!) directory itself
         cacheRootDir.rmdir(dirName);
@@ -612,14 +618,14 @@ public:
         : m_totalSizeOnDisk(0)
     {
         // qDebug();
-        m_fileNameList = cacheDir.entryList();
+        m_fileNameList = cacheDir.entryList(QDir::Files);
     }
 
     // Delete some of the files that need to be deleted. Return true when done, false otherwise.
     // This makes interleaved cleaning / serving ioslaves possible.
     bool processSlice(Scoreboard *scoreboard = nullptr)
     {
-        QTime t;
+        QElapsedTimer t;
         t.start();
         // phase one: gather information about cache files
         if (!m_fileNameList.isEmpty()) {
@@ -632,7 +638,7 @@ public:
                 bool nameOk = true;
                 for (int i = 0; i < s_hashedUrlNibbles && nameOk; i++) {
                     QChar c = baseName[i];
-                    nameOk = (c >= '0' && c <= '9') || (c >= 'a' && c <= 'f');
+                    nameOk = (c >= QLatin1Char('0') && c <= QLatin1Char('9')) || (c >= QLatin1Char('a') && c <= QLatin1Char('f'));
                 }
                 if (!nameOk) {
                     continue;
@@ -749,7 +755,7 @@ int main(int argc, char **argv)
             return 1;
         }
 
-        if (!QDBusConnection::sessionBus().registerService(appFullName)) {
+        if (!QDBusConnection::sessionBus().registerService(QString::fromLatin1(appFullName))) {
             fprintf(stderr, "%s: Already running!\n", appName);
             return 0;
         }
@@ -770,17 +776,18 @@ int main(int argc, char **argv)
     removeOldFiles();
 
     if (mode == DeleteCache) {
-        QTime t;
+        QElapsedTimer t;
         t.start();
         cacheDir.refresh();
         //qDebug() << "time to refresh the cacheDir QDir:" << t.elapsed();
         CacheCleaner cleaner(cacheDir);
         while (!cleaner.processSlice()) { }
+        QFile::remove(filePath(QStringLiteral("scoreboard")));
         return 0;
     }
 
     QLocalServer lServer;
-    QString socketFileName = QStandardPaths::writableLocation(QStandardPaths::RuntimeLocation) + QLatin1Char('/') + "kio_http_cache_cleaner";
+    const QString socketFileName = QStandardPaths::writableLocation(QStandardPaths::RuntimeLocation) + QLatin1String("/kio_http_cache_cleaner");
     // we need to create the file by opening the socket, otherwise it won't work
     QFile::remove(socketFileName);
     if (!lServer.listen(socketFileName)) {
@@ -791,20 +798,13 @@ int main(int argc, char **argv)
 
     Scoreboard scoreboard;
     CacheCleaner *cleaner = nullptr;
-    while (true) {
+    while (QDBusConnection::sessionBus().isConnected()) {
         g_currentDate = QDateTime::currentDateTime();
-        if (cleaner) {
-            QCoreApplication::processEvents(QEventLoop::AllEvents, 100);
-        } else {
-            // We will not immediately know when a socket was disconnected. Causes:
-            // - WaitForMoreEvents does not make processEvents() return when a socket disconnects
-            // - WaitForMoreEvents *and* a timeout is not possible.
-            QCoreApplication::processEvents(QEventLoop::WaitForMoreEvents);
-        }
+
         if (!lServer.isListening()) {
             return 1;
         }
-        lServer.waitForNewConnection(1);
+        lServer.waitForNewConnection(100);
 
         while (QLocalSocket *sock = lServer.nextPendingConnection()) {
             sock->waitForConnected();
