@@ -25,10 +25,10 @@
 #ifndef KCOOKIESERVER_H
 #define KCOOKIESERVER_H
 
-#include <QtCore/QStringList>
+#include <QStringList>
 #include <kdedmodule.h>
 #include <QDBusConnection>
-#include <QtDBus/QDBusContext>
+#include <QDBusContext>
 
 class KHttpCookieList;
 class KCookieJar;
@@ -103,10 +103,6 @@ private:
     QString mFilename;
 
 private:
-    virtual int newInstance(QList<QByteArray>)
-    {
-        return 0;
-    }
     bool cookieMatches(const KHttpCookie &, const QString &, const QString &, const QString &, const QString &);
     void putCookie(QStringList &, const KHttpCookie &, const QList<int> &);
     void saveCookieJar();
