@@ -26,7 +26,6 @@
 #include <QDoubleSpinBox>
 #include <QFormLayout>
 #include <QLabel>
-#include <QLayout>
 #include <QListWidget>
 #include <QListWidgetItem>
 
@@ -39,10 +38,8 @@
 #include <kpluginloader.h>
 #include <KFormat>
 #include <QSpinBox>
-#include <QDebug>
 
 K_PLUGIN_FACTORY(KCMTrashConfigFactory, registerPlugin<TrashConfigModule>(QStringLiteral("trash"));)
-K_EXPORT_PLUGIN(KCMTrashConfigFactory("kcmtrash"))
 
 TrashConfigModule::TrashConfigModule(QWidget *parent, const QVariantList &)
     : KCModule( //KCMTrashConfigFactory::componentData(),
@@ -57,22 +54,22 @@ TrashConfigModule::TrashConfigModule(QWidget *parent, const QVariantList &)
 
     useTypeChanged();
 
-    connect(mUseTimeLimit, SIGNAL(toggled(bool)),
-            this, SLOT(changed()));
-    connect(mUseTimeLimit, SIGNAL(toggled(bool)),
-            this, SLOT(useTypeChanged()));
-    connect(mDays, SIGNAL(valueChanged(int)),
-            this, SLOT(changed()));
-    connect(mUseSizeLimit, SIGNAL(toggled(bool)),
-            this, SLOT(changed()));
-    connect(mUseSizeLimit, SIGNAL(toggled(bool)),
-            this, SLOT(useTypeChanged()));
-    connect(mPercent, SIGNAL(valueChanged(double)),
-            this, SLOT(percentChanged(double)));
-    connect(mPercent, SIGNAL(valueChanged(double)),
-            this, SLOT(changed()));
-    connect(mLimitReachedAction, SIGNAL(currentIndexChanged(int)),
-            this, SLOT(changed()));
+    connect(mUseTimeLimit, &QAbstractButton::toggled,
+            this, &TrashConfigModule::markAsChanged);
+    connect(mUseTimeLimit, &QAbstractButton::toggled,
+            this, &TrashConfigModule::useTypeChanged);
+    connect(mDays, QOverload<int>::of(&QSpinBox::valueChanged),
+            this, &TrashConfigModule::markAsChanged);
+    connect(mUseSizeLimit, &QAbstractButton::toggled,
+            this, &TrashConfigModule::markAsChanged);
+    connect(mUseSizeLimit, &QAbstractButton::toggled,
+            this, &TrashConfigModule::useTypeChanged);
+    connect(mPercent, QOverload<double>::of(&QDoubleSpinBox::valueChanged),
+            this, &TrashConfigModule::percentChanged);
+    connect(mPercent, QOverload<double>::of(&QDoubleSpinBox::valueChanged),
+            this, &TrashConfigModule::markAsChanged);
+    connect(mLimitReachedAction, QOverload<int>::of(&QComboBox::currentIndexChanged),
+            this, &TrashConfigModule::markAsChanged);
 
     trashChanged(0);
     trashInitialize = true;
@@ -118,7 +115,7 @@ void TrashConfigModule::percentChanged(double percent)
     double size = ((double)(partitionSize / 100)) * percent;
 
     KFormat format;
-    mSizeLabel->setText("(" + format.formatByteSize(size, 2) + ")");
+    mSizeLabel->setText(QLatin1Char('(') + format.formatByteSize(size, 2) + QLatin1Char(')'));
 }
 
 void TrashConfigModule::trashChanged(QListWidgetItem *item)
@@ -141,8 +138,9 @@ void TrashConfigModule::trashChanged(int value)
     }
 
     mCurrentTrash = map[ value ];
-    if (mConfigMap.contains(mCurrentTrash)) {
-        const ConfigEntry entry = mConfigMap[ mCurrentTrash ];
+    const auto currentTrashIt = mConfigMap.constFind(mCurrentTrash);
+    if (currentTrashIt != mConfigMap.constEnd()) {
+        const ConfigEntry &entry = *currentTrashIt;
         mUseTimeLimit->setChecked(entry.useTimeLimit);
         mDays->setValue(entry.days);
         mUseSizeLimit->setChecked(entry.useSizeLimit);
@@ -164,7 +162,8 @@ void TrashConfigModule::trashChanged(int value)
 void TrashConfigModule::useTypeChanged()
 {
     mDays->setEnabled(mUseTimeLimit->isChecked());
-    mSizeWidget->setEnabled(mUseSizeLimit->isChecked());
+    mPercent->setEnabled(mUseSizeLimit->isChecked());
+    mSizeLabel->setEnabled(mUseSizeLimit->isChecked());
 }
 
 void TrashConfigModule::readConfig()
@@ -174,7 +173,7 @@ void TrashConfigModule::readConfig()
 
     const QStringList groups = config.groupList();
     for (int i = 0; i < groups.count(); ++i) {
-        if (groups[ i ].startsWith('/')) {
+        if (groups[ i ].startsWith(QLatin1Char('/'))) {
             const KConfigGroup group = config.group(groups[ i ]);
 
             ConfigEntry entry;
@@ -195,7 +194,7 @@ void TrashConfigModule::writeConfig()
     // first delete all existing groups
     const QStringList groups = config.groupList();
     for (int i = 0; i < groups.count(); ++i)
-        if (groups[ i ].startsWith('/')) {
+        if (groups[ i ].startsWith(QLatin1Char('/'))) {
             config.deleteGroup(groups[ i ]);
         }
 
@@ -246,16 +245,18 @@ void TrashConfigModule::setupGui()
 
         mountPoints->setCurrentRow(0);
 
-        connect(mountPoints, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)),
-                this, SLOT(trashChanged(QListWidgetItem*)));
+        connect(mountPoints, &QListWidget::currentItemChanged,
+                this, QOverload<QListWidgetItem*>::of(&TrashConfigModule::trashChanged));
     } else {
         mCurrentTrash = map.value(0);
     }
 
-    QHBoxLayout *daysLayout = new QHBoxLayout();
-    layout->addLayout(daysLayout);
+    QFormLayout* formLayout = new QFormLayout();
+    layout->addLayout(formLayout);
 
-    mUseTimeLimit = new QCheckBox(i18n("Delete files older than:"), this);
+    QHBoxLayout *daysLayout = new QHBoxLayout();
+
+    mUseTimeLimit = new QCheckBox(i18n("Delete files older than"), this);
     mUseTimeLimit->setWhatsThis(xi18nc("@info:whatsthis",
                                        "<para>Check this box to allow <emphasis strong='true'>automatic deletion</emphasis> of files that are older than the value specified. "
                                        "Leave this disabled to <emphasis strong='true'>not</emphasis> automatically delete any items after a certain timespan</para>"));
@@ -270,25 +271,19 @@ void TrashConfigModule::setupGui()
                                "Any files older than this will be automatically deleted.</para>"));
     daysLayout->addWidget(mDays);
     daysLayout->addStretch();
+    formLayout->addRow(i18n("Cleanup:"), daysLayout);
 
-    QFormLayout *sizeLayout = new QFormLayout();
-    layout->addLayout(sizeLayout);
 
-    mUseSizeLimit = new QCheckBox(i18n("Limit to maximum size"), this);
+    QHBoxLayout *maximumSizeLayout = new QHBoxLayout();
+    mUseSizeLimit = new QCheckBox(i18n("Limit to"), this);
     mUseSizeLimit->setWhatsThis(xi18nc("@info:whatsthis",
                                        "<para>Check this box to limit the trash to the maximum amount of disk space that you specify below. "
                                        "Otherwise, it will be unlimited.</para>"));
-    sizeLayout->addRow(mUseSizeLimit);
+    maximumSizeLayout->addWidget(mUseSizeLimit);
+    formLayout->addRow(i18n("Size:"), maximumSizeLayout);
 
-    mSizeWidget = new QWidget(this);
-    sizeLayout->addRow(mSizeWidget);
 
-    QFormLayout *sizeWidgetLayout = new QFormLayout(mSizeWidget);
-    sizeWidgetLayout->setMargin(0);
-
-    QHBoxLayout *maximumSizeLayout = new QHBoxLayout();
-
-    mPercent = new QDoubleSpinBox(mSizeWidget);
+    mPercent = new QDoubleSpinBox(this);
     mPercent->setRange(0.001, 100);
     mPercent->setDecimals(3);
     mPercent->setSingleStep(1);
@@ -297,24 +292,20 @@ void TrashConfigModule::setupGui()
                                   "<para>This is the maximum percent of disk space that will be used for the trash.</para>"));
     maximumSizeLayout->addWidget(mPercent);
 
-    mSizeLabel = new QLabel(mSizeWidget);
+    mSizeLabel = new QLabel(this);
     mSizeLabel->setWhatsThis(xi18nc("@info:whatsthis",
                                     "<para>This is the calculated amount of disk space that will be allowed for the trash, the maximum.</para>"));
     maximumSizeLayout->addWidget(mSizeLabel);
 
-    sizeWidgetLayout->addRow(i18n("Maximum size:"), maximumSizeLayout);
 
-    QLabel *label = new QLabel(i18n("When limit reached:"));
-    sizeWidgetLayout->addRow(label);
-
-    mLimitReachedAction = new QComboBox(mSizeWidget);
-    mLimitReachedAction->addItem(i18n("Warn Me"));
+    mLimitReachedAction = new QComboBox();
+    mLimitReachedAction->addItem(i18n("Show a Warning"));
     mLimitReachedAction->addItem(i18n("Delete Oldest Files From Trash"));
     mLimitReachedAction->addItem(i18n("Delete Biggest Files From Trash"));
     mLimitReachedAction->setWhatsThis(xi18nc("@info:whatsthis",
                                       "<para>When the size limit is reached, it will prefer to delete the type of files that you specify, first. "
                                       "If this is set to warn you, it will do so instead of automatically deleting files.</para>"));
-    sizeWidgetLayout->addRow(nullptr, mLimitReachedAction);
+    formLayout->addRow(i18n("Full Trash:"), mLimitReachedAction);
 
     layout->addStretch();
 }
