@@ -22,7 +22,7 @@
 #include "smbrodlg.h"
 
 // Qt
-#include <QtCore/QTextCodec>
+#include <QTextCodec>
 #include <QLabel>
 #include <QGridLayout>
 
@@ -30,7 +30,6 @@
 #include <kcharsets.h>
 #include <kcombobox.h>
 #include <kconfig.h>
-#include <QDialog>
 #include <kpluginfactory.h>
 #include <KLocalizedString>
 //#include <config-apps.h>
@@ -81,8 +80,8 @@ SMBRoOptions::SMBRoOptions(QWidget *parent, const QVariantList &)
    layout->addWidget(new QWidget(this),4,0);
 
 //   connect(m_showHiddenShares, SIGNAL(toggled(bool)), this, SLOT(changed()));
-   connect(m_userLe, SIGNAL(textChanged(QString)), this, SLOT(changed()));
-   connect(m_passwordLe, SIGNAL(textChanged(QString)), this, SLOT(changed()));
+   connect(m_userLe, &QLineEdit::textChanged, this, &SMBRoOptions::changed);
+   connect(m_passwordLe, &QLineEdit::textChanged, this, &SMBRoOptions::changed);
 //   connect(m_workgroupLe, SIGNAL(textChanged(QString)), this, SLOT(changed()));
 //   connect( m_encodingList, SIGNAL(activated(QString)), this , SLOT(changed()) );
 
@@ -98,7 +97,6 @@ void SMBRoOptions::load()
 {
    KConfig *cfg = new KConfig(QStringLiteral("kioslaverc"));
 
-   QString tmp;
    KConfigGroup group = cfg->group("Browser Settings/SMBro" );
    m_userLe->setText(group.readEntry("User"));
 //   m_workgroupLe->setText(group.readEntry("Workgroup"));
@@ -110,9 +108,10 @@ void SMBRoOptions::load()
 
    // unscramble
    QString scrambled = group.readEntry( "Password" );
-   QString password = QLatin1String("");
-   for (int i=0; i<scrambled.length()/3; i++)
-   {
+   QString password;
+   const int passwordLength = scrambled.length() / 3;
+   password.reserve(passwordLength);
+   for (int i=0; i<passwordLength; ++i) {
       QChar qc1 = scrambled[i*3];
       QChar qc2 = scrambled[i*3+1];
       QChar qc3 = scrambled[i*3+2];
@@ -120,7 +119,7 @@ void SMBRoOptions::load()
       unsigned int a2 = qc2.toLatin1() - 'A';
       unsigned int a3 = qc3.toLatin1() - '0';
       unsigned int num = ((a1 & 0x3F) << 10) | ((a2& 0x1F) << 5) | (a3 & 0x1F);
-      password[i] = QChar((uchar)((num - 17) ^ 173)); // restore
+      password[i] = QLatin1Char((num - 17) ^ 173); // restore
    }
    m_passwordLe->setText(password);
 
@@ -142,16 +141,14 @@ void SMBRoOptions::save()
    //least better than storing the plain password
    QString password(m_passwordLe->text());
    QString scrambled;
-   for (int i=0; i<password.length(); i++)
-   {
-      QChar c = password[i];
+   for (const QChar c : qAsConst(password)) {
       unsigned int num = (c.unicode() ^ 173) + 17;
       unsigned int a1 = (num & 0xFC00) >> 10;
       unsigned int a2 = (num & 0x3E0) >> 5;
       unsigned int a3 = (num & 0x1F);
-      scrambled += (char)(a1+'0');
-      scrambled += (char)(a2+'A');
-      scrambled += (char)(a3+'0');
+      scrambled += QLatin1Char((char)(a1+'0'));
+      scrambled += QLatin1Char((char)(a2+'A'));
+      scrambled += QLatin1Char((char)(a3+'0'));
    }
    group.writeEntry( "Password", scrambled);
 
@@ -160,8 +157,8 @@ void SMBRoOptions::save()
 
 void SMBRoOptions::defaults()
 {
-   m_userLe->setText(QLatin1String(""));
-   m_passwordLe->setText(QLatin1String(""));
+   m_userLe->setText(QString());
+   m_passwordLe->setText(QString());
 //   m_workgroupLe->setText("");
 //   m_showHiddenShares->setChecked(false);
 }
