@@ -35,7 +35,6 @@
 #endif
 
 #ifdef USE_KPROCESS_FOR_KIOSLAVES
-#include <QDir>
 #include <QProcess>
 #include <QStringList>
 #ifdef Q_OS_WIN
@@ -56,7 +55,7 @@ extern "C" KIO::AuthInfo *_kioslave_init_kio()
 int main(int argc, char **argv)
 {
     if (argc < 5) {
-        fprintf(stderr, "Usage: kioslave <slave-lib> <protocol> <klauncher-socket> <app-socket>\n\nThis program is part of KDE.\n");
+        fprintf(stderr, "Usage: kioslave5 <slave-lib> <protocol> <klauncher-socket> <app-socket>\n\nThis program is part of KDE.\n");
         return 1;
     }
 #ifndef _WIN32_WCE
@@ -125,5 +124,15 @@ int main(int argc, char **argv)
 
     int (*func)(int, char *[]) = (int (*)(int, char *[])) sym;
 
-    exit(func(argc - 1, argv + 1)); /* Launch! */
+    // We need argv[0] to remain /path/to/kioslave5
+    // so that applicationDirPath() is correct on non-Linux (no /proc)
+    // and we want to skip argv[1] so the kioslave5 exe is transparent to kdemain.
+    const int newArgc = argc - 1;
+    QVarLengthArray<char*, 5> newArgv(newArgc);
+    newArgv[0] = argv[0];
+    for (int i = 1; i < newArgc; ++i) {
+        newArgv[i] = argv[i+1];
+    }
+
+    return func(newArgc, newArgv.data()); /* Launch! */
 }
