@@ -39,7 +39,6 @@
 #include <QPointer>
 #include <QWidget>
 #include <QDBusInterface>
-#include <QDBusConnection>
 #include <QDBusReply>
 #include <QNetworkReply>
 #include <QNetworkRequest>
@@ -129,7 +128,6 @@ bool AccessManager::isExternalContentAllowed() const
     return d->externalContentAllowed;
 }
 
-#ifndef KIOWIDGETS_NO_DEPRECATED
 void AccessManager::setCookieJarWindowId(WId id)
 {
     QWidget *window = QWidget::find(id);
@@ -144,7 +142,6 @@ void AccessManager::setCookieJarWindowId(WId id)
 
     d->window = window->isWindow() ? window : window->window();
 }
-#endif
 
 void AccessManager::setWindow(QWidget *widget)
 {
@@ -164,7 +161,6 @@ void AccessManager::setWindow(QWidget *widget)
     }
 }
 
-#ifndef KIOWIDGETS_NO_DEPRECATED
 WId AccessManager::cookieJarWindowid() const
 {
     KIO::Integration::CookieJar *jar = qobject_cast<KIO::Integration::CookieJar *> (cookieJar());
@@ -174,7 +170,6 @@ WId AccessManager::cookieJarWindowid() const
 
     return 0;
 }
-#endif
 
 QWidget *AccessManager::window() const
 {
@@ -265,7 +260,7 @@ QNetworkReply *AccessManager::createRequest(Operation op, const QNetworkRequest
     }
     case PostOperation: {
         kioJob = KIO::storedHttpPost(outgoingData, reqUrl, sizeFromRequest(req), KIO::HideProgressInfo);
-        if (!metaData.contains(QStringLiteral("content-type")))  {
+        if (!metaData.contains(QLatin1String("content-type")))  {
             const QVariant header = req.header(QNetworkRequest::ContentTypeHeader);
             if (header.isValid()) {
                 metaData.insert(QStringLiteral("content-type"),
@@ -421,7 +416,8 @@ void AccessManager::AccessManagerPrivate::setMetaDataForRequest(QNetworkRequest
     request.setRawHeader("x-kdewebkit-ignore-disposition", QByteArray());
 
     QStringList customHeaders;
-    Q_FOREACH (const QByteArray &key, request.rawHeaderList()) {
+    const QList<QByteArray> list = request.rawHeaderList();
+    for (const QByteArray &key : list) {
         const QByteArray value = request.rawHeader(key);
         if (value.length()) {
             customHeaders << (QString::fromUtf8(key) + QLatin1String(": ") + QString::fromUtf8(value));
@@ -429,7 +425,7 @@ void AccessManager::AccessManagerPrivate::setMetaDataForRequest(QNetworkRequest
     }
 
     if (!customHeaders.isEmpty()) {
-        metaData.insert(QStringLiteral("customHTTPHeader"), customHeaders.join(QStringLiteral("\r\n")));
+        metaData.insert(QStringLiteral("customHTTPHeader"), customHeaders.join(QLatin1String("\r\n")));
     }
 
     // Append per request meta data, if any...
@@ -468,7 +464,7 @@ bool KIO::Integration::sslConfigFromMetaData(const KIO::MetaData &metadata, QSsl
 {
     bool success = false;
 
-    if (metadata.value(QStringLiteral("ssl_in_use")) == QStringLiteral("TRUE")) {
+    if (metadata.value(QStringLiteral("ssl_in_use")) == QLatin1String("TRUE")) {
         const QSsl::SslProtocol sslProto = qSslProtocolFromString(metadata.value(QStringLiteral("ssl_protocol_version")));
         QList<QSslCipher> cipherList;
         cipherList << QSslCipher(metadata.value(QStringLiteral("ssl_cipher_name")), sslProto);
@@ -518,8 +514,12 @@ QList<QNetworkCookie> CookieJar::cookiesForUrl(const QUrl &url) const
     }
 
     const QString cookieStr = reply.value();
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
     const QStringList cookies = cookieStr.split(QStringLiteral("; "), QString::SkipEmptyParts);
-    Q_FOREACH (const QString &cookie, cookies) {
+#else
+    const QStringList cookies = cookieStr.split(QStringLiteral("; "), Qt::SkipEmptyParts);
+#endif
+    for (const QString &cookie : cookies) {
         const int index = cookie.indexOf(QL1C('='));
         const QStringRef name = cookie.leftRef(index);
         const QStringRef value = cookie.rightRef((cookie.length() - index - 1));
@@ -537,7 +537,7 @@ bool CookieJar::setCookiesFromUrl(const QList<QNetworkCookie> &cookieList, const
     }
 
     QDBusInterface kcookiejar(QStringLiteral("org.kde.kcookiejar5"), QStringLiteral("/modules/kcookiejar"), QStringLiteral("org.kde.KCookieServer"));
-    Q_FOREACH (const QNetworkCookie &cookie, cookieList) {
+    for (const QNetworkCookie &cookie : cookieList) {
         QByteArray cookieHeader("Set-Cookie: ");
         if (d->isStorageDisabled && !cookie.isSessionCookie()) {
             QNetworkCookie sessionCookie(cookie);
