@@ -21,7 +21,6 @@
 
 #include "delegateanimationhandler_p.h"
 
-#include <QListView>
 #include <QAbstractItemView>
 #include <QPersistentModelIndex>
 #include <QTime>
@@ -39,6 +38,7 @@ namespace KIO
 // Needed because state() is a protected method
 class ProtectedAccessor : public QAbstractItemView
 {
+    Q_OBJECT
 public:
     bool draggingState() const
     {
@@ -52,7 +52,7 @@ public:
 
 // ---------------------------------------------------------------------------
 
-CachedRendering::CachedRendering(QStyle::State state, const QSize &size, QModelIndex index, qreal devicePixelRatio)
+CachedRendering::CachedRendering(QStyle::State state, const QSize &size, const QModelIndex &index, qreal devicePixelRatio)
     : state(state), regular(QPixmap(size*devicePixelRatio)), hover(QPixmap(size*devicePixelRatio)), valid(true), validityIndex(index)
 {
     regular.setDevicePixelRatio(devicePixelRatio);
@@ -61,9 +61,10 @@ CachedRendering::CachedRendering(QStyle::State state, const QSize &size, QModelI
     hover.fill(Qt::transparent);
 
     if (index.model()) {
-        connect(index.model(), SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-                SLOT(dataChanged(QModelIndex,QModelIndex)));
-        connect(index.model(), SIGNAL(modelReset()), SLOT(modelReset()));
+        connect(index.model(), &QAbstractItemModel::dataChanged,
+                this, &CachedRendering::dataChanged);
+        connect(index.model(), &QAbstractItemModel::modelReset,
+                this, &CachedRendering::modelReset);
     }
 }
 
@@ -85,7 +86,7 @@ void CachedRendering::modelReset()
 AnimationState::AnimationState(const QModelIndex &index)
     : index(index), direction(QTimeLine::Forward),
       animating(false), jobAnimation(false), progress(0.0), m_fadeProgress(1.0),
-      m_jobAnimationAngle(0.0), renderCache(NULL), fadeFromRenderCache(NULL)
+      m_jobAnimationAngle(0.0), renderCache(nullptr), fadeFromRenderCache(nullptr)
 {
     creationTime.start();
 }
@@ -115,7 +116,7 @@ bool AnimationState::update()
         m_fadeProgress = qMin(qreal(1.0), m_fadeProgress + delta);
         animating |= (m_fadeProgress < 1.0);
         if (m_fadeProgress == 1) {
-            setCachedRenderingFadeFrom(0);
+            setCachedRenderingFadeFrom(nullptr);
         }
     }
 
@@ -176,7 +177,7 @@ DelegateAnimationHandler::DelegateAnimationHandler(QObject *parent)
 {
     iconSequenceTimer.setSingleShot(true);
     iconSequenceTimer.setInterval(switchIconInterval);
-    connect(&iconSequenceTimer, SIGNAL(timeout()), SLOT(sequenceTimerTimeout()));;
+    connect(&iconSequenceTimer, &QTimer::timeout, this, &DelegateAnimationHandler::sequenceTimerTimeout);;
 }
 
 DelegateAnimationHandler::~DelegateAnimationHandler()
@@ -238,7 +239,7 @@ void DelegateAnimationHandler::setSequenceIndex(int sequenceIndex)
     }
 }
 
-void DelegateAnimationHandler::eventuallyStartIteration(QModelIndex index)
+void DelegateAnimationHandler::eventuallyStartIteration(const QModelIndex &index)
 {
 //      if (KGlobalSettings::graphicEffectsLevel() & KGlobalSettings::SimpleAnimationEffects) {
     ///Think about it.
@@ -261,7 +262,7 @@ AnimationState *DelegateAnimationHandler::animationState(const QStyleOption &opt
     // item will be drawn in two locations at the same time and hovered in one and
     // not the other. We can't tell them apart because they both have the same index.
     if (!view || static_cast<const ProtectedAccessor *>(view)->draggingState()) {
-        return NULL;
+        return nullptr;
     }
 
     AnimationState *state = findAnimationState(view, index);
@@ -327,16 +328,16 @@ AnimationState *DelegateAnimationHandler::findAnimationState(const QAbstractItem
         const QModelIndex &index) const
 {
     // Try to find a list of animation states for the view
-    AnimationList *list = animationLists.value(view);
+    const AnimationList *list = animationLists.value(view);
 
     if (list) {
-        foreach (AnimationState *state, *list)
+        for (AnimationState *state : *list)
             if (state->index == index) {
                 return state;
             }
     }
 
-    return NULL;
+    return nullptr;
 }
 
 void DelegateAnimationHandler::addAnimationState(AnimationState *state, const QAbstractItemView *view)
@@ -345,7 +346,7 @@ void DelegateAnimationHandler::addAnimationState(AnimationState *state, const QA
 
     // If this is the first time we've seen this view
     if (!list) {
-        connect(view, SIGNAL(destroyed(QObject*)), SLOT(viewDeleted(QObject*)));
+        connect(view, &QObject::destroyed, this, &DelegateAnimationHandler::viewDeleted);
 
         list = new AnimationList;
         animationLists.insert(view, list);
@@ -438,3 +439,4 @@ void DelegateAnimationHandler::timerEvent(QTimerEvent *)
 
 }
 
+#include "delegateanimationhandler.moc"
