@@ -44,8 +44,6 @@
 #include <KRun>
 #include <KService>
 
-#include <QApplication>
-#include <QDebug>
 #include <QDropEvent>
 #include <QFileInfo>
 #include <QMenu>
@@ -66,7 +64,7 @@ class KIO::DropMenu : public QMenu
     Q_OBJECT
 public:
     explicit DropMenu(QWidget *parent = nullptr);
-    ~DropMenu();
+    ~DropMenu() override;
 
     void addCancelAction();
     void addExtraActions(const QList<QAction *> &appActions, const QList<QAction *> &pluginActions);
@@ -82,7 +80,7 @@ private:
 class KIO::DropJobPrivate : public KIO::JobPrivate
 {
 public:
-    DropJobPrivate(const QDropEvent *dropEvent, const QUrl &destUrl, JobFlags flags)
+    DropJobPrivate(const QDropEvent *dropEvent, const QUrl &destUrl, DropJobFlags dropjobFlags, JobFlags flags)
         : JobPrivate(),
           // Extract everything from the dropevent, since it will be deleted before the job starts
           m_mimeData(dropEvent->mimeData()),
@@ -92,6 +90,7 @@ public:
           m_keyboardModifiers(dropEvent->keyboardModifiers()),
           m_destUrl(destUrl),
           m_destItem(KCoreDirLister::cachedItemForUrl(destUrl)),
+          m_dropjobFlags(dropjobFlags),
           m_flags(flags),
           m_triggered(false)
     {
@@ -149,6 +148,7 @@ public:
     QUrl m_destUrl;
     KFileItem m_destItem; // null for remote URLs not found in the dirlister cache
     const JobFlags m_flags;
+    const DropJobFlags m_dropjobFlags;
     QList<QAction *> m_appActions;
     QList<QAction *> m_pluginActions;
     bool m_triggered;  // Tracks whether an action has been triggered in the popup menu.
@@ -160,9 +160,9 @@ public:
     void slotTriggered(QAction *);
     void slotAboutToHide();
 
-    static inline DropJob *newJob(const QDropEvent *dropEvent, const QUrl &destUrl, JobFlags flags)
+    static inline DropJob *newJob(const QDropEvent *dropEvent, const QUrl &destUrl, DropJobFlags dropjobFlags, JobFlags flags)
     {
-        DropJob *job = new DropJob(*new DropJobPrivate(dropEvent, destUrl, flags));
+        DropJob *job = new DropJob(*new DropJobPrivate(dropEvent, destUrl, dropjobFlags, flags));
         job->setUiDelegate(KIO::createDefaultJobUiDelegate());
         // Note: never KIO::getJobTracker()->registerJob here.
         // We don't want a progress dialog during the copy/move/link popup, it would in fact close
@@ -269,7 +269,7 @@ int DropJobPrivate::determineDropAction()
 
     bool allItemsAreFromTrash = true;
     bool containsTrashRoot = false;
-    foreach (const QUrl &url, m_urls) {
+    for (const QUrl &url : m_urls) {
         const bool local = url.isLocalFile();
         if (!local /*optimization*/ && url.scheme() == QLatin1String("trash")) {
             if (url.path().isEmpty() || url.path() == QLatin1String("/")) {
@@ -328,7 +328,7 @@ void DropJobPrivate::fillPopupMenu(KIO::DropMenu *popup)
     // we would need to KIO::stat each URL in turn, asynchronously....
     KFileItemList fileItems;
     fileItems.reserve(m_urls.size());
-    foreach (const QUrl &url, m_urls) {
+    for (const QUrl &url : m_urls) {
         fileItems.append(KFileItem(url));
     }
     const KFileItemListProperties itemProps(fileItems);
@@ -360,7 +360,7 @@ void DropJobPrivate::fillPopupMenu(KIO::DropMenu *popup)
 
     if (sMoving || (sReading && sDeleting)) {
         bool equalDestination = true;
-        foreach (const QUrl &src, m_urls) {
+        for (const QUrl &src : m_urls) {
             if (!m_destUrl.matches(src.adjusted(QUrl::RemoveFilename), QUrl::StripTrailingSlash)) {
                 equalDestination = false;
                 break;
@@ -386,7 +386,7 @@ void DropJobPrivate::fillPopupMenu(KIO::DropMenu *popup)
 void DropJobPrivate::addPluginActions(KIO::DropMenu *popup, const KFileItemListProperties &itemProps)
 {
     const QVector<KPluginMetaData> plugin_offers = KPluginLoader::findPlugins(QStringLiteral("kf5/kio_dnd"));
-    foreach (const KPluginMetaData &service, plugin_offers) {
+    for (const KPluginMetaData &service : plugin_offers) {
         KPluginFactory *factory = KPluginLoader(service.fileName()).factory();
         if (factory) {
             KIO::DndPopupMenuPlugin *plugin = factory->create<KIO::DndPopupMenuPlugin>();
@@ -412,6 +412,20 @@ void DropJob::setApplicationActions(const QList<QAction *> &actions)
     for (KIO::DropMenu *menu : qAsConst(d->m_menus)) {
         menu->addExtraActions(d->m_appActions, d->m_pluginActions);
     }
+
+}
+
+void DropJob::showMenu(const QPoint &p, QAction *atAction)
+{
+    Q_D(DropJob);
+
+    if (!(d->m_dropjobFlags & KIO::ShowMenuManually)) {
+        return;
+    }
+
+    for (KIO::DropMenu *menu : qAsConst(d->m_menus)) {
+        menu->popup(p, atAction);
+    }
 }
 
 void DropJobPrivate::slotTriggered(QAction *action)
@@ -463,7 +477,10 @@ void DropJobPrivate::handleCopyToDirectory()
                 m_triggered = true;
                 slotTriggered(action);
             });
-            menu->popup(window ? window->mapToGlobal(m_relativePos) : QCursor::pos());
+
+            if (!(m_dropjobFlags & KIO::ShowMenuManually)) {
+                menu->popup(window ? window->mapToGlobal(m_relativePos) : QCursor::pos());
+            }
             m_menus.insert(menu);
             QObject::connect(menu, &QObject::destroyed, q, [this, menu]() {
                 m_menus.remove(menu);
@@ -526,7 +543,7 @@ void DropJobPrivate::handleDropToDesktopFile()
     if (desktopFile.hasApplicationType()) {
         // Drop to application -> start app with urls as argument
         KService service(destFile);
-        if (!KRun::runService(service, m_urls, KJobWidgets::window(q))) {
+        if (!KRun::runApplication(service, m_urls, KJobWidgets::window(q))) {
             q->setError(KIO::ERR_CANNOT_LAUNCH_PROCESS);
             q->setErrorText(destFile);
         }
@@ -572,7 +589,12 @@ void DropJob::slotResult(KJob *job)
 
 DropJob * KIO::drop(const QDropEvent *dropEvent, const QUrl &destUrl, JobFlags flags)
 {
-    return DropJobPrivate::newJob(dropEvent, destUrl, flags);
+    return DropJobPrivate::newJob(dropEvent, destUrl, KIO::DropJobDefaultFlags, flags);
+}
+
+DropJob * KIO::drop(const QDropEvent *dropEvent, const QUrl &destUrl, DropJobFlags dropjobFlags, JobFlags flags)
+{
+    return DropJobPrivate::newJob(dropEvent, destUrl, dropjobFlags, flags);
 }
 
 #include "moc_dropjob.cpp"
