@@ -33,9 +33,25 @@ class KFileItemListProperties;
 namespace KIO
 {
 
+/**
+* Special flag of DropJob in addition to KIO::JobFlag
+*
+* @since 5.67
+*/
+enum DropJobFlag
+{
+    DropJobDefaultFlags = 0,
+    ShowMenuManually = 1, ///< show the menu manually with DropJob::showMenu
+};
+Q_DECLARE_FLAGS(DropJobFlags, DropJobFlag)
+Q_DECLARE_OPERATORS_FOR_FLAGS(DropJobFlags)
+
+class CopyJob;
 class DropJobPrivate;
 
 /**
+ * @class KIO::DropJob dropjob.h <KIO/DropJob>
+ *
  * A KIO job that handles dropping into a file-manager-like view.
  * @see KIO::drop
  *
@@ -59,12 +75,32 @@ public:
      */
     void setApplicationActions(const QList<QAction *> &actions);
 
+    /**
+     * Allows the application to show the menu manually.
+     * DropJob instance has to be created with the KIO::ShowMenuManually flag
+     *
+     * @since 5.67
+     */
+    void showMenu(const QPoint &p, QAction *atAction = nullptr);
+
 Q_SIGNALS:
     /**
      * Signals that a file or directory was created.
      */
     void itemCreated(const QUrl &url);
 
+    /**
+    * Emitted when a copy job was started as subjob after user selection.
+    *
+    * You can use @p job to monitor the progress of the copy/move/link operation. Note that a
+    * CopyJob isn't always started by DropJob. For instance dropping files onto an executable will
+    * simply launch the executable.
+    *
+    * @param job the job started for moving, copying or symlinking files
+    * @since 5.30
+    */
+    void copyJobStarted(KIO::CopyJob *job);
+
     /**
      * Signals that the popup menu is about to be shown.
      * Applications can use the information provided about the dropped URLs
@@ -74,7 +110,7 @@ Q_SIGNALS:
     void popupMenuAboutToShow(const KFileItemListProperties &itemProps);
 
 protected Q_SLOTS:
-    void slotResult(KJob *job) Q_DECL_OVERRIDE;
+    void slotResult(KJob *job) override;
 
 protected:
     DropJob(DropJobPrivate &dd);
@@ -92,7 +128,7 @@ private:
  * which is then implemented by the job, using KIO::move, KIO::copy or KIO::link
  * Additional actions provided by the application or by plugins can be shown in the popup.
  *
- * If the mime data contains other data than URLs, it is saved into a file after asking
+ * If the mime data contains data other than URLs, it is saved into a file after asking
  * the user to choose a filename and the preferred data format.
  *
  * This job takes care of recording the subjob in the FileUndoManager, and emits
@@ -105,10 +141,28 @@ private:
  * @param flags passed to the sub job
  *
  * @return A pointer to the job handling the operation.
+ * @warning Don't forget to call KJobWidgets::setWindow() on this job, otherwise the popup
+ *          menu won't be properly positioned with Wayland compositors.
  * @since 5.4
  */
 KIOWIDGETS_EXPORT DropJob *drop(const QDropEvent *dropEvent, const QUrl &destUrl, JobFlags flags = DefaultFlags);
 
+/**
+ * Similar to KIO::drop
+ *
+ * @param dropEvent the drop event, from which the job will extract mimeData, dropAction, etc.
+         The application should take care of calling dropEvent->acceptProposedAction().
+ * @param destUrl The URL of the target file or directory
+ * @param dropjobFlags Show the menu immediately or manually.
+ * @param flags passed to the sub job
+ *
+ * @return A pointer to the job handling the operation.
+ * @warning Don't forget to call DropJob::showMenu on this job, otherwise the popup will never be shown
+ *
+ * @since 5.67
+ */
+KIOWIDGETS_EXPORT DropJob *drop(const QDropEvent *dropEvent, const QUrl &destUrl, DropJobFlags dropjobFlags, JobFlags flags = DefaultFlags); // TODO KF6: merge with DropJobFlags dropjobFlag = DropJobDefaultFlags
+
 }
 
 #endif
