@@ -20,7 +20,7 @@
 #include "kdirmodel.h"
 #include "kdirlister.h"
 #include "kfileitem.h"
-#include "kio_widgets_debug.h"
+
 #include <klocalizedstring.h>
 #include <kjobuidelegate.h>
 #include <kio/simplejob.h>
@@ -28,14 +28,16 @@
 #include "joburlcache_p.h"
 #include <kurlmimedata.h>
 #include <kiconloader.h>
+
 #include <QMimeData>
 #include <QBitArray>
 #include <QDebug>
+#include <QDirIterator>
 #include <QFile>
 #include <QFileInfo>
 #include <QDir>
 #include <QIcon>
-#include <QLocale>
+#include <QLoggingCategory>
 #include <qplatformdefs.h>
 
 #include <algorithm>
@@ -44,6 +46,8 @@
 #include <qt_windows.h>
 #endif
 
+static QLoggingCategory category("kf5.kio.kdirmodel", QtInfoMsg);
+
 class KDirModelNode;
 class KDirModelDirNode;
 
@@ -149,7 +153,7 @@ public:
     void collectAllChildUrls(QList<QUrl> &urls) const
     {
         urls.reserve(urls.size() + m_childNodes.size());
-        Q_FOREACH (KDirModelNode *node, m_childNodes) {
+        for (KDirModelNode *node : m_childNodes) {
             const KFileItem &item = node->item();
             urls.append(cleanupUrl(item.url()));
             if (item.isDir()) {
@@ -295,7 +299,7 @@ KDirModelNode *KDirModelPrivate::expandAllParentsUntil(const QUrl &_url) const /
             nodePath += QLatin1Char('/');
         }
         if (!pathStr.startsWith(nodePath)) {
-            qCWarning(KIO_WIDGETS) << "The kioslave for" << url.scheme() << "violates the hierarchy structure:"
+            qCWarning(category) << "The kioslave for" << url.scheme() << "violates the hierarchy structure:"
                        << "I arrived at node" << nodePath << ", but" << pathStr << "does not start with that path.";
             return nullptr;
         }
@@ -307,7 +311,7 @@ KDirModelNode *KDirModelPrivate::expandAllParentsUntil(const QUrl &_url) const /
         nodeUrl = nodeUrl.adjusted(QUrl::StripTrailingSlash); // #172508
         KDirModelNode *node = nodeForUrl(nodeUrl);
         if (!node) {
-            //qDebug() << "child equal or starting with" << url << "not found";
+            qCDebug(category) << nodeUrl << "not found, needs to be listed";
             // return last parent found:
             return dirNode;
         }
@@ -316,10 +320,10 @@ KDirModelNode *KDirModelPrivate::expandAllParentsUntil(const QUrl &_url) const /
 
         //qDebug() << " nodeUrl=" << nodeUrl;
         if (nodeUrl == url) {
-            //qDebug() << "Found node" << node << "for" << url;
+            qCDebug(category) << "Found node" << node << "for" << url;
             return node;
         }
-        //qDebug() << "going into" << node->item().url();
+        qCDebug(category) << "going into" << node->item().url();
         Q_ASSERT(isDir(node));
         dirNode = static_cast<KDirModelDirNode *>(node);
     }
@@ -330,11 +334,11 @@ KDirModelNode *KDirModelPrivate::expandAllParentsUntil(const QUrl &_url) const /
 #ifndef NDEBUG
 void KDirModelPrivate::dump()
 {
-    qCDebug(KIO_WIDGETS) << "Dumping contents of KDirModel" << q << "dirLister url:" << m_dirLister->url();
+    qCDebug(category) << "Dumping contents of KDirModel" << q << "dirLister url:" << m_dirLister->url();
     QHashIterator<QUrl, KDirModelNode *> it(m_nodeHash);
     while (it.hasNext()) {
         it.next();
-        qDebug(KIO_WIDGETS) << it.key() << it.value();
+        qCDebug(category) << it.key() << it.value();
     }
 }
 #endif
@@ -367,7 +371,6 @@ KDirModelNode *KDirModelPrivate::nodeForIndex(const QModelIndex &index) const
  * Invalid parent index means root of the tree, m_rootNode
  */
 
-#ifndef NDEBUG
 static QString debugIndex(const QModelIndex &index)
 {
     QString str;
@@ -383,7 +386,6 @@ static QString debugIndex(const QModelIndex &index)
     }
     return str;
 }
-#endif
 
 KDirModel::KDirModel(QObject *parent)
     : QAbstractItemModel(parent),
@@ -419,6 +421,12 @@ void KDirModel::setDirLister(KDirLister *dirLister)
         [this](const QUrl &oldUrl, const QUrl &newUrl){d->_k_slotRedirection(oldUrl, newUrl);} );
 }
 
+Qt::DropActions KDirModel::supportedDropActions() const
+{
+    return Qt::CopyAction | Qt::MoveAction | Qt::LinkAction | Qt::IgnoreAction;
+}
+
+
 KDirLister *KDirModel::dirLister() const
 {
     return d->m_dirLister;
@@ -432,7 +440,7 @@ void KDirModelPrivate::_k_slotNewItems(const QUrl &directoryUrl, const KFileItem
     // If the directory containing the items wasn't found, then we have a big problem.
     // Are you calling KDirLister::openUrl(url,true,false)? Please use expandToUrl() instead.
     if (!result) {
-        qCWarning(KIO_WIDGETS) << "Items emitted in directory" << directoryUrl
+        qCWarning(category) << "Items emitted in directory" << directoryUrl
                    << "but that directory isn't in KDirModel!"
                    << "Root directory:" << urlForNode(m_rootNode);
         for (const KFileItem &item : items) {
@@ -449,17 +457,16 @@ void KDirModelPrivate::_k_slotNewItems(const QUrl &directoryUrl, const KFileItem
     const QModelIndex index = indexForNode(dirNode); // O(n)
     const int newItemsCount = items.count();
     const int newRowCount = dirNode->m_childNodes.count() + newItemsCount;
-#if 0
-#ifndef NDEBUG // debugIndex only defined in debug mode
-    //qDebug() << items.count() << "in" << directoryUrl
-            << "index=" << debugIndex(index) << "newRowCount=" << newRowCount;
-#endif
-#endif
+
+    qCDebug(category) << items.count() << "in" << directoryUrl
+                      << "index=" << debugIndex(index) << "newRowCount=" << newRowCount;
 
     q->beginInsertRows(index, newRowCount - newItemsCount, newRowCount - 1);   // parent, first, last
 
     const QList<QUrl> urlsBeingFetched = m_urlsBeingFetched.value(dirNode);
-    //qDebug() << "urlsBeingFetched for dir" << dirNode << directoryUrl << ":" << urlsBeingFetched;
+    if (!urlsBeingFetched.isEmpty()) {
+        qCDebug(category) << "urlsBeingFetched for dir" << dirNode << directoryUrl << ":" << urlsBeingFetched;
+    }
 
     QList<QModelIndex> emitExpandFor;
 
@@ -476,7 +483,7 @@ void KDirModelPrivate::_k_slotNewItems(const QUrl &directoryUrl, const KFileItem
         // not sure if/how it ever happened.
         //if (dirNode->m_childNodes.count() &&
         //    dirNode->m_childNodes.last()->item().name() == (*it).name()) {
-        //    qCWarning(KIO_WIDGETS) << "Already having" << (*it).name() << "in" << directoryUrl
+        //    qCWarning(category) << "Already having" << (*it).name() << "in" << directoryUrl
         //             << "url=" << dirNode->m_childNodes.last()->item().url();
         //    abort();
         //}
@@ -484,11 +491,10 @@ void KDirModelPrivate::_k_slotNewItems(const QUrl &directoryUrl, const KFileItem
         dirNode->m_childNodes.append(node);
         const QUrl url = it->url();
         m_nodeHash.insert(cleanupUrl(url), node);
-        //qDebug() << url;
 
         if (!urlsBeingFetched.isEmpty()) {
-            const QUrl dirUrl(url);
-            foreach (const QUrl &urlFetched, urlsBeingFetched) {
+            const QUrl &dirUrl = url;
+            for (const QUrl &urlFetched : qAsConst(urlsBeingFetched)) {
                 if (dirUrl.matches(urlFetched, QUrl::StripTrailingSlash) || dirUrl.isParentOf(urlFetched)) {
                     //qDebug() << "Listing found" << dirUrl.url() << "which is a parent of fetched url" << urlFetched;
                     const QModelIndex parentIndex = indexForNode(node, dirNode->m_childNodes.count() - 1);
@@ -507,7 +513,7 @@ void KDirModelPrivate::_k_slotNewItems(const QUrl &directoryUrl, const KFileItem
 
     // Emit expand signal after rowsInserted signal has been emitted,
     // so that any proxy model will have updated its mapping already
-    Q_FOREACH (const QModelIndex &idx, emitExpandFor) {
+    for (const QModelIndex &idx : qAsConst(emitExpandFor)) {
         emit q->expand(idx);
     }
 }
@@ -522,7 +528,7 @@ void KDirModelPrivate::_k_slotCompleted(const QUrl &directoryUrl)
 
 void KDirModelPrivate::_k_slotDeleteItems(const KFileItemList &items)
 {
-    //qDebug() << items.count();
+    qCDebug(category) << items.count() << "items";
 
     // I assume all items are from the same directory.
     // From KDirLister's code, this should be the case, except maybe emitChanges?
@@ -531,7 +537,7 @@ void KDirModelPrivate::_k_slotDeleteItems(const KFileItemList &items)
     QUrl url = item.url();
     KDirModelNode *node = nodeForUrl(url); // O(depth)
     if (!node) {
-        qCWarning(KIO_WIDGETS) << "No node found for item that was just removed:" << url;
+        qCWarning(category) << "No node found for item that was just removed:" << url;
         return;
     }
 
@@ -561,7 +567,7 @@ void KDirModelPrivate::_k_slotDeleteItems(const KFileItemList &items)
             url = item.url();
             node = nodeForUrl(url);
             if (!node) {
-                qCWarning(KIO_WIDGETS) << "No node found for item that was just removed:" << url;
+                qCWarning(category) << "No node found for item that was just removed:" << url;
                 continue;
             }
             if (!node->parent()) {
@@ -652,10 +658,7 @@ void KDirModelPrivate::_k_slotRefreshItems(const QList<QPair<KFileItem, KFileIte
             }
         }
     }
-#ifndef NDEBUG // debugIndex only defined in debug mode
     //qDebug() << "dataChanged(" << debugIndex(topLeft) << " - " << debugIndex(bottomRight);
-    Q_UNUSED(debugIndex(QModelIndex())); // fix compiler warning
-#endif
     bottomRight = bottomRight.sibling(bottomRight.row(), q->columnCount(QModelIndex()) - 1);
     emit q->dataChanged(topLeft, bottomRight);
 }
@@ -751,9 +754,7 @@ void KDirModel::itemChanged(const QModelIndex &index)
         node->setPreview(QIcon());
     }
 
-#ifndef NDEBUG // debugIndex only defined in debug mode
-    //qDebug() << "dataChanged(" << debugIndex(index);
-#endif
+    qCDebug(category) << "dataChanged(" << debugIndex(index) << ")";
     emit dataChanged(index, index);
 }
 
@@ -898,7 +899,7 @@ bool KDirModel::setData(const QModelIndex &index, const QVariant &value, int rol
             KDirModelNode *node = static_cast<KDirModelNode *>(index.internalPointer());
             const KFileItem &item = node->item();
             const QString newName = value.toString();
-            if (newName.isEmpty() || newName == item.text() || (newName == QLatin1String(".")) || (newName == QLatin1String(".."))) {
+            if (newName.isEmpty() || newName == item.text() || (newName == QLatin1Char('.')) || (newName == QLatin1String(".."))) {
                 return true;
             }
             QUrl newUrl = item.url().adjusted(QUrl::RemoveFilename);
@@ -1074,14 +1075,12 @@ KFileItem KDirModel::itemForIndex(const QModelIndex &index) const
     }
 }
 
-#ifndef KIOWIDGETS_NO_DEPRECATED
 QModelIndex KDirModel::indexForItem(const KFileItem *item) const
 {
     // Note that we can only use the URL here, not the pointer.
     // KFileItems can be copied.
     return indexForUrl(item->url()); // O(n)
 }
-#endif
 
 QModelIndex KDirModel::indexForItem(const KFileItem &item) const
 {
@@ -1146,9 +1145,21 @@ bool KDirModel::hasChildren(const QModelIndex &parent) const
         return true;
     }
 
-    const KFileItem &parentItem = static_cast<KDirModelNode *>(parent.internalPointer())->item();
+    const KDirModelNode *parentNode = static_cast<KDirModelNode *>(parent.internalPointer());
+    const KFileItem &parentItem = parentNode->item();
     Q_ASSERT(!parentItem.isNull());
-    return parentItem.isDir();
+    if (!parentItem.isDir()) {
+        return false;
+    }
+    if (static_cast<const KDirModelDirNode *>(parentNode)->isPopulated()) {
+        return !static_cast<const KDirModelDirNode *>(parentNode)->m_childNodes.isEmpty();
+    }
+    if (parentItem.isLocalFile()) {
+        QDirIterator it(parentItem.localPath(), QDir::Dirs | QDir::NoSymLinks | QDir::NoDotAndDotDot,  QDirIterator::Subdirectories);
+        return it.hasNext();
+    }
+    // Remote and not listed yet, we can't know; let the user click on it so we'll find out
+    return true;
 }
 
 Qt::ItemFlags KDirModel::flags(const QModelIndex &index) const
@@ -1167,7 +1178,7 @@ Qt::ItemFlags KDirModel::flags(const QModelIndex &index) const
         } else {
             KFileItem item = itemForIndex(index);
             if (item.isNull()) {
-                qCWarning(KIO_WIDGETS) << "Invalid item returned for index";
+                qCWarning(category) << "Invalid item returned for index";
             } else if (item.isDir()) {
                 if (d->m_dropsAllowed & DropOnDirectory) {
                     f |= Qt::ItemIsDropEnabled;
@@ -1255,26 +1266,26 @@ void KDirModel::expandToUrl(const QUrl &url)
 {
     // emit expand for each parent and return last parent
     KDirModelNode *result = d->expandAllParentsUntil(url); // O(depth)
-    //qDebug() << url << result;
 
     if (!result) { // doesn't seem related to our base url?
+        qCDebug(category) << url << "does not seem related to our base URL, aborting";
         return;
     }
-    if (!(result->item().isNull()) && result->item().url() == url) {
+    if (!result->item().isNull() && result->item().url() == url) {
         // We have it already, nothing to do
-        //qDebug() << "have it already item=" <<url /*result->item()*/;
+        qCDebug(category) << "we have it already:" << url;
         return;
     }
 
     d->m_urlsBeingFetched[result].append(url);
 
     if (result == d->m_rootNode) {
-        //qDebug() << "Remembering to emit expand after listing the root url";
+        qCDebug(category) << "Remembering to emit expand after listing the root url";
         // the root is fetched by default, so it must be currently being fetched
         return;
     }
 
-    //qDebug() << "Remembering to emit expand after listing" << result->item().url();
+    qCDebug(category) << "Remembering to emit expand after listing" << result->item().url();
 
     // start a new fetch to look for the next level down the URL
     const QModelIndex parentIndex = d->indexForNode(result); // O(n)
