@@ -26,17 +26,15 @@
 #include <QApplication>
 #include <QCheckBox>
 #include <QDate>
-#include <QDebug>
 #include <QDesktopWidget>
-#include <QDir>
 #include <QLabel>
-#include <QLayout>
 #include <QLineEdit>
 #include <QMimeDatabase>
 #include <QPixmap>
 #include <QPushButton>
 #include <QScrollArea>
 #include <QScrollBar>
+#include <QLayout>
 
 #include <kiconloader.h>
 #include <kmessagebox.h>
@@ -53,6 +51,7 @@
 #include <kfilemetadatawidget.h>
 #endif
 #include <previewjob.h>
+#include <kfileutils.h>
 
 using namespace KIO;
 
@@ -166,40 +165,40 @@ RenameDialog::RenameDialog(QWidget *parent, const QString &_caption,
 
     d->bCancel = new QPushButton(this);
     KGuiItem::assign(d->bCancel, KStandardGuiItem::cancel());
-    connect(d->bCancel, SIGNAL(clicked()), this, SLOT(cancelPressed()));
+    connect(d->bCancel, &QAbstractButton::clicked, this, &RenameDialog::cancelPressed);
 
     if (_options & RenameDialog_MultipleItems) {
         d->bApplyAll = new QCheckBox(i18n("Appl&y to All"), this);
         d->bApplyAll->setToolTip((_options & RenameDialog_IsDirectory) ? i18n("When this is checked the button pressed will be applied to all subsequent folder conflicts for the remainder of the current job.\nUnless you press Skip you will still be prompted in case of a conflict with an existing file in the directory.")
                                  : i18n("When this is checked the button pressed will be applied to all subsequent conflicts for the remainder of the current job."));
-        connect(d->bApplyAll, SIGNAL(clicked()), this, SLOT(applyAllPressed()));
+        connect(d->bApplyAll, &QAbstractButton::clicked, this, &RenameDialog::applyAllPressed);
     }
 
     if (!(_options & RenameDialog_NoRename)) {
         d->bRename = new QPushButton(i18n("&Rename"), this);
         d->bRename->setEnabled(false);
         d->bSuggestNewName = new QPushButton(i18n("Suggest New &Name"), this);
-        connect(d->bSuggestNewName, SIGNAL(clicked()), this, SLOT(suggestNewNamePressed()));
-        connect(d->bRename, SIGNAL(clicked()), this, SLOT(renamePressed()));
+        connect(d->bSuggestNewName, &QAbstractButton::clicked, this, &RenameDialog::suggestNewNamePressed);
+        connect(d->bRename, &QAbstractButton::clicked, this, &RenameDialog::renamePressed);
     }
 
     if ((_options & RenameDialog_MultipleItems) && (_options & RenameDialog_Skip)) {
         d->bSkip = new QPushButton(i18n("&Skip"), this);
         d->bSkip->setToolTip((_options & RenameDialog_IsDirectory) ? i18n("Do not copy or move this folder, skip to the next item instead")
                              : i18n("Do not copy or move this file, skip to the next item instead"));
-        connect(d->bSkip, SIGNAL(clicked()), this, SLOT(skipPressed()));
+        connect(d->bSkip, &QAbstractButton::clicked, this, &RenameDialog::skipPressed);
     }
 
     if (_options & RenameDialog_Overwrite) {
         const QString text = (_options & RenameDialog_IsDirectory) ? i18nc("Write files into an existing folder", "&Write Into") : i18n("&Overwrite");
         d->bOverwrite = new QPushButton(text, this);
         d->bOverwrite->setToolTip(i18n("Files and folders will be copied into the existing directory, alongside its existing contents.\nYou will be prompted again in case of a conflict with an existing file in the directory."));
-        connect(d->bOverwrite, SIGNAL(clicked()), this, SLOT(overwritePressed()));
+        connect(d->bOverwrite, &QAbstractButton::clicked, this, &RenameDialog::overwritePressed);
     }
 
     if (_options & RenameDialog_Resume) {
         d->bResume = new QPushButton(i18n("&Resume"), this);
-        connect(d->bResume, SIGNAL(clicked()), this, SLOT(resumePressed()));
+        connect(d->bResume, &QAbstractButton::clicked, this, &RenameDialog::resumePressed);
     }
 
     QVBoxLayout *pLayout = new QVBoxLayout(this);
@@ -220,10 +219,10 @@ RenameDialog::RenameDialog(QWidget *parent, const QString &_caption,
         } else {
             UDSEntry srcUds;
 
-            srcUds.insert(UDSEntry::UDS_NAME, d->src.fileName());
-            srcUds.insert(UDSEntry::UDS_MODIFICATION_TIME, mtimeSrc.toMSecsSinceEpoch() / 1000);
-            srcUds.insert(UDSEntry::UDS_CREATION_TIME, ctimeSrc.toMSecsSinceEpoch() / 1000);
-            srcUds.insert(UDSEntry::UDS_SIZE, sizeSrc);
+            srcUds.fastInsert(UDSEntry::UDS_NAME, d->src.fileName());
+            srcUds.fastInsert(UDSEntry::UDS_MODIFICATION_TIME, mtimeSrc.toMSecsSinceEpoch() / 1000);
+            srcUds.fastInsert(UDSEntry::UDS_CREATION_TIME, ctimeSrc.toMSecsSinceEpoch() / 1000);
+            srcUds.fastInsert(UDSEntry::UDS_SIZE, sizeSrc);
 
             d->srcItem = KFileItem(srcUds, d->src);
         }
@@ -233,10 +232,10 @@ RenameDialog::RenameDialog(QWidget *parent, const QString &_caption,
         } else {
             UDSEntry destUds;
 
-            destUds.insert(UDSEntry::UDS_NAME, d->dest.fileName());
-            destUds.insert(UDSEntry::UDS_MODIFICATION_TIME, mtimeDest.toMSecsSinceEpoch() / 1000);
-            destUds.insert(UDSEntry::UDS_CREATION_TIME, ctimeDest.toMSecsSinceEpoch() / 1000);
-            destUds.insert(UDSEntry::UDS_SIZE, sizeDest);
+            destUds.fastInsert(UDSEntry::UDS_NAME, d->dest.fileName());
+            destUds.fastInsert(UDSEntry::UDS_MODIFICATION_TIME, mtimeDest.toMSecsSinceEpoch() / 1000);
+            destUds.fastInsert(UDSEntry::UDS_CREATION_TIME, ctimeDest.toMSecsSinceEpoch() / 1000);
+            destUds.fastInsert(UDSEntry::UDS_SIZE, sizeDest);
 
             d->destItem = KFileItem(destUds, d->dest);
         }
@@ -257,10 +256,10 @@ RenameDialog::RenameDialog(QWidget *parent, const QString &_caption,
         d->m_srcArea = createContainerLayout(parent, d->srcItem, d->m_srcPreview);
         d->m_destArea = createContainerLayout(parent, d->destItem, d->m_destPreview);
 
-        connect(d->m_srcArea->verticalScrollBar(), SIGNAL(valueChanged(int)), d->m_destArea->verticalScrollBar(), SLOT(setValue(int)));
-        connect(d->m_destArea->verticalScrollBar(), SIGNAL(valueChanged(int)), d->m_srcArea->verticalScrollBar(), SLOT(setValue(int)));
-        connect(d->m_srcArea->horizontalScrollBar(), SIGNAL(valueChanged(int)), d->m_destArea->horizontalScrollBar(), SLOT(setValue(int)));
-        connect(d->m_destArea->horizontalScrollBar(), SIGNAL(valueChanged(int)), d->m_srcArea->horizontalScrollBar(), SLOT(setValue(int)));
+        connect(d->m_srcArea->verticalScrollBar(), &QAbstractSlider::valueChanged, d->m_destArea->verticalScrollBar(), &QAbstractSlider::setValue);
+        connect(d->m_destArea->verticalScrollBar(), &QAbstractSlider::valueChanged, d->m_srcArea->verticalScrollBar(), &QAbstractSlider::setValue);
+        connect(d->m_srcArea->horizontalScrollBar(), &QAbstractSlider::valueChanged, d->m_destArea->horizontalScrollBar(), &QAbstractSlider::setValue);
+        connect(d->m_destArea->horizontalScrollBar(), &QAbstractSlider::valueChanged, d->m_srcArea->horizontalScrollBar(), &QAbstractSlider::setValue);
 
         // create layout
         QGridLayout *gridLayout = new QGridLayout();
@@ -321,8 +320,8 @@ RenameDialog::RenameDialog(QWidget *parent, const QString &_caption,
         pLayout->addWidget(lb);
     }
 
-    if ((_options != RenameDialog_OverwriteItself) && (_options != RenameDialog_NoRename)) {
-        if (_options == RenameDialog_Overwrite) {
+    if (!(_options & RenameDialog_OverwriteItself) && !(_options & RenameDialog_NoRename)) {
+        if (_options & RenameDialog_Overwrite) {
             pLayout->addSpacing(15);    // spacer
         }
 
@@ -340,8 +339,8 @@ RenameDialog::RenameDialog(QWidget *parent, const QString &_caption,
         const QString fileName = d->dest.fileName();
         d->setRenameBoxText(KIO::decodeFileName(fileName));
 
-        connect(d->m_pLineEdit, SIGNAL(textChanged(QString)),
-                SLOT(enableRenameButton(QString)));
+        connect(d->m_pLineEdit, &QLineEdit::textChanged,
+                this, &RenameDialog::enableRenameButton);
 
         d->m_pLineEdit->setFocus();
     } else {
@@ -433,7 +432,7 @@ QUrl RenameDialog::newDestUrl()
 QUrl RenameDialog::autoDestUrl() const
 {
     const QUrl destDirectory = d->dest.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash);
-    const QString newName = KIO::suggestName(destDirectory, d->dest.fileName());
+    const QString newName = KFileUtils::suggestName(destDirectory, d->dest.fileName());
     QUrl newDest(destDirectory);
     newDest.setPath(concatPaths(newDest.path(), newName));
     return newDest;
@@ -465,12 +464,10 @@ void RenameDialog::renamePressed()
     }
 }
 
-#ifndef KIOWIDGETS_NO_DEPRECATED
 QString RenameDialog::suggestName(const QUrl &baseURL, const QString &oldName)
 {
-    return KIO::suggestName(baseURL, oldName);
+    return KFileUtils::suggestName(baseURL, oldName);
 }
-#endif
 
 // Propose button clicked
 void RenameDialog::suggestNewNamePressed()
@@ -481,7 +478,7 @@ void RenameDialog::suggestNewNamePressed()
     }
 
     QUrl destDirectory = d->dest.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash);
-    d->setRenameBoxText(KIO::suggestName(destDirectory, d->m_pLineEdit->text()));
+    d->setRenameBoxText(KFileUtils::suggestName(destDirectory, d->m_pLineEdit->text()));
 }
 
 void RenameDialog::skipPressed()
@@ -623,14 +620,14 @@ void RenameDialog::resizePanels()
                                QSize(d->m_destPreview->width() * qreal(0.9), d->m_destPreview->height()));
     destJob->setScaleType(KIO::PreviewJob::Unscaled);
 
-    connect(srcJob, SIGNAL(gotPreview(KFileItem,QPixmap)),
-            this, SLOT(showSrcPreview(KFileItem,QPixmap)));
-    connect(destJob, SIGNAL(gotPreview(KFileItem,QPixmap)),
-            this, SLOT(showDestPreview(KFileItem,QPixmap)));
-    connect(srcJob, SIGNAL(failed(KFileItem)),
-            this, SLOT(showSrcIcon(KFileItem)));
-    connect(destJob, SIGNAL(failed(KFileItem)),
-            this, SLOT(showDestIcon(KFileItem)));
+    connect(srcJob, &PreviewJob::gotPreview,
+            this, &RenameDialog::showSrcPreview);
+    connect(destJob, &PreviewJob::gotPreview,
+            this, &RenameDialog::showDestPreview);
+    connect(srcJob, &PreviewJob::failed,
+            this, &RenameDialog::showSrcIcon);
+    connect(destJob, &PreviewJob::failed,
+            this, &RenameDialog::showDestIcon);
 }
 
 QScrollArea *RenameDialog::createContainerLayout(QWidget *parent, const KFileItem &item, QLabel *preview)
